@@ -1,10 +1,8 @@
-/*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
+/*  GPU.cpp
+	Copyright (C) 2006 yopyop
     Copyright (C) 2006-2007 Theo Berkau
     Copyright (C) 2007 shash
-	Copyright (C) 2008-2009 DeSmuME team
+	Copyright (C) 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -38,12 +36,12 @@
 #include "NDSSystem.h"
 #include "readwrite.h"
 
-//#undef FORCEINLINE
-//#define FORCEINLINE
-//#define SSE2_NOINTRIN
-
-//compilation speed hack (cuts time exactly in half by cutting out permutations)
-//#define DISABLE_MOSAIC
+#ifdef FASTBUILD
+	#undef FORCEINLINE
+	#define FORCEINLINE
+	//compilation speed hack (cuts time exactly in half by cutting out permutations)
+	#define DISABLE_MOSAIC
+#endif
 
 extern BOOL click;
 NDS_Screen MainScreen;
@@ -208,7 +206,6 @@ void GPU_Reset(GPU *g, u8 l)
 	g->core = l;
 	g->BGSize[0][0] = g->BGSize[1][0] = g->BGSize[2][0] = g->BGSize[3][0] = 256;
 	g->BGSize[0][1] = g->BGSize[1][1] = g->BGSize[2][1] = g->BGSize[3][1] = 256;
-	g->dispOBJ = g->dispBG[0] = g->dispBG[1] = g->dispBG[2] = g->dispBG[3] = TRUE;
 
 	g->spriteRenderMode = GPU::SPRITE_1D;
 
@@ -249,11 +246,11 @@ static void GPU_resortBGs(GPU *gpu)
 #define OP ^ !
 // if we untick boxes, layers become invisible
 //#define OP &&
-	gpu->LayersEnable[0] = gpu->dispBG[0] OP(cnt->BG0_Enable/* && !(cnt->BG0_3D && (gpu->core==0))*/);
-	gpu->LayersEnable[1] = gpu->dispBG[1] OP(cnt->BG1_Enable);
-	gpu->LayersEnable[2] = gpu->dispBG[2] OP(cnt->BG2_Enable);
-	gpu->LayersEnable[3] = gpu->dispBG[3] OP(cnt->BG3_Enable);
-	gpu->LayersEnable[4] = gpu->dispOBJ   OP(cnt->OBJ_Enable);
+	gpu->LayersEnable[0] = CommonSettings.dispLayers[gpu->core][0] OP(cnt->BG0_Enable/* && !(cnt->BG0_3D && (gpu->core==0))*/);
+	gpu->LayersEnable[1] = CommonSettings.dispLayers[gpu->core][1] OP(cnt->BG1_Enable);
+	gpu->LayersEnable[2] = CommonSettings.dispLayers[gpu->core][2] OP(cnt->BG2_Enable);
+	gpu->LayersEnable[3] = CommonSettings.dispLayers[gpu->core][3] OP(cnt->BG3_Enable);
+	gpu->LayersEnable[4] = CommonSettings.dispLayers[gpu->core][4] OP(cnt->OBJ_Enable);
 
 	// KISS ! lower priority first, if same then lower num
 	for (i=0;i<NB_PRIORITIES;i++) {
@@ -480,15 +477,12 @@ void GPU_setBGProp(GPU * gpu, u16 num, u16 p)
 
 void GPU_remove(GPU * gpu, u8 num)
 {
-	if (num == 4)	gpu->dispOBJ = 0;
-	else		gpu->dispBG[num] = 0;
+	CommonSettings.dispLayers[gpu->core][num] = false;
 	GPU_resortBGs(gpu);
 }
 void GPU_addBack(GPU * gpu, u8 num)
 {
-	//REG_DISPx_pack_test(gpu);
-	if (num == 4)	gpu->dispOBJ = 1;
-	else		gpu->dispBG[num] = 1;
+	CommonSettings.dispLayers[gpu->core][num] = true;
 	GPU_resortBGs(gpu);
 }
 
@@ -577,7 +571,9 @@ FORCEINLINE FASTCALL void GPU::_master_setFinal3dColor(int dstX, int srcX)
 	u8* dst = currDst;
 	u16 final;
 
-	bool windowEffect = true;
+	bool windowEffect = blend1; //bomberman land touch dialogbox will fail without setting to blend1
+	
+	//TODO - should we do an alpha==0 -> bail out entirely check here?
 
 	if(WINDOW)
 	{
@@ -597,7 +593,7 @@ FORCEINLINE FASTCALL void GPU::_master_setFinal3dColor(int dstX, int srcX)
 			//if the layer underneath is a blend bottom layer, then 3d always alpha blends with it
 			COLOR c2, cfinal;
 
-			c2.val = T2ReadWord(dst, passing);
+			c2.val = HostReadWord(dst, passing);
 
 			cfinal.bits.red = ((red * alpha) + ((c2.bits.red<<1) * (32 - alpha)))>>6;
 			cfinal.bits.green = ((green * alpha) + ((c2.bits.green<<1) * (32 - alpha)))>>6;
@@ -607,7 +603,7 @@ FORCEINLINE FASTCALL void GPU::_master_setFinal3dColor(int dstX, int srcX)
 		}
 		else final = R6G6B6TORGB15(red,green,blue);
 	}
-	else 
+	else
 	{
 		final = R6G6B6TORGB15(red,green,blue);
 		//perform the special effect
@@ -621,7 +617,7 @@ FORCEINLINE FASTCALL void GPU::_master_setFinal3dColor(int dstX, int srcX)
 			}
 	}
 
-	T2WriteWord(dst, passing, (final | 0x8000));
+	HostWriteWord(dst, passing, (final | 0x8000));
 	bgPixels[x] = 0;
 }
 
@@ -657,7 +653,7 @@ FORCEINLINE FASTCALL bool GPU::_master_setFinalBGColor(u16 &color, const u32 x)
 
 	//perform the special effect
 	switch(FUNC) {
-		case Blend: if(blend2[bg_under]) color = blend(color,T2ReadWord(currDst, x<<1)); break;
+		case Blend: if(blend2[bg_under]) color = blend(color,HostReadWord(currDst, x<<1)); break;
 		case Increase: color = currentFadeInColors[color]; break;
 		case Decrease: color = currentFadeOutColors[color]; break;
 		case None: break;
@@ -700,7 +696,7 @@ static FORCEINLINE void _master_setFinalOBJColor(GPU *gpu, u8 *dst, u16 color, u
 
 	if(allowBlend)
 	{
-		u16 backColor = T2ReadWord(dst,x<<1);
+		u16 backColor = HostReadWord(dst,x<<1);
 		//this hasn't been tested: this blending occurs without regard to the color effect,
 		//but rather purely from the sprite's alpha
 		if(type == GPU_OBJ_MODE_Bitmap)
@@ -709,7 +705,7 @@ static FORCEINLINE void _master_setFinalOBJColor(GPU *gpu, u8 *dst, u16 color, u
 			color = gpu->blend(color,backColor);
 	}
 
-	T2WriteWord(dst, x<<1, (color | 0x8000));
+	HostWriteWord(dst, x<<1, (color | 0x8000));
 	gpu->bgPixels[x] = 4;	
 }
 
@@ -740,7 +736,7 @@ FORCEINLINE void GPU::setFinalColorBG(u16 color, const u32 x)
 
 	if(BACKDROP || draw) //backdrop must always be drawn
 	{
-		T2WriteWord(currDst, x<<1, color | 0x8000);
+		HostWriteWord(currDst, x<<1, color | 0x8000);
 		if(!BACKDROP) bgPixels[x] = currBgNum; //lets do this in the backdrop drawing loop, should be faster
 	}
 }
@@ -868,10 +864,9 @@ template<bool MOSAIC> void lineLarge8bpp(GPU * gpu)
 		return;
 	}
 
-	BGxOFS * ofs = &gpu->dispx_st->dispx_BGxOFS[gpu->currBgNum];
 	u8 num = gpu->currBgNum;
-	u16 XBG = T1ReadWord((u8 *)&ofs->BGxHOFS, 0);
-	u16 YBG = gpu->currLine + T1ReadWord((u8 *)&ofs->BGxVOFS, 0);
+	u16 XBG = gpu->getHOFS(gpu->currBgNum);
+	u16 YBG = gpu->currLine + gpu->getVOFS(gpu->currBgNum);
 	u16 lg     = gpu->BGSize[num][0];
 	u16 ht     = gpu->BGSize[num][1];
 	u16 wmask  = (lg-1);
@@ -881,7 +876,7 @@ template<bool MOSAIC> void lineLarge8bpp(GPU * gpu)
 	//TODO - handle wrapping / out of bounds correctly from rot_scale_op?
 
 	u32 tmp_map = gpu->BG_bmp_large_ram[num] + lg * YBG;
-	u8* map = MMU_gpu_map(tmp_map);
+	u8* map = (u8 *)MMU_gpu_map(tmp_map);
 
 	u8* pal = MMU.ARM9_VMEM + gpu->core * ADDRESS_STEP_1KB;
 
@@ -1003,15 +998,16 @@ template<bool MOSAIC> INLINE void renderline_textBG(GPU * gpu, u16 XBG, u16 YBG,
 	}
 
 	yoff = ((YBG&7)<<3);
-
+	u8* tilePal;
 	xfin = 8 - (xoff&7);
+	u32 extPalMask = -dispCnt->ExBGxPalette_Enable;
 	for(x = 0; x < LG; xfin = std::min<u16>(x+8, LG))
 	{
 		tmp = (xoff & (lg-1))>>3;
 		mapinfo = map + (tmp & 31) * 2;
 		if(tmp > 31) mapinfo += 32*32*2;
 		tileentry.val = T1ReadWord(MMU_gpu_map(mapinfo), 0);
-
+		tilePal = pal + ((tileentry.bits.Palette<<9)&extPalMask);
 		line = (u8*)MMU_gpu_map(tile + (tileentry.bits.TileNum*0x40) + ((tileentry.bits.VFlip) ? (7*8)-yoff : yoff));
 
 		if(tileentry.bits.HFlip)
@@ -1024,10 +1020,7 @@ template<bool MOSAIC> INLINE void renderline_textBG(GPU * gpu, u16 XBG, u16 YBG,
 		}
 		for(; x < xfin; )
 		{
-			if(dispCnt->ExBGxPalette_Enable)
-				color = T1ReadWord(pal, ((*line) + (tileentry.bits.Palette<<8)) << 1);
-			else
-				color = T1ReadWord(pal, (*line) << 1);
+			color = T1ReadWord(tilePal, (*line) << 1);
 
 			gpu->__setFinalColorBck<MOSAIC,false>(color,x,*line);
 			
@@ -1214,10 +1207,6 @@ static void lineNull(GPU * gpu)
 
 template<bool MOSAIC> void lineText(GPU * gpu)
 {
-	BGxOFS * ofs = &gpu->dispx_st->dispx_BGxOFS[gpu->currBgNum];
-
-
-
 	if(gpu->debug)
 	{
 		const s32 wh = gpu->BGSize[gpu->currBgNum][0];
@@ -1225,8 +1214,8 @@ template<bool MOSAIC> void lineText(GPU * gpu)
 	}
 	else
 	{
-		const u16 vofs = T1ReadWord((u8 *)&ofs->BGxVOFS,0);
-		const u16 hofs = T1ReadWord((u8 *)&ofs->BGxHOFS,0);
+		const u16 vofs = gpu->getVOFS(gpu->currBgNum);
+		const u16 hofs = gpu->getHOFS(gpu->currBgNum);
 		renderline_textBG<MOSAIC>(gpu, hofs, gpu->currLine + vofs, 256);
 	}
 }
@@ -1307,17 +1296,13 @@ INLINE void render_sprite_BMP (GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u16 * s
 		color = LE_TO_LOCAL_16(src[x]);
 
 		// alpha bit = invisible
-		if ((color&0x8000)&&(prio<=prioTab[sprX]))
+		if ((color&0x8000)&&(prio<prioTab[sprX]))
 		{
-			/* if we don't draw, do not set prio, or else */
-		//	if (gpu->setFinalColorSpr(gpu, sprX << 1,4,dst, color, sprX))
-			{
-				T2WriteWord(dst, (sprX<<1), color);
-				dst_alpha[sprX] = alpha;
-				typeTab[sprX] = 3;
-				prioTab[sprX] = prio;
-				gpu->sprNum[sprX] = spriteNum;
-			}
+			HostWriteWord(dst, (sprX<<1), color);
+			dst_alpha[sprX] = alpha;
+			typeTab[sprX] = 3;
+			prioTab[sprX] = prio;
+			gpu->sprNum[sprX] = spriteNum;
 		}
 	}
 }
@@ -1335,17 +1320,13 @@ INLINE void render_sprite_256 (	GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u8 * s
 		color = LE_TO_LOCAL_16(pal[palette_entry]);
 
 		// palette entry = 0 means backdrop
-		if ((palette_entry>0)&&(prio<=prioTab[sprX]))
+		if ((palette_entry>0)&&(prio<prioTab[sprX]))
 		{
-			/* if we don't draw, do not set prio, or else */
-			//if (gpu->setFinalColorSpr(gpu, sprX << 1,4,dst, color, sprX))
-			{
-				T2WriteWord(dst, (sprX<<1), color);
-				dst_alpha[sprX] = 16;
-				typeTab[sprX] = (alpha ? 1 : 0);
-				prioTab[sprX] = prio;
-				gpu->sprNum[sprX] = spriteNum;
-			}
+			HostWriteWord(dst, (sprX<<1), color);
+			dst_alpha[sprX] = 16;
+			typeTab[sprX] = (alpha ? 1 : 0);
+			prioTab[sprX] = prio;
+			gpu->sprNum[sprX] = spriteNum;
 		}
 	}
 }
@@ -1366,16 +1347,12 @@ INLINE void render_sprite_16 (	GPU * gpu, u16 l, u8 * dst, u8 * src, u16 * pal,
 		color = LE_TO_LOCAL_16(pal[palette_entry]);
 
 		// palette entry = 0 means backdrop
-		if ((palette_entry>0)&&(prio<=prioTab[sprX]))
+		if ((palette_entry>0)&&(prio<prioTab[sprX]))
 		{
-			/* if we don't draw, do not set prio, or else */
-			//if (gpu->setFinalColorSpr(gpu, sprX << 1,4,dst, color, sprX ))
-			{
-				T2WriteWord(dst, (sprX<<1), color);
-				dst_alpha[sprX] = 16;
-				typeTab[sprX] = (alpha ? 1 : 0);
-				prioTab[sprX] = prio;
-			}
+			HostWriteWord(dst, (sprX<<1), color);
+			dst_alpha[sprX] = 16;
+			typeTab[sprX] = (alpha ? 1 : 0);
+			prioTab[sprX] = prio;
 		}
 	}
 }
@@ -1494,26 +1471,41 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 	u16 l = currLine;
 	GPU *gpu = this;
 
+	int cost = 0;
+
 	struct _DISPCNT * dispCnt = &(gpu->dispx_st)->dispx_DISPCNT.bits;
-	_OAM_ * spriteInfo = (_OAM_ *)(gpu->oam + (nbShow-1));// + 127;
+	_OAM_ * spriteInfo = (_OAM_ *)(gpu->oam);
 	u8 block = gpu->sprBoundary;
 	u8 i;
 
-#ifdef WORDS_BIGENDIAN
-	*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) >> 1) | *(((u16*)spriteInfo)+1) << 15;
-	*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) >> 2) | *(((u16*)spriteInfo)+2) << 14;
-#endif
+	//what the hell? why is all this here? the #ifdefs in the bitfields definition should take care of this.
+	//this needs to be fixed anyway since i changed the sprite render order
+	//better yet, just dont do it this way at all. _OAM_ is so small, why not just copy it and then twiddle it?
 
-	for(i = 0; i<nbShow; ++i, --spriteInfo
-#ifdef WORDS_BIGENDIAN    
-	,*(((u16*)(spriteInfo+1))+1) = (*(((u16*)(spriteInfo+1))+1) << 1) | *(((u16*)(spriteInfo+1))+1) >> 15
-	,*(((u16*)(spriteInfo+1))+2) = (*(((u16*)(spriteInfo+1))+2) << 2) | *(((u16*)(spriteInfo+1))+2) >> 14
-	,*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) >> 1) | *(((u16*)spriteInfo)+1) << 15
-	,*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) >> 2) | *(((u16*)spriteInfo)+2) << 14
-#endif
+//#ifdef WORDS_BIGENDIAN
+//	*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) >> 1) | *(((u16*)spriteInfo)+1) << 15;
+//	*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) >> 2) | *(((u16*)spriteInfo)+2) << 14;
+//#endif
+
+	for(i = 0; i<nbShow; ++i, ++spriteInfo
+//#ifdef WORDS_BIGENDIAN    
+//	,*(((u16*)(spriteInfo+1))+1) = (*(((u16*)(spriteInfo+1))+1) << 1) | *(((u16*)(spriteInfo+1))+1) >> 15
+//	,*(((u16*)(spriteInfo+1))+2) = (*(((u16*)(spriteInfo+1))+2) << 2) | *(((u16*)(spriteInfo+1))+2) >> 14
+//	,*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) >> 1) | *(((u16*)spriteInfo)+1) << 15
+//	,*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) >> 2) | *(((u16*)spriteInfo)+2) << 14
+//#endif
 	)     
 	{
 		//for each sprite:
+		if(cost>=2130)
+		{
+			//out of sprite rendering time
+			//printf("sprite overflow!\n");
+			//return;		
+		}
+
+		//do we incur a cost if a sprite is disabled?? we guess so.
+		cost += 2;
 
 		size sprSize;
 		s32 sprX, sprY, x, y, lg;
@@ -1562,6 +1554,8 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 				(sprX==256) || (sprX+fieldX<=0))
 				continue;
 
+			cost += sprSize.x*2 + 10;
+
 			y = l - sprY;
 
 			// Get which four parameter block is assigned to this sprite
@@ -1628,9 +1622,9 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 
 						colour = src[offset];
 
-						if (colour && (prioTab[sprX]>=prio))
+						if (colour && (prio<prioTab[sprX]))
 						{ 
-							T2WriteWord(dst, (sprX<<1), T2ReadWord(pal, (colour<<1)));
+							HostWriteWord(dst, (sprX<<1), HostReadWord(pal, (colour<<1)));
 							dst_alpha[sprX] = 16;
 							typeTab[sprX] = spriteInfo->Mode;
 							prioTab[sprX] = prio;
@@ -1671,9 +1665,9 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 
 						colour = T1ReadWord (src, offset<<1);
 
-						if((colour&0x8000) && (prioTab[sprX]>=prio))
+						if((colour&0x8000) && (prio<prioTab[sprX]))
 						{
-							T2WriteWord(dst, (sprX<<1), colour);
+							HostWriteWord(dst, (sprX<<1), colour);
 							dst_alpha[sprX] = spriteInfo->PaletteIndex;
 							typeTab[sprX] = spriteInfo->Mode;
 							prioTab[sprX] = prio;
@@ -1721,12 +1715,17 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 						if (auxX&1)	colour >>= 4;
 						else		colour &= 0xF;
 
-						if(colour && (prioTab[sprX]>=prio))
+						if(colour && (prio<prioTab[sprX]))
 						{
-							T2WriteWord(dst, (sprX<<1), LE_TO_LOCAL_16(T2ReadWord(pal, colour << 1)));
-							dst_alpha[sprX] = 16;
-							typeTab[sprX] = spriteInfo->Mode;
-							prioTab[sprX] = prio;
+							if(spriteInfo->Mode==2)
+								sprWin[sprX] = 1;
+							else
+							{
+								HostWriteWord(dst, (sprX<<1), LE_TO_LOCAL_16(HostReadWord(pal, colour << 1)));
+								dst_alpha[sprX] = 16;
+								typeTab[sprX] = spriteInfo->Mode;
+								prioTab[sprX] = prio;
+							}
 						}
 					}
 
@@ -1742,11 +1741,12 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 		else
 		{
 			u16 * pal;
-
 	
 			if (!compute_sprite_vars(spriteInfo, l, sprSize, sprX, sprY, x, y, lg, xdir))
 				continue;
 
+			cost += sprSize.x;
+
 			if (spriteInfo->Mode == 2)
 			{
 				if(MODE == SPRITE_2D)
@@ -1815,10 +1815,11 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 		}
 	}
 
-#ifdef WORDS_BIGENDIAN
-	*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) << 1) | *(((u16*)spriteInfo)+1) >> 15;
-	*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) << 2) | *(((u16*)spriteInfo)+2) >> 14;
-#endif
+//#ifdef WORDS_BIGENDIAN
+//	*(((u16*)spriteInfo)+1) = (*(((u16*)spriteInfo)+1) << 1) | *(((u16*)spriteInfo)+1) >> 15;
+//	*(((u16*)spriteInfo)+2) = (*(((u16*)spriteInfo)+2) << 2) | *(((u16*)spriteInfo)+2) >> 14;
+//#endif
+
 }
 
 
@@ -2057,7 +2058,7 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 	{
 		//n.b. - this is clearing the sprite line buffer to the background color,
 		//but it has been changed to write u32 instead of u16 for a little speedup
-		for(int i = 0; i< 128; ++i) T2WriteLong(spr, i << 2, backdrop_color | (backdrop_color<<16));
+		for(int i = 0; i< 128; ++i) HostWriteTwoWords(spr, i << 2, backdrop_color | (backdrop_color<<16));
 		//zero 06-may-09: I properly supported window color effects for backdrop, but I am not sure
 		//how it interacts with this. I wish we knew why we needed this
 		
@@ -2109,8 +2110,7 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 						{
 							gpu->currBgNum = 0;
 
-							BGxOFS *bgofs = &gpu->dispx_st->dispx_BGxOFS[i16];
-							u16 hofs = (T1ReadWord((u8*)&bgofs->BGxHOFS, 0) & 0x1FF);
+							const u16 hofs = gpu->getHOFS(i16);
 
 							gfx3d_GetLineData(l, &gpu->_3dColorLine);
 							u8* colorLine = gpu->_3dColorLine;
@@ -2133,10 +2133,8 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 					//useful for debugging individual layers
 					//if(gpu->core == 1 || i16 != 2) continue;
 
-					if(gpu->core == 0 || i16 == 2)
-					{
-						int zzz=9;
-					}
+
+
 
 #ifndef DISABLE_MOSAIC
 					if(gpu->curr_mosaic_enabled)
@@ -2157,7 +2155,7 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 			for (int i=0; i < item->nbPixelsX; i++)
 			{
 				i16=item->PixelsX[i];
-				setFinalColorSpr(gpu, gpu->currDst, T2ReadWord(spr, (i16<<1)), sprAlpha[i16], sprType[i16], i16);
+				setFinalColorSpr(gpu, gpu->currDst, HostReadWord(spr, (i16<<1)), sprAlpha[i16], sprType[i16], i16);
 			}
 		}
 	}
@@ -2170,11 +2168,11 @@ template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 	switch(gpu->dispCapCnt.capx) { \
 		case DISPCAPCNT::_128: \
 			for (int i = 0; i < 128; i++)  \
-				T2WriteWord(DST, i << 1, T2ReadWord(SRC, i << 1) | (1<<15)); \
+				HostWriteWord(DST, i << 1, HostReadWord(SRC, i << 1) | (1<<15)); \
 			break; \
 		case DISPCAPCNT::_256: \
 			for (int i = 0; i < 256; i++)  \
-				T2WriteWord(DST, i << 1, T2ReadWord(SRC, i << 1) | (1<<15)); \
+				HostWriteWord(DST, i << 1, HostReadWord(SRC, i << 1) | (1<<15)); \
 			break; \
 			default: assert(false); \
 		}
@@ -2332,7 +2330,7 @@ template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 							g = std::min((u16)31,g);
 							b = std::min((u16)31,b);
 
-							T2WriteWord(cap_dst, i << 1, a | (b << 10) | (g << 5) | r);
+							HostWriteWord(cap_dst, i << 1, a | (b << 10) | (g << 5) | r);
 						}
 					}
 				break;
@@ -2591,7 +2589,7 @@ void GPU_RenderLine(NDS_Screen * screen, u16 l, bool skip)
 				u8 * dst =  GPU_screen + (screen->offset + l) * 512;
 
 				for (int i=0; i<256; i++)
-					T2WriteWord(dst, i << 1, 0x7FFF);
+					HostWriteWord(dst, i << 1, 0x7FFF);
 			}
 			break;
 
@@ -2801,51 +2799,3 @@ void gpu_SetRotateScreen(u16 angle)
 {
 	gpu_angle = angle;
 }
-
-//here is an old bg mosaic with some old code commented out. I am going to leave it here for a while to look at it
-//sometimes in case I find a problem with the mosaic.
-//static void __setFinalColorBck(GPU *gpu, u32 passing, u8 bgnum, u8 *dst, u16 color, u16 x, bool opaque)
-//{
-//	struct _BGxCNT *bgCnt = &(gpu->dispx_st)->dispx_BGxCNT[bgnum].bits;
-//	bool enabled = bgCnt->Mosaic_Enable;
-//
-////	if(!opaque) color = 0xFFFF;
-////	else color &= 0x7FFF;
-//	if(!opaque)
-//		return;
-//
-//	//mosaic test hacks
-//	enabled = true;
-//
-//	//due to this early out, we will get incorrect behavior in cases where 
-//	//we enable mosaic in the middle of a frame. this is deemed unlikely.
-//	if(enabled)
-//	{
-//		u8 y = gpu->currLine;
-//
-//		//I intend to cache all this at the beginning of line rendering
-//		
-//		u16 mosaic_control = T1ReadWord((u8 *)&gpu->dispx_st->dispx_MISC.MOSAIC, 0);
-//		u8 mw = (mosaic_control & 0xF);
-//		u8 mh = ((mosaic_control>>4) & 0xF); 
-//
-//		//mosaic test hacks
-//		mw = 3;
-//		mh = 3;
-//
-//		MosaicLookup::TableEntry &te_x = mosaicLookup.table[mw][x];
-//		MosaicLookup::TableEntry &te_y = mosaicLookup.table[mh][y];
-//
-//		//int x_int;
-//		//if(enabled) 
-//			int x_int = te_x.trunc;
-//		//else x_int = x;
-//
-//		if(te_x.begin && te_y.begin) {}
-//		else color = gpu->MosaicColors.bg[bgnum][x_int];
-//		gpu->MosaicColors.bg[bgnum][x] = color;
-//	}
-//
-////	if(color != 0xFFFF)
-//		gpu->setFinalColorBck(gpu,0,bgnum,dst,color,x);
-//}
