@@ -691,9 +691,6 @@ struct GPU
 
 	//FIFO	fifo;
 
-	BOOL dispBG[4];
-	BOOL dispOBJ;
-
 	u8 bgPrio[5];
 
 	BOOL bg0HasHighestPrio;
@@ -844,6 +841,9 @@ struct GPU
 		updateBLDALPHA();
 	}
 
+	u32 getHOFS(int bg) { return T1ReadWord(&dispx_st->dispx_BGxOFS[bg].BGxHOFS,0) & 0x1FF; }
+	u32 getVOFS(int bg) { return T1ReadWord(&dispx_st->dispx_BGxOFS[bg].BGxVOFS,0) & 0x1FF; }
+
 	typedef u8 TBlendTable[32][32];
 	TBlendTable *blendTable;
 
@@ -986,10 +986,11 @@ void SetupFinalPixelBlitter (GPU *gpu);
 
 
 
-#define GPU_setBLDY_EVY(gpu, val) {gpu->BLDY_EVY = (val&0x1f) > 16 ? 16 : (val&0x1f);}
+#define GPU_setBLDY_EVY(gpu, val) {gpu->BLDY_EVY = ((val)&0x1f) > 16 ? 16 : ((val)&0x1f);}
 
-#define GPU_setBGxHOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxHOFS = (val & 0x1F)
-#define GPU_setBGxVOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxVOFS = (val & 0x1F)
+//these arent needed right now since the values get poked into memory via default mmu handling and dispx_st
+//#define GPU_setBGxHOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxHOFS = ((val) & 0x1FF)
+//#define GPU_setBGxVOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxVOFS = ((val) & 0x1FF)
 
 // render
 void gpu_UpdateRender();
