@@ -1,8 +1,5 @@
 /*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
-    Copyright (C) 2006-2008 DeSmuME team
+    Copyright (C) 2006-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -34,6 +31,7 @@
 
 #include "aggdraw.h"
 #include "movie.h"
+#include "rtc.h"
 #include "NDSSystem.h"
 #include "mic.h"
 #include "saves.h"
@@ -163,6 +161,11 @@ void HudStruct::reset()
 	Microphone.xsize=20;
 	Microphone.ysize=10;
 
+	RTCDisplay.x=0;
+	RTCDisplay.y=105;
+	RTCDisplay.xsize=220;
+	RTCDisplay.ysize=10;
+
 	SavestateSlots.x = 8;
 	SavestateSlots.y = 160;
 	SavestateSlots.xsize = 240;
@@ -372,7 +375,7 @@ static void TextualInputDisplay() {
 	// touch pad
 	{
 		char str [32];
-		bool gameTouchOn = nds.isTouch;
+		BOOL gameTouchOn = nds.isTouch;
 		int gameTouchX = nds.touchX >> 4;
 		int gameTouchY = nds.touchY >> 4;
 		bool physicalTouchOn = NDS_getRawUserInput().touch.isTouch;
@@ -550,13 +553,20 @@ void DrawHUD()
 		drawPad(Hud.GraphicalInputDisplay.x, Hud.GraphicalInputDisplay.y, 2.5);
 	}
 
-	#ifdef WIN32
+	#if defined(WIN32) && !defined(WXPORT)
 	if (CommonSettings.hud.ShowMicrophone) 
 	{
 		osd->addFixed(Hud.Microphone.x, Hud.Microphone.y, "%03d [%07d]",MicDisplay, Hud.cpuloopIterationCount);
 	}
 	#endif
 
+	if (CommonSettings.hud.ShowRTC) 
+	{
+		struct tm tm = rtcGetTime();
+		const char *wday[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
+		osd->addFixed(Hud.RTCDisplay.x, Hud.RTCDisplay.y, "%04d-%02d-%02d %s %02d:%02d:%02d", 1900 + tm.tm_year, 1 + tm.tm_mon, tm.tm_mday, wday[tm.tm_wday%7], tm.tm_hour, tm.tm_min, tm.tm_sec);
+	}
+
 	DrawStateSlots();
 }
 
