@@ -1,9 +1,6 @@
 /*	Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com 
-
 	Copyright (C) 2007 shash
-	Copyright (C) 2007-2009 DeSmuME team
+	Copyright (C) 2007-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -30,6 +27,9 @@
 #include "registers.h"
 #include "mc.h"
 #include "bits.h"
+#include "readwrite.h"
+#include "debug.h"
+
 #ifdef HAVE_LUA
 #include "lua-engine.h"
 #endif
@@ -77,7 +77,9 @@ enum EDMADestinationUpdate
 	EDMADestinationUpdate_IncrementReload = 3,
 };
 
-
+//TODO
+//n.b. this may be a bad idea, for complex registers like the dma control register.
+//we need to know exactly what part was written to, instead of assuming all 32bits were written.
 class TRegister_32
 {
 public:
@@ -87,7 +89,6 @@ public:
 		if(size==32) write32(val);
 		else {
 			const u32 offset = adr&3;
-			const u32 baseaddr = adr&~offset;
 			if(size==8) {
 				printf("WARNING! 8BIT DMA ACCESS\n"); 
 				u32 mask = 0xFF<<(offset<<3);
@@ -105,7 +106,6 @@ public:
 		if(size==32) return read32();
 		else {
 			const u32 offset = adr&3;
-			const u32 baseaddr = adr&~offset;
 			if(size==8) { printf("WARNING! 8BIT DMA ACCESS\n"); return (read32()>>(offset<<3))&0xFF; }
 			else return (read32()>>(offset<<3))&0xFFFF;
 		}
@@ -115,11 +115,12 @@ public:
 struct TGXSTAT : public TRegister_32
 {
 	TGXSTAT() {
-		gxfifo_irq = se = tr = tb = 0;
+		gxfifo_irq = se = tr = tb = sb = 0;
 	}
 	u8 tb; //test busy
 	u8 tr; //test result
 	u8 se; //stack error
+	u8 sb; //stack busy
 	u8 gxfifo_irq; //irq configuration
 
 	virtual u32 read32();
@@ -131,6 +132,60 @@ struct TGXSTAT : public TRegister_32
 
 void triggerDma(EDMAMode mode);
 
+class DivController
+{
+public:
+	DivController()
+		: mode(0), busy(0)
+	{}
+	void exec();
+	u8 mode, busy, div0;
+	u16 read16() { return mode|(busy<<15)|(div0<<14); }
+	void write16(u16 val) { 
+		mode = val&3;
+		//todo - do we clear the div0 flag here or is that strictly done by the divider unit?
+	}
+	void savestate(EMUFILE* os)
+	{
+		write8le(&mode,os);
+		write8le(&busy,os);
+		write8le(&div0,os);
+	}
+	bool loadstate(EMUFILE* is, int version)
+	{
+		int ret = 1;
+		ret &= read8le(&mode,is);
+		ret &= read8le(&busy,is);
+		ret &= read8le(&div0,is);
+		return ret==1;
+	}
+};
+
+class SqrtController
+{
+public:
+	SqrtController()
+		: mode(0), busy(0)
+	{}
+	void exec();
+	u8 mode, busy;
+	u16 read16() { return mode|(busy<<15); }
+	void write16(u16 val) { mode = val&1; }
+	void savestate(EMUFILE* os)
+	{
+		write8le(&mode,os);
+		write8le(&busy,os);
+	}
+	bool loadstate(EMUFILE* is, int version)
+	{
+		int ret=1;
+		ret &= read8le(&mode,is);
+		ret &= read8le(&busy,is);
+		return ret==1;
+	}
+};
+
+
 class DmaController
 {
 public:
@@ -142,6 +197,7 @@ public:
 	EDMASourceUpdate sar;
 	EDMADestinationUpdate dar;
 	u32 saddr, daddr;
+	u32 saddr_user, daddr_user;
 	
 	//indicates whether the dma needs to be checked for triggering
 	BOOL check;
@@ -178,13 +234,14 @@ public:
 		//if saddr isnt cleared then rings of fate will trigger copy protection
 		//by inspecting dma3 saddr when it boots
 		saddr(0), daddr(0),
+		saddr_user(0), daddr_user(0),
 		check(FALSE),
 		running(FALSE),
 		paused(FALSE),
 		triggered(FALSE),
 		nextEvent(0),
-		sad(&saddr),
-		dad(&daddr)
+		sad(&saddr_user),
+		dad(&daddr_user)
 	{
 		sad.controller = this;
 		dad.controller = this;
@@ -332,12 +389,10 @@ struct MMU_struct
 	BOOL divRunning;
 	s64 divResult;
 	s64 divMod;
-	u32 divCnt;
 	u64 divCycles;
 
 	BOOL sqrtRunning;
 	u32 sqrtResult;
-	u32 sqrtCnt;
 	u64 sqrtCycles;
 
 	u16 SPI_CNT;
@@ -349,7 +404,7 @@ struct MMU_struct
 
 	u8 powerMan_CntReg;
 	BOOL powerMan_CntRegWritten;
-	u8 powerMan_Reg[4];
+	u8 powerMan_Reg[5];
 
 	memory_chip_t fw;
 
@@ -363,6 +418,8 @@ struct MMU_struct_new
 	BackupDevice backupDevice;
 	DmaController dma[2][4];
 	TGXSTAT gxstat;
+	SqrtController sqrt;
+	DivController div;
 
 	void write_dma(const int proc, const int size, const u32 adr, const u32 val);
 	u32 read_dma(const int proc, const int size, const u32 adr);
@@ -488,11 +545,6 @@ FORCEINLINE void* MMU_gpu_map(u32 vram_addr)
 }
 
 
-enum MMU_ACCESS_TYPE
-{
-	MMU_AT_CODE, MMU_AT_DATA, MMU_AT_GPU, MMU_AT_DMA
-};
-
 template<int PROCNUM, MMU_ACCESS_TYPE AT> u8 _MMU_read08(u32 addr);
 template<int PROCNUM, MMU_ACCESS_TYPE AT> u16 _MMU_read16(u32 addr);
 template<int PROCNUM, MMU_ACCESS_TYPE AT> u32 _MMU_read32(u32 addr);
@@ -533,14 +585,34 @@ inline void SetupMMU(BOOL debugConsole) {
 	_MMU_MAIN_MEM_MASK32 = _MMU_MAIN_MEM_MASK & ~3;
 }
 
-//TODO: at one point some of the early access code included this. consider re-adding it
-  //ARM7 private memory
-  //if ( (adr & 0x0f800000) == 0x03800000) {
-    //T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM7][(adr >> 20) & 0xFF],
-      //         adr & MMU.MMU_MASK[ARMCPU_ARM7][(adr >> 20) & 0xFF]); 
+
+FORCEINLINE void CheckMemoryDebugEvent(const EDEBUG_EVENT event, const MMU_ACCESS_TYPE type, const u32 procnum, const u32 addr, const u32 size, const u32 val)
+{
+	if(CheckDebugEvent(event))
+	{
+		DebugEventData.memAccessType = type;
+		DebugEventData.procnum = procnum;
+		DebugEventData.addr = addr;
+		DebugEventData.size = size;
+		DebugEventData.val = val;
+		HandleDebugEvent(event);
+
+		if(type == MMU_AT_CODE && event == DEBUG_EVENT_READ)
+		{
+			HandleDebugEvent(DEBUG_EVENT_EXECUTE);
+		}
+	}
+}
+
+
+//ALERT!!!!!!!!!!!!!!
+//the following inline functions dont do the 0x0FFFFFFF mask.
+//this may result in some unexpected behavior
 
 FORCEINLINE u8 _MMU_read08(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u32 addr)
 {
+	CheckMemoryDebugEvent(DEBUG_EVENT_READ,AT,PROCNUM,addr,8,0);
+
 	//special handling for DMA: read 0 from TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -568,6 +640,8 @@ FORCEINLINE u8 _MMU_read08(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u3
 
 FORCEINLINE u16 _MMU_read16(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u32 addr) 
 {
+	CheckMemoryDebugEvent(DEBUG_EVENT_READ,AT,PROCNUM,addr,16,0);
+
 	//special handling for DMA: read 0 from TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -608,6 +682,8 @@ dunno:
 
 FORCEINLINE u32 _MMU_read32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u32 addr)
 {
+	CheckMemoryDebugEvent(DEBUG_EVENT_READ,AT,PROCNUM,addr,32,0);
+
 	//special handling for DMA: read 0 from TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -663,6 +739,8 @@ dunno:
 
 FORCEINLINE void _MMU_write08(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u32 addr, u8 val)
 {
+	CheckMemoryDebugEvent(DEBUG_EVENT_WRITE,AT,PROCNUM,addr,8,val);
+
 	//special handling for DMA: discard writes to TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -697,6 +775,8 @@ FORCEINLINE void _MMU_write08(const int PROCNUM, const MMU_ACCESS_TYPE AT, const
 
 FORCEINLINE void _MMU_write16(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u32 addr, u16 val)
 {
+	CheckMemoryDebugEvent(DEBUG_EVENT_WRITE,AT,PROCNUM,addr,16,val);
+
 	//special handling for DMA: discard writes to TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -731,6 +811,8 @@ FORCEINLINE void _MMU_write16(const int PROCNUM, const MMU_ACCESS_TYPE AT, const
 
 FORCEINLINE void _MMU_write32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u32 addr, u32 val)
 {
+	CheckMemoryDebugEvent(DEBUG_EVENT_WRITE,AT,PROCNUM,addr,32,val);
+
 	//special handling for DMA: discard writes to TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -764,21 +846,21 @@ FORCEINLINE void _MMU_write32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const
 }
 
 
-#ifdef MMU_ENABLE_ACL
-	void FASTCALL MMU_write8_acl(u32 proc, u32 adr, u8 val);
-	void FASTCALL MMU_write16_acl(u32 proc, u32 adr, u16 val);
-	void FASTCALL MMU_write32_acl(u32 proc, u32 adr, u32 val);
-	u8 FASTCALL MMU_read8_acl(u32 proc, u32 adr, u32 access);
-	u16 FASTCALL MMU_read16_acl(u32 proc, u32 adr, u32 access);
-	u32 FASTCALL MMU_read32_acl(u32 proc, u32 adr, u32 access);
-#else
-	#define MMU_write8_acl(proc, adr, val)  _MMU_write08<proc>(adr, val)
-	#define MMU_write16_acl(proc, adr, val) _MMU_write16<proc>(adr, val)
-	#define MMU_write32_acl(proc, adr, val) _MMU_write32<proc>(adr, val)
-	#define MMU_read8_acl(proc,adr,access)  _MMU_read08<proc>(adr)
-	#define MMU_read16_acl(proc,adr,access) ((access==CP15_ACCESS_EXECUTE)?_MMU_read16<proc,MMU_AT_CODE>(adr):_MMU_read16<proc,MMU_AT_DATA>(adr))
-	#define MMU_read32_acl(proc,adr,access) ((access==CP15_ACCESS_EXECUTE)?_MMU_read32<proc,MMU_AT_CODE>(adr):_MMU_read32<proc,MMU_AT_DATA>(adr))
-#endif
+//#ifdef MMU_ENABLE_ACL
+//	void FASTCALL MMU_write8_acl(u32 proc, u32 adr, u8 val);
+//	void FASTCALL MMU_write16_acl(u32 proc, u32 adr, u16 val);
+//	void FASTCALL MMU_write32_acl(u32 proc, u32 adr, u32 val);
+//	u8 FASTCALL MMU_read8_acl(u32 proc, u32 adr, u32 access);
+//	u16 FASTCALL MMU_read16_acl(u32 proc, u32 adr, u32 access);
+//	u32 FASTCALL MMU_read32_acl(u32 proc, u32 adr, u32 access);
+//#else
+//	#define MMU_write8_acl(proc, adr, val)  _MMU_write08<proc>(adr, val)
+//	#define MMU_write16_acl(proc, adr, val) _MMU_write16<proc>(adr, val)
+//	#define MMU_write32_acl(proc, adr, val) _MMU_write32<proc>(adr, val)
+//	#define MMU_read8_acl(proc,adr,access)  _MMU_read08<proc>(adr)
+//	#define MMU_read16_acl(proc,adr,access) ((access==CP15_ACCESS_EXECUTE)?_MMU_read16<proc,MMU_AT_CODE>(adr):_MMU_read16<proc,MMU_AT_DATA>(adr))
+//	#define MMU_read32_acl(proc,adr,access) ((access==CP15_ACCESS_EXECUTE)?_MMU_read32<proc,MMU_AT_CODE>(adr):_MMU_read32<proc,MMU_AT_DATA>(adr))
+//#endif
 
 // Use this macros for reading/writing, so the GDB stub isn't broken
 #ifdef GDB_STUB
