@@ -1,6 +1,7 @@
 /*
 	Copyright (C) 2006 yopyop
 	Copyright (C) 2006-2007 shash
+	Copyright (C) 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -15,8 +16,8 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    along with DeSmuME; if not, write to the 
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 //problem - alpha-on-alpha texture rendering might work but the dest alpha buffer isnt tracked correctly
@@ -44,13 +45,13 @@ static void ENDGL() {
 		oglrender_endOpenGL();
 }
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(WXPORT)
 	#define WIN32_LEAN_AND_MEAN
 	#include <windows.h>
 	#include <GL/gl.h>
 	#include <GL/glext.h>
 #else
-#ifdef DESMUME_COCOA
+#ifdef __APPLE__
 	#include <OpenGL/gl.h>
 	#include <OpenGL/glext.h>
 #else
@@ -100,12 +101,12 @@ static u32 textureFormat=0, texturePalette=0;
 
 #ifdef _WIN32
 #define INITOGLEXT(x,y) y = (x)wglGetProcAddress(#y);
-#elif !defined(DESMUME_COCOA)
+#elif !defined(__APPLE__)
 #include <GL/glx.h>
 #define INITOGLEXT(x,y) y = (x)glXGetProcAddress((const GLubyte *) #y);
 #endif
 
-#ifndef DESMUME_COCOA
+#ifndef __APPLE__
 OGLEXT(PFNGLCREATESHADERPROC,glCreateShader)
 //zero: i dont understand this at all. my glext.h has the wrong thing declared here... so I have to do it myself
 typedef void (APIENTRYP X_PFNGLGETSHADERSOURCEPROC) (GLuint shader, GLsizei bufSize, const GLchar **source, GLsizei *length);
@@ -227,7 +228,7 @@ GLenum			oglToonTableTextureID;
 		log = new GLchar[logSize]; \
 		glGetShaderInfoLog(s, logSize, &logSize, log); \
 		INFO("SEVERE : FAILED TO COMPILE GL SHADER : %s\n", log); \
-		delete log; \
+		delete[] log; \
 		if(s)glDeleteShader(s); \
 		NOSHADERS("Failed to compile the "t" shader."); \
 	} \
@@ -244,7 +245,7 @@ GLenum			oglToonTableTextureID;
 		log = new GLchar[logSize]; \
 		glGetProgramInfoLog(p, logSize, &logSize, log); \
 		INFO("SEVERE : FAILED TO LINK GL SHADER PROGRAM : %s\n", log); \
-		delete log; \
+		delete[] log; \
 		if(s1)glDeleteShader(s1); \
 		if(s2)glDeleteShader(s2); \
 		NOSHADERS("Failed to link the shader program."); \
@@ -337,6 +338,8 @@ static void OGLReset()
 	}
 
 	TexCache_Reset();
+	if (currTexture) 
+		delete currTexture;
 	currTexture = NULL;
 
 //	memset(GPU_screenStencil,0,sizeof(GPU_screenStencil));
@@ -415,7 +418,7 @@ static char OGLInit(void)
 
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-#ifndef DESMUME_COCOA
+#ifndef __APPLE__
 	INITOGLEXT(PFNGLCREATESHADERPROC,glCreateShader)
 	INITOGLEXT(X_PFNGLGETSHADERSOURCEPROC,glShaderSource)
 	INITOGLEXT(PFNGLCOMPILESHADERPROC,glCompileShader)
@@ -565,7 +568,7 @@ static void setTexture(unsigned int format, unsigned int texpal)
 			currTexture->texid = (u64)freeTextureIds.front();
 			freeTextureIds.pop();
 
-			glBindTexture(GL_TEXTURE_2D,(GLuint)currTexture->texid);
+						glBindTexture(GL_TEXTURE_2D,(GLuint)currTexture->texid);
 
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
@@ -683,7 +686,7 @@ static void BeginRenderPoly()
 		{
 			lastEnvMode = envMode;
 
-			int _envModes[4] = {0, 1, (2 + gfx3d.shading), 0};
+			int _envModes[4] = {0, 1, (2 + gfx3d.renderState.shading), 0};
 			glUniform1i(texBlendLoc, _envModes[envMode]);
 		}
 	}
@@ -721,16 +724,16 @@ static void InstallPolygonAttrib(unsigned long val)
 
 static void Control()
 {
-	if(gfx3d.enableTexturing) glEnable (GL_TEXTURE_2D);
+	if(gfx3d.renderState.enableTexturing) glEnable (GL_TEXTURE_2D);
 	else glDisable (GL_TEXTURE_2D);
 
-	if(gfx3d.enableAlphaTest)
+	if(gfx3d.renderState.enableAlphaTest)
 		// FIXME: alpha test should pass gfx3d.alphaTestRef==poly->getAlpha
-		glAlphaFunc	(GL_GREATER, gfx3d.alphaTestRef/31.f);
+		glAlphaFunc	(GL_GREATER, gfx3d.renderState.alphaTestRef/31.f);
 	else
 		glAlphaFunc	(GL_GREATER, 0);
 
-	if(gfx3d.enableAlphaBlending)
+	if(gfx3d.renderState.enableAlphaBlending)
 	{
 		glEnable		(GL_BLEND);
 	}
@@ -818,28 +821,27 @@ static void OGLRender()
 
 	if(hasShaders)
 	{
-		//TODO - maybe this should only happen if the toon table is stale (for a slight speedup)
+		if (gfx3d.renderState.invalidateToon)
+		{
+			glActiveTexture(GL_TEXTURE1);
+			glBindTexture(GL_TEXTURE_1D, oglToonTableTextureID);
 
-		glActiveTexture(GL_TEXTURE1);
-		glBindTexture(GL_TEXTURE_1D, oglToonTableTextureID);
-		
-		//generate a 8888 toon table from the ds format one and store it in a texture
-		u32 rgbToonTable[32];
-		for(int i=0;i<32;i++) rgbToonTable[i] = RGB15TO32(gfx3d.u16ToonTable[i], 255);
-		glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGBA, GL_UNSIGNED_BYTE, rgbToonTable);
+			glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGBA, GL_UNSIGNED_BYTE, &gfx3d.renderState.rgbToonTable[0]);
+			gfx3d.renderState.invalidateToon = false;
+		}
 	}
 
 	xglDepthMask(GL_TRUE);
 
 	float clearColor[4] = {
-		((float)(gfx3d.clearColor&0x1F))/31.0f,
-		((float)((gfx3d.clearColor>>5)&0x1F))/31.0f,
-		((float)((gfx3d.clearColor>>10)&0x1F))/31.0f,
-		((float)((gfx3d.clearColor>>16)&0x1F))/31.0f,
+		((float)(gfx3d.renderState.clearColor&0x1F))/31.0f,
+		((float)((gfx3d.renderState.clearColor>>5)&0x1F))/31.0f,
+		((float)((gfx3d.renderState.clearColor>>10)&0x1F))/31.0f,
+		((float)((gfx3d.renderState.clearColor>>16)&0x1F))/31.0f,
 	};
 	glClearColor(clearColor[0],clearColor[1],clearColor[2],clearColor[3]);
-	glClearDepth(gfx3d.clearDepth);
-	glClearStencil((gfx3d.clearColor >> 24) & 0x3F);
+	glClearDepth(gfx3d.renderState.clearDepth);
+	glClearStencil((gfx3d.renderState.clearColor >> 24) & 0x3F);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
 
 	glMatrixMode(GL_PROJECTION);
@@ -854,7 +856,7 @@ static void OGLRender()
 		// int lastProjIndex = -1;
 
 		for(int i=0;i<gfx3d.polylist->count;i++) {
-			POLY *poly = &gfx3d.polylist->list[gfx3d.indexlist[i]];
+			POLY *poly = &gfx3d.polylist->list[gfx3d.indexlist.list[i]];
 			int type = poly->type;
 
 			//a very macro-level state caching approach:
