@@ -1,8 +1,7 @@
-/*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
+/*  SPU.cpp
+    Copyright (C) 2006 yopyop
     Copyright (C) 2006 Theo Berkau
-    Copyright (C) 2008-2009 DeSmuME team
+    Copyright (C) 2008-2010 DeSmuME team
 
     Ideas borrowed from Stephane Dallongeville's SCSP core
 
@@ -47,10 +46,10 @@
 
 #define K_ADPCM_LOOPING_RECOVERY_INDEX 99999
 
-//#undef FORCEINLINE
-//#define FORCEINLINE
-
-
+//#ifdef FASTBUILD
+	#undef FORCEINLINE
+	#define FORCEINLINE
+//#endif
 
 //static ISynchronizingAudioBuffer* synchronizer = metaspu_construct(ESynchMethod_Z);
 static ISynchronizingAudioBuffer* synchronizer = metaspu_construct(ESynchMethod_N);
@@ -251,15 +250,19 @@ void SPU_Reset(void)
 	int i;
 
 	SPU_core->reset();
-	if(SPU_user) SPU_user->reset();
 
-	if(SNDCore && SPU_user) {
-		SNDCore->DeInit();
-		SNDCore->Init(SPU_user->bufsize*2);
-		SNDCore->SetVolume(volume);
-		//todo - check success?
+	if(SPU_user) {
+		if(SNDCore)
+		{
+			SNDCore->DeInit();
+			SNDCore->Init(SPU_user->bufsize*2);
+			SNDCore->SetVolume(volume);
+		}
+		SPU_user->reset();
 	}
 
+	//zero - 09-apr-2010: this concerns me, regarding savestate synch.
+	//After 0.9.6, lets experiment with removing it and just properly zapping the spu instead
 	// Reset Registers
 	for (i = 0x400; i < 0x51D; i++)
 		T1WriteByte(MMU.ARM7_REG, i, 0);
@@ -276,6 +279,8 @@ void SPU_struct::reset()
 
 	memset((void *)channels, 0, sizeof(channel_struct) * 16);
 
+	reconstruct(&regs);
+
 	for(int i = 0; i < 16; i++)
 	{
 		channels[i].num = i;
@@ -323,12 +328,39 @@ static FORCEINLINE void adjust_channel_timer(channel_struct *chan)
 	chan->sampinc = (((double)ARM7_CLOCK) / (DESMUME_SAMPLE_RATE * 2)) / (double)(0x10000 - chan->timer);
 }
 
+void SPU_struct::KeyProbe(int chan_num)
+{
+	channel_struct &thischan = channels[chan_num];
+	if(thischan.status == CHANSTAT_STOPPED)
+	{
+		if(thischan.keyon && regs.masteren)
+			KeyOn(chan_num);
+	}
+	else if(thischan.status == CHANSTAT_PLAY)
+	{
+		if(!thischan.keyon || !regs.masteren)
+			KeyOff(chan_num);
+	}
+}
+
+void SPU_struct::KeyOff(int channel)
+{
+	//printf("keyoff%d\n",channel);
+	channel_struct &thischan = channels[channel];
+	thischan.status = CHANSTAT_STOPPED;
+}
+
 void SPU_struct::KeyOn(int channel)
 {
 	channel_struct &thischan = channels[channel];
+	thischan.status = CHANSTAT_PLAY;
 
+	thischan.totlength = thischan.length + thischan.loopstart;
 	adjust_channel_timer(&thischan);
 
+	//printf("keyon %d totlength:%d\n",channel,thischan.totlength);
+
+
 	//LOG("Channel %d key on: vol = %d, datashift = %d, hold = %d, pan = %d, waveduty = %d, repeat = %d, format = %d, source address = %07X,"
 	//		"timer = %04X, loop start = %04X, length = %06X, MMU.ARM7_REG[0x501] = %02X\n", channel, chan->vol, chan->datashift, chan->hold, 
 	//		chan->pan, chan->waveduty, chan->repeat, chan->format, chan->addr, chan->timer, chan->loopstart, chan->length, T1ReadByte(MMU.ARM7_REG, 0x501));
@@ -369,6 +401,8 @@ void SPU_struct::KeyOn(int channel)
 	default: break;
 	}
 
+	thischan.double_totlength_shifted = (double)(thischan.totlength << format_shift[thischan.format]);
+
 	if(thischan.format != 3)
 	{
 		if(thischan.double_totlength_shifted == 0)
@@ -377,157 +411,355 @@ void SPU_struct::KeyOn(int channel)
 			thischan.status = CHANSTAT_STOPPED;
 		}
 	}
-	
-	thischan.double_totlength_shifted = (double)(thischan.totlength << format_shift[thischan.format]);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SPU_struct::WriteByte(u32 addr, u8 val)
+#define SETBYTE(which,oldval,newval) oldval = (oldval & (~(0xFF<<(which*8)))) | ((newval)<<(which*8))
+#define GETBYTE(which,val) ((val>>(which*8))&0xFF)
+
+
+u8 SPU_ReadByte(u32 addr) { 
+	addr &= 0xFFF;
+	return SPU_core->ReadByte(addr);
+}
+u16 SPU_ReadWord(u32 addr) {
+	addr &= 0xFFF;
+	return SPU_core->ReadWord(addr);
+}
+u32 SPU_ReadLong(u32 addr) {
+	addr &= 0xFFF;
+	return SPU_core->ReadLong(addr);
+}
+
+u16 SPU_struct::ReadWord(u32 addr)
 {
-	channel_struct &thischan=channels[(addr >> 4) & 0xF];
-	switch(addr & 0xF) {
-		case 0x0:
-			thischan.vol = val & 0x7F;
-			break;
-		case 0x1: {
-			thischan.datashift = val & 0x3;
-			if (thischan.datashift == 3)
-				thischan.datashift = 4;
-			thischan.hold = (val >> 7) & 0x1;
-			break;
-		}
-		case 0x2:
-			thischan.pan = val & 0x7F;
-			break;
-		case 0x3: {
-			thischan.waveduty = val & 0x7;
-			thischan.repeat = (val >> 3) & 0x3;
-			thischan.format = (val >> 5) & 0x3;
-			if((!thischan.status) && BIT7(val))
-				KeyOn((addr >> 4) & 0xF);
-			thischan.status = BIT7(val);
-			break;
-		}
+	return ReadByte(addr)|(ReadByte(addr+1)<<8);
+}
+
+u32 SPU_struct::ReadLong(u32 addr)
+{
+	return ReadByte(addr)|(ReadByte(addr+1)<<8)|(ReadByte(addr+2)<<16)|(ReadByte(addr+3)<<24);
+}
+
+u8 SPU_struct::ReadByte(u32 addr)
+{
+	switch(addr)
+	{
+	//SOUNDCNT
+	case 0x500: return regs.mastervol;
+	case 0x501: 
+		return (regs.ctl_left)|(regs.ctl_right<<2)|(regs.ctl_ch1bypass<<4)|(regs.ctl_ch3bypass<<5)|(regs.masteren<<7);
+	case 0x502: return 0;
+	case 0x503: return 0;
+
+	//SOUNDBIAS
+	case 0x504: return regs.soundbias&0xFF;
+	case 0x505: return (regs.soundbias>>8)&0xFF;
+	case 0x506: return 0;
+	case 0x507: return 0;
+	
+	//SNDCAP0CNT/SNDCAP1CNT
+	case 0x508:
+	case 0x509: {
+		u32 which = addr-0x508;
+		return regs.cap[which].add
+			| (regs.cap[which].source<<1)
+			| (regs.cap[which].oneshot<<2)
+			| (regs.cap[which].bits8<<3)
+			//| (regs.cap[which].active<<7); //? which is right? need test
+			| (regs.cap[which].runtime.running<<7);
+	}	
+
+	//SNDCAP0DAD
+	case 0x510: return GETBYTE(0,regs.cap[0].dad);
+	case 0x511: return GETBYTE(1,regs.cap[0].dad);
+	case 0x512: return GETBYTE(2,regs.cap[0].dad);
+	case 0x513: return GETBYTE(3,regs.cap[0].dad);
+
+	//SNDCAP0LEN
+	case 0x514: return GETBYTE(0,regs.cap[0].len);
+	case 0x515: return GETBYTE(1,regs.cap[0].len);
+	case 0x516: return 0; //not used
+	case 0x517: return 0; //not used
+
+	//SNDCAP1DAD
+	case 0x518: return GETBYTE(0,regs.cap[1].dad);
+	case 0x519: return GETBYTE(1,regs.cap[1].dad);
+	case 0x51A: return GETBYTE(2,regs.cap[1].dad);
+	case 0x51B: return GETBYTE(3,regs.cap[1].dad);
+
+	//SNDCAP1LEN
+	case 0x51C: return GETBYTE(0,regs.cap[1].len);
+	case 0x51D: return GETBYTE(1,regs.cap[1].len);
+	case 0x51E: return 0; //not used
+	case 0x51F: return 0; //not used
+
+	default: {
+		//individual channel regs
+
+		u32 chan_num = (addr >> 4) & 0xF;
+		if(chan_num>0xF) return 0;
+		channel_struct &thischan=channels[chan_num];
+
+		switch(addr & 0xF) {
+			case 0x0: return thischan.vol;
+			case 0x1: {
+				u8 ret = thischan.datashift;
+				if(ret==4) ret=3;
+				ret |= thischan.hold<<7;
+				return ret;
+			}
+			case 0x2: return thischan.pan;
+			case 0x3: return thischan.waveduty|(thischan.repeat<<3)|(thischan.format<<5)|((thischan.status == CHANSTAT_PLAY)?0x80:0);
+			case 0x4: return GETBYTE(0,thischan.addr);
+			case 0x5: return GETBYTE(1,thischan.addr);
+			case 0x6: return GETBYTE(2,thischan.addr);
+			case 0x7: return GETBYTE(3,thischan.addr);
+			case 0x8: return GETBYTE(0,thischan.timer);
+			case 0x9: return GETBYTE(1,thischan.timer);
+			case 0xA: return GETBYTE(0,thischan.loopstart);
+			case 0xB: return GETBYTE(1,thischan.loopstart);
+			case 0xC: return GETBYTE(0,thischan.length);
+			case 0xD: return GETBYTE(1,thischan.length);
+			case 0xE: return GETBYTE(2,thischan.length);
+			case 0xF: return GETBYTE(3,thischan.length);
+			default: return 0; //impossible
+		} //switch on individual channel regs
+		} //default case
+	} //switch on address
+}
+
+SPUFifo::SPUFifo()
+{
+	reset();
+}
+
+void SPUFifo::reset()
+{
+	head = tail = size = 0;
+}
+
+void SPUFifo::enqueue(s16 val)
+{
+	if(size==16) return;
+	buffer[tail] = val;
+	tail++;
+	tail &= 15;
+	size++;
+}
+
+s16 SPUFifo::dequeue()
+{
+	if(size==0) return 0;
+	head++;
+	head &= 15;
+	s16 ret = buffer[head];
+	size--;
+	return ret;
+}
+
+void SPUFifo::save(EMUFILE* fp)
+{
+	u32 version = 1;
+	write32le(version,fp);
+	write32le(head,fp);
+	write32le(tail,fp);
+	write32le(size,fp);
+	for(int i=0;i<16;i++)
+		write16le(buffer[i],fp);
+}
+
+bool SPUFifo::load(EMUFILE* fp)
+{
+	u32 version;
+	if(read32le(&version,fp) != 1) return false;
+	read32le(&head,fp);
+	read32le(&tail,fp);
+	read32le(&size,fp);
+	for(int i=0;i<16;i++)
+		read16le(&buffer[i],fp);
+	return true;
+}
+
+void SPU_struct::ProbeCapture(int which)
+{
+	//VERY UNTESTED -- HOW MUCH OF THIS RESETS, AND WHEN?
+
+	if(!regs.cap[which].active)
+	{
+		regs.cap[which].runtime.running = 0;
+		return;
 	}
 
+	REGS::CAP &cap = regs.cap[which];
+	cap.runtime.running = 1;
+	cap.runtime.curdad = cap.dad;
+	u32 len = cap.len;
+	if(len==0) len=1;
+	cap.runtime.maxdad = cap.dad + len*4;
+	cap.runtime.sampcnt = 0;
+	cap.runtime.fifo.reset();
+}
+
+void SPU_struct::WriteByte(u32 addr, u8 val)
+{
+	switch(addr)
+	{
+	//SOUNDCNT
+	case 0x500:
+		regs.mastervol = val&0x7F;
+		break;
+	case 0x501:
+		regs.ctl_left  = (val>>0)&3;
+		regs.ctl_right = (val>>2)&3;
+		regs.ctl_ch1bypass = (val>>4)&1;
+		regs.ctl_ch3bypass = (val>>5)&1;
+		regs.masteren = (val>>7)&1;
+		for(int i=0;i<16;i++)
+			KeyProbe(i);
+		break;
+	case 0x502: break; //not used
+	case 0x503: break; //not used
+
+	//SOUNDBIAS
+	case 0x504: SETBYTE(0,regs.soundbias, val); break;
+	case 0x505: SETBYTE(1,regs.soundbias, val&3); break;
+	case 0x506: break; //these dont answer anyway
+	case 0x507: break; //these dont answer anyway
+
+	//SNDCAP0CNT/SNDCAP1CNT
+	case 0x508:
+	case 0x509: {
+		u32 which = addr-0x508;
+		regs.cap[which].add = BIT0(val);
+		regs.cap[which].source = BIT1(val);
+		regs.cap[which].oneshot = BIT2(val);
+		regs.cap[which].bits8 = BIT3(val);
+		regs.cap[which].active = BIT7(val);
+		ProbeCapture(which);
+		break;
+	}
+
+	//SNDCAP0DAD
+	case 0x510: SETBYTE(0,regs.cap[0].dad,val); break;
+	case 0x511: SETBYTE(1,regs.cap[0].dad,val); break;
+	case 0x512: SETBYTE(2,regs.cap[0].dad,val); break;
+	case 0x513: SETBYTE(3,regs.cap[0].dad,val&7); break;
+
+	//SNDCAP0LEN
+	case 0x514: SETBYTE(0,regs.cap[0].len,val); break;
+	case 0x515: SETBYTE(1,regs.cap[0].len,val); break;
+	case 0x516: break; //not used
+	case 0x517: break; //not used
+
+	//SNDCAP1DAD
+	case 0x518: SETBYTE(0,regs.cap[1].dad,val); break;
+	case 0x519: SETBYTE(1,regs.cap[1].dad,val); break;
+	case 0x51A: SETBYTE(2,regs.cap[1].dad,val); break;
+	case 0x51B: SETBYTE(3,regs.cap[1].dad,val&7); break;
+
+	//SNDCAP1LEN
+	case 0x51C: SETBYTE(0,regs.cap[1].len,val); break;
+	case 0x51D: SETBYTE(1,regs.cap[1].len,val); break;
+	case 0x51E: break; //not used
+	case 0x51F: break; //not used
+
+
+	
+	default: {
+		//individual channel regs
+		
+		u32 chan_num = (addr >> 4) & 0xF;
+		if(chan_num>0xF) break;
+		channel_struct &thischan=channels[chan_num];
+
+		switch(addr & 0xF) {
+			case 0x0:
+				thischan.vol = val & 0x7F;
+				break;
+			case 0x1:
+				thischan.datashift = val & 0x3;
+				if (thischan.datashift == 3)
+					thischan.datashift = 4;
+				thischan.hold = (val >> 7) & 0x1;
+				break;
+			case 0x2:
+				thischan.pan = val & 0x7F;
+				break;
+			case 0x3:
+				thischan.waveduty = val & 0x7;
+				thischan.repeat = (val >> 3) & 0x3;
+				thischan.format = (val >> 5) & 0x3;
+				thischan.keyon = BIT7(val);
+				KeyProbe(chan_num);
+				break;
+			case 0x4: SETBYTE(0,thischan.addr,val); break;
+			case 0x5: SETBYTE(1,thischan.addr,val); break;
+			case 0x6: SETBYTE(2,thischan.addr,val); break;
+			case 0x7: SETBYTE(3,thischan.addr,val); break;
+			case 0x8: 
+				SETBYTE(0,thischan.timer,val); 
+				adjust_channel_timer(&thischan);
+				break;
+			case 0x9: 
+				SETBYTE(1,thischan.timer,val); 
+				adjust_channel_timer(&thischan);
+				break;
+			case 0xA: SETBYTE(0,thischan.loopstart,val); break;
+			case 0xB: SETBYTE(1,thischan.loopstart,val); break;
+			case 0xC: SETBYTE(0,thischan.length,val); break;
+			case 0xD: SETBYTE(1,thischan.length,val); break;
+			case 0xE: SETBYTE(2,thischan.length,val); break;
+			case 0xF: SETBYTE(3,thischan.length,val); break;
+		} //switch on individual channel regs
+		} //default case
+	} //switch on address
 }
 
 void SPU_WriteByte(u32 addr, u8 val)
 {
+	//printf("%08X: chan:%02X reg:%02X val:%02X\n",addr,(addr>>4)&0xF,addr&0xF,val);
 	addr &= 0xFFF;
 
-	if (addr < 0x500)
-	{
-		SPU_core->WriteByte(addr,val);
-		if(SPU_user) SPU_user->WriteByte(addr,val);
-	}
-
-	T1WriteByte(MMU.ARM7_REG, addr, val);
+	SPU_core->WriteByte(addr,val);
+	if(SPU_user) SPU_user->WriteByte(addr,val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 void SPU_struct::WriteWord(u32 addr, u16 val)
 {
-	channel_struct &thischan=channels[(addr >> 4) & 0xF];
-	switch(addr & 0xF)
-	{
-	case 0x0:
-		thischan.vol = val & 0x7F;
-		thischan.datashift = (val >> 8) & 0x3;
-		if (thischan.datashift == 3)
-			thischan.datashift = 4;
-		thischan.hold = (val >> 15) & 0x1;
-		break;
-	case 0x2:
-		thischan.pan = val & 0x7F;
-		thischan.waveduty = (val >> 8) & 0x7;
-		thischan.repeat = (val >> 11) & 0x3;
-		thischan.format = (val >> 13) & 0x3;
-		if((!thischan.status) && BIT15(val))
-			KeyOn((addr >> 4) & 0xF);
-		thischan.status = BIT15(val);
-		break;
-	case 0x8:
-		thischan.timer = val & 0xFFFF;
-		adjust_channel_timer(&thischan);
-		break;
-	case 0xA:
-		thischan.loopstart = val;
-		thischan.totlength = thischan.length + thischan.loopstart;
-		thischan.double_totlength_shifted = (double)(thischan.totlength << format_shift[thischan.format]);
-		break;
-	case 0xC:
-		WriteLong(addr,((u32)T1ReadWord(MMU.ARM7_REG, addr+2) << 16) | val);
-		break;
-	case 0xE:
-		WriteLong(addr,((u32)T1ReadWord(MMU.ARM7_REG, addr-2)) | ((u32)val<<16));
-		break;
-	}
+	WriteByte(addr,val&0xFF);
+	WriteByte(addr+1,(val>>8)&0xFF);
 }
 
 void SPU_WriteWord(u32 addr, u16 val)
 {
+	//printf("%08X: chan:%02X reg:%02X val:%04X\n",addr,(addr>>4)&0xF,addr&0xF,val);
 	addr &= 0xFFF;
 
-	if (addr < 0x500)
-	{
-		SPU_core->WriteWord(addr,val);
-		if(SPU_user) SPU_user->WriteWord(addr,val);
-	}
-
-	T1WriteWord(MMU.ARM7_REG, addr, val);
+	SPU_core->WriteWord(addr,val);
+	if(SPU_user) SPU_user->WriteWord(addr,val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 void SPU_struct::WriteLong(u32 addr, u32 val)
 {
-	channel_struct &thischan=channels[(addr >> 4) & 0xF];
-	switch(addr & 0xF)
-	{
-	case 0x0:
-		thischan.vol = val & 0x7F;
-		thischan.datashift = (val >> 8) & 0x3;
-		if (thischan.datashift == 3)
-			thischan.datashift = 4;
-		thischan.hold = (val >> 15) & 0x1;
-		thischan.pan = (val >> 16) & 0x7F;
-		thischan.waveduty = (val >> 24) & 0x7;
-		thischan.repeat = (val >> 27) & 0x3;
-		thischan.format = (val >> 29) & 0x3;
-		if((!thischan.status) && BIT31(val))
-			KeyOn((addr >> 4) & 0xF);
-		thischan.status = BIT31(val);
-		break;
-	case 0x4:
-		thischan.addr = val & 0x7FFFFFF;
-		break;
-	case 0x8:
-		thischan.timer = val & 0xFFFF;
-		thischan.loopstart = val >> 16;
-		adjust_channel_timer(&thischan);
-		break;
-	case 0xC:
-		thischan.length = val & 0x3FFFFF;
-		thischan.totlength = thischan.length + thischan.loopstart;
-		thischan.double_totlength_shifted = (double)(thischan.totlength << format_shift[thischan.format]);
-		break;
-	}
+	WriteByte(addr,val&0xFF);
+	WriteByte(addr+1,(val>>8)&0xFF);
+	WriteByte(addr+2,(val>>16)&0xFF);
+	WriteByte(addr+3,(val>>24)&0xFF);
 }
 
 void SPU_WriteLong(u32 addr, u32 val)
 {
+	//printf("%08X: chan:%02X reg:%02X val:%08X\n",addr,(addr>>4)&0xF,addr&0xF,val);
 	addr &= 0xFFF;
 
-	if (addr < 0x500)
-	{
-		SPU_core->WriteLong(addr,val);
-		if(SPU_user) SPU_user->WriteLong(addr,val);
-	}
-
-	T1WriteLong(MMU.ARM7_REG, addr, val);
+	SPU_core->WriteLong(addr,val);
+	if(SPU_user) SPU_user->WriteLong(addr,val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -722,10 +954,7 @@ template<int FORMAT> static FORCEINLINE void TestForLoop(SPU_struct *SPU, channe
 		}
 		else
 		{
-			chan->status = CHANSTAT_STOPPED;
-
-			if(SPU == SPU_core)
-				MMU.ARM7_REG[0x403 + (((chan-SPU->channels) ) * 0x10)] &= 0x7F;
+			SPU->KeyOff(chan->num);
 			SPU->bufpos = SPU->buflength;
 		}
 	}
@@ -759,8 +988,7 @@ static FORCEINLINE void TestForLoop2(SPU_struct *SPU, channel_struct *chan)
 		else
 		{
 			chan->status = CHANSTAT_STOPPED;
-			if(SPU == SPU_core)
-				MMU.ARM7_REG[0x403 + (((chan-SPU->channels) ) * 0x10)] &= 0x7F;
+			SPU->KeyOff(chan->num);
 			SPU->bufpos = SPU->buflength;
 		}
 	}
@@ -774,8 +1002,10 @@ template<int CHANNELS> FORCEINLINE static void SPU_Mix(SPU_struct* SPU, channel_
 		case 1: MixLR(SPU, chan, data); break;
 		case 2: MixR(SPU, chan, data); break;
 	}
+	SPU->lastdata = data;
 }
 
+//WORK
 template<int FORMAT, SPUInterpolationMode INTERPOLATE_MODE, int CHANNELS> 
 	FORCEINLINE static void ____SPU_ChanUpdate(SPU_struct* const SPU, channel_struct* const chan)
 {
@@ -839,54 +1069,278 @@ FORCEINLINE static void _SPU_ChanUpdate(const bool actuallyMix, SPU_struct* cons
 	}
 }
 
+//ENTERNEW
+static void SPU_MixAudio_Advanced(bool actuallyMix, SPU_struct *SPU, int length)
+{
+	//the advanced spu function correctly handles all sound control mixing options, as well as capture
+	//this code is not entirely optimal, as it relies on sort of manhandling the core mixing functions
+	//in order to get the results it needs.
+
+	//THIS IS MAX HACKS!!!!
+	//AND NEEDS TO BE REWRITTEN ALONG WITH THE DEEPEST PARTS OF THE SPU
+	//ONCE WE KNOW THAT IT WORKS
+	
+	//BIAS gets ignored since our spu is still not bit perfect,
+	//and it doesnt matter for purposes of capture
+
+	//-----------DEBUG CODE
+	bool skipcap = false;
+	//-----------------
+
+	s32 samp0[2];
+	
+	//believe it or not, we are going to do this one sample at a time.
+	//like i said, it is slower.
+	for(int samp=0;samp<length;samp++)
+	{
+		SPU->sndbuf[0] = 0;
+		SPU->sndbuf[1] = 0;
+		SPU->buflength = 1;
+
+		s32 capmix[2] = {0,0};
+		s32 mix[2] = {0,0};
+		s32 chanout[16];
+		s32 submix[32];
+
+		//generate each channel, and helpfully mix it at the same time
+		for(int i=0;i<16;i++)
+		{
+			channel_struct *chan = &SPU->channels[i];
+
+			if (chan->status == CHANSTAT_PLAY)
+			{
+				SPU->bufpos = 0;
+
+				bool bypass = false;
+				if(i==1 && SPU->regs.ctl_ch1bypass) bypass=true;
+				if(i==3 && SPU->regs.ctl_ch3bypass) bypass=true;
+
+
+				//output to mixer unless we are bypassed.
+				//dont output to mixer if the user muted us
+				bool outputToMix = true;
+				if(CommonSettings.spu_muteChannels[i]) outputToMix = false;
+				if(bypass) outputToMix = false;
+				bool outputToCap = outputToMix;
+				if(CommonSettings.spu_captureMuted && !bypass) outputToCap = true;
+
+				//channels 1 and 3 should probably always generate their audio
+				//internally at least, just in case they get used by the spu output
+				bool domix = outputToCap || outputToMix || i==1 || i==3;
+
+				//clear the output buffer since this is where _SPU_ChanUpdate wants to accumulate things
+				SPU->sndbuf[0] = SPU->sndbuf[1] = 0;
+
+				//get channel's next output sample.
+				_SPU_ChanUpdate(domix, SPU, chan);
+				chanout[i] = SPU->lastdata >> chan->datashift;
+
+				//save the panned results
+				submix[i*2] = SPU->sndbuf[0];
+				submix[i*2+1] = SPU->sndbuf[1];
+
+				//send sample to our capture mix
+				if(outputToCap)
+				{
+					capmix[0] += submix[i*2];
+					capmix[1] += submix[i*2+1];
+				}
+
+				//send sample to our main mixer
+				if(outputToMix)
+				{
+					mix[0] += submix[i*2];
+					mix[1] += submix[i*2+1];
+				}
+			}
+			else 
+			{
+				chanout[i] = 0;
+				submix[i*2] = 0;
+				submix[i*2+1] = 0;
+			}
+		} //foreach channel
+
+		s32 mixout[2] = {mix[0],mix[1]};
+		s32 capmixout[2] = {capmix[0],capmix[1]};
+		s32 sndout[2];
+		s32 capout[2];
+
+		//create SPU output
+		switch(SPU->regs.ctl_left)
+		{
+		case SPU_struct::REGS::LOM_LEFT_MIXER: sndout[0] = mixout[0]; break;
+		case SPU_struct::REGS::LOM_CH1: sndout[0] = submix[1*2+0]; break;
+		case SPU_struct::REGS::LOM_CH3: sndout[0] = submix[3*2+0]; break;
+		case SPU_struct::REGS::LOM_CH1_PLUS_CH3: sndout[0] = submix[1*2+0] + submix[3*2+0]; break;
+		}
+		switch(SPU->regs.ctl_right)
+		{
+		case SPU_struct::REGS::ROM_RIGHT_MIXER: sndout[1] = mixout[1]; break;
+		case SPU_struct::REGS::ROM_CH1: sndout[1] = submix[1*2+1]; break;
+		case SPU_struct::REGS::ROM_CH3: sndout[1] = submix[3*2+1]; break;
+		case SPU_struct::REGS::ROM_CH1_PLUS_CH3: sndout[1] = submix[1*2+1] + submix[3*2+1]; break;
+		}
+
+
+		//generate capture output ("capture bugs" from gbatek are not emulated)
+		if(SPU->regs.cap[0].source==0) 
+			capout[0] = capmixout[0]; //cap0 = L-mix
+		else if(SPU->regs.cap[0].add)
+			capout[0] = chanout[0] + chanout[1]; //cap0 = ch0+ch1
+		else capout[0] = chanout[0]; //cap0 = ch0
+
+		if(SPU->regs.cap[1].source==0) 
+			capout[1] = capmixout[1]; //cap1 = R-mix
+		else if(SPU->regs.cap[1].add)
+			capout[1] = chanout[2] + chanout[3]; //cap1 = ch2+ch3
+		else capout[1] = chanout[2]; //cap1 = ch2
+
+		capout[0] = MinMax(capout[0],-0x8000,0x7FFF);
+		capout[1] = MinMax(capout[1],-0x8000,0x7FFF);
+
+		//write the output sample where it is supposed to go
+		if(samp==0)
+		{
+			samp0[0] = sndout[0];
+			samp0[1] = sndout[1];
+		}
+		else
+		{
+			SPU->sndbuf[samp*2+0] = sndout[0];
+			SPU->sndbuf[samp*2+1] = sndout[1];
+		}
+
+		for(int capchan=0;capchan<2;capchan++)
+		{
+			if(SPU->regs.cap[capchan].runtime.running)
+			{
+				SPU_struct::REGS::CAP& cap = SPU->regs.cap[capchan];
+				u32 last = sputrunc(cap.runtime.sampcnt);
+				cap.runtime.sampcnt += SPU->channels[1+2*capchan].sampinc;
+				u32 curr = sputrunc(cap.runtime.sampcnt);
+				for(u32 j=last;j<curr;j++)
+				{
+					//so, this is a little strange. why go through a fifo?
+					//it seems that some games will set up a reverb effect by capturing
+					//to the nearly same address as playback, but ahead by a couple.
+					//So, playback will always end up being what was captured a couple of samples ago.
+					//This system counts on playback always having read ahead 16 samples.
+					//In that case, playback will end up being what was processed at one entire buffer length ago,
+					//since the 16 samples would have read ahead before they got captured over
+
+					//It's actually the source channels which should have a fifo, but we are
+					//not going to take the hit in speed and complexity. Save it for a future rewrite.
+					//Instead, what we do here is delay the capture by 16 samples to create a similar effect.
+					//Subjectively, it seems to be working.
+
+					//Don't do anything until the fifo is filled, so as to delay it
+					if(cap.runtime.fifo.size<16)
+					{
+						cap.runtime.fifo.enqueue(capout[capchan]);
+						continue;
+					}
+
+					//(actually capture sample from fifo instead of most recently generated)
+					u32 multiplier;
+					s32 sample = cap.runtime.fifo.dequeue();
+					cap.runtime.fifo.enqueue(capout[capchan]);
+
+					//static FILE* fp = NULL;
+					//if(!fp) fp = fopen("d:\\capout.raw","wb");
+					//fwrite(&sample,2,1,fp);
+					
+					if(cap.bits8)
+					{
+						s8 sample8 = sample>>8;
+						if(skipcap) _MMU_write08<1,MMU_AT_DMA>(cap.runtime.curdad,0);
+						else _MMU_write08<1,MMU_AT_DMA>(cap.runtime.curdad,sample8);
+						cap.runtime.curdad++;
+						multiplier = 4;
+					}
+					else
+					{
+						s16 sample16 = sample;
+						if(skipcap) _MMU_write16<1,MMU_AT_DMA>(cap.runtime.curdad,0);
+						else _MMU_write16<1,MMU_AT_DMA>(cap.runtime.curdad,sample16);
+						cap.runtime.curdad+=2;
+						multiplier = 2;
+					}
+
+					if(cap.runtime.curdad>=cap.runtime.maxdad) {
+						cap.runtime.curdad = cap.dad;
+						cap.runtime.sampcnt -= cap.len*multiplier;
+					}
+				} //sampinc loop
+			} //if capchan running
+		} //capchan loop
+	} //main sample loop
+
+	SPU->sndbuf[0] = samp0[0];
+	SPU->sndbuf[1] = samp0[1];
+}
+
+//ENTER
 static void SPU_MixAudio(bool actuallyMix, SPU_struct *SPU, int length)
 {
-	u8 vol;
-
 	if(actuallyMix)
 	{
 		memset(SPU->sndbuf, 0, length*4*2);
 		memset(SPU->outbuf, 0, length*2*2);
 	}
-	
-	// If the sound speakers are disabled, don't output audio
-	if(!(T1ReadWord(MMU.ARM7_REG, 0x304) & 0x01))
-		return;
 
-	// If Master Enable isn't set, don't output audio
-	if (!(T1ReadByte(MMU.ARM7_REG, 0x501) & 0x80))
-		return;
+	//we used to use master enable here, and do nothing if audio is disabled.
+	//now, master enable is emulated better..
+	//but for a speed optimization we will still do it
+	if(!SPU->regs.masteren) return;
 
-	vol = T1ReadByte(MMU.ARM7_REG, 0x500) & 0x7F;
+	bool advanced = CommonSettings.spu_advanced ;
 
-	for(int i=0;i<16;i++)
+	//branch here so that slow computers don't have to take the advanced (slower) codepath.
+	//it remainds to be seen exactly how much slower it is
+	//if it isnt much slower then we should refactor everything to be simpler, once it is working
+	if(advanced && SPU == SPU_core)
 	{
-		channel_struct *chan = &SPU->channels[i];
-	
-		if (chan->status != CHANSTAT_PLAY)
-			continue;
+		SPU_MixAudio_Advanced(actuallyMix, SPU, length);
+	}
+	else
+	{
+		//non-advanced mode
+		for(int i=0;i<16;i++)
+		{
+			channel_struct *chan = &SPU->channels[i];
 
-		SPU->bufpos = 0;
-		SPU->buflength = length;
+			if (chan->status != CHANSTAT_PLAY)
+				continue;
 
-		// Mix audio
-		_SPU_ChanUpdate(!CommonSettings.spu_muteChannels[i] && actuallyMix, SPU, chan);
+			SPU->bufpos = 0;
+			SPU->buflength = length;
+
+			// Mix audio
+			_SPU_ChanUpdate(!CommonSettings.spu_muteChannels[i] && actuallyMix, SPU, chan);
+		}
 	}
 
+	//we used to bail out if speakers were disabled.
+	//this is technically wrong. sound may still be captured, or something.
+	//in all likelihood, any game doing this probably master disabled the SPU also
+	//so, optimization of this case is probably not necessary.
+	//later, we'll just silence the output
+	bool speakers = T1ReadWord(MMU.ARM7_REG, 0x304) & 0x01;
+
+	u8 vol = SPU->regs.mastervol;
+
 	// convert from 32-bit->16-bit
-	if(actuallyMix)
+	if(actuallyMix && speakers)
 		for (int i = 0; i < length*2; i++)
 		{
 			// Apply Master Volume
 			SPU->sndbuf[i] = spumuldiv7(SPU->sndbuf[i], vol);
-
-			if (SPU->sndbuf[i] > 0x7FFF)
-				SPU->outbuf[i] = 0x7FFF;
-			else if (SPU->sndbuf[i] < -0x8000)
-				SPU->outbuf[i] = -0x8000;
-			else
-				SPU->outbuf[i] = (s16)SPU->sndbuf[i];
+			s16 outsample = MinMax(SPU->sndbuf[i],-0x8000,0x7FFF);
+			SPU->outbuf[i] = outsample;
 		}
+
+
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1105,7 +1559,7 @@ void WAV_WavSoundUpdate(void* soundData, int numSamples, WAVMode mode)
 void spu_savestate(EMUFILE* os)
 {
 	//version
-	write32le(3,os);
+	write32le(6,os);
 
 	SPU_struct *spu = SPU_core;
 
@@ -1132,9 +1586,36 @@ void spu_savestate(EMUFILE* os)
 		write32le(chan.index,os);
 		write16le(chan.x,os);
 		write16le(chan.psgnoise_last,os);
+		write8le(chan.keyon,os);
 	}
 
 	write64le(double_to_u64(samples),os);
+
+	write8le(spu->regs.mastervol,os);
+	write8le(spu->regs.ctl_left,os);
+	write8le(spu->regs.ctl_right,os);
+	write8le(spu->regs.ctl_ch1bypass,os);
+	write8le(spu->regs.ctl_ch3bypass,os);
+	write8le(spu->regs.masteren,os);
+	write16le(spu->regs.soundbias,os);
+
+	for(int i=0;i<2;i++)
+	{
+		write8le(spu->regs.cap[i].add,os);
+		write8le(spu->regs.cap[i].source,os);
+		write8le(spu->regs.cap[i].oneshot,os);
+		write8le(spu->regs.cap[i].bits8,os);
+		write8le(spu->regs.cap[i].active,os);
+		write32le(spu->regs.cap[i].dad,os);
+		write16le(spu->regs.cap[i].len,os);
+		write8le(spu->regs.cap[i].runtime.running,os);
+		write32le(spu->regs.cap[i].runtime.curdad,os);
+		write32le(spu->regs.cap[i].runtime.maxdad,os);
+		write_double_le(spu->regs.cap[i].runtime.sampcnt,os);
+	}
+
+	for(int i=0;i<2;i++)
+		spu->regs.cap[i].runtime.fifo.save(os);
 }
 
 bool spu_loadstate(EMUFILE* is, int size)
@@ -1146,6 +1627,7 @@ bool spu_loadstate(EMUFILE* is, int size)
 	if(read32le(&version,is) != 1) return false;
 
 	SPU_struct *spu = SPU_core;
+	reconstruct(&SPU_core->regs);
 
 	for(int j=0;j<16;j++) {
 		channel_struct &chan = spu->channels[j];
@@ -1164,6 +1646,7 @@ bool spu_loadstate(EMUFILE* is, int size)
 		read32le(&chan.length,is);
 		chan.totlength = chan.length + chan.loopstart;
 		chan.double_totlength_shifted = (double)(chan.totlength << format_shift[chan.format]);
+		//printf("%f\n",chan.double_totlength_shifted);
 		if(version >= 2)
 		{
 			read64le(&temp64,is); chan.sampcnt = u64_to_double(temp64);
@@ -1181,20 +1664,68 @@ bool spu_loadstate(EMUFILE* is, int size)
 		read16le(&chan.x,is);
 		read16le(&chan.psgnoise_last,is);
 
+		if(version>=4)
+			read8le(&chan.keyon,is);
+
 		//hopefully trigger a recovery of the adpcm looping system
 		chan.loop_index = K_ADPCM_LOOPING_RECOVERY_INDEX;
 
 		//fixup the pointers which we had are supposed to keep cached
 		chan.buf8 = (s8*)&MMU.MMU_MEM[1][(chan.addr>>20)&0xFF][(chan.addr & MMU.MMU_MASK[1][(chan.addr >> 20) & 0xFF])];
+		chan.buf16 = (s16*)chan.buf8;
 	}
 
 	if(version>=2) {
 		read64le(&temp64,is); samples = u64_to_double(temp64);
 	}
 
+	if(version>=4)
+	{
+		read8le(&spu->regs.mastervol,is);
+		read8le(&spu->regs.ctl_left,is);
+		read8le(&spu->regs.ctl_right,is);
+		read8le(&spu->regs.ctl_ch1bypass,is);
+		read8le(&spu->regs.ctl_ch3bypass,is);
+		read8le(&spu->regs.masteren,is);
+		read16le(&spu->regs.soundbias,is);
+	}
+
+	if(version>=5)
+	{
+		for(int i=0;i<2;i++)
+		{
+			read8le(&spu->regs.cap[i].add,is);
+			read8le(&spu->regs.cap[i].source,is);
+			read8le(&spu->regs.cap[i].oneshot,is);
+			read8le(&spu->regs.cap[i].bits8,is);
+			read8le(&spu->regs.cap[i].active,is);
+			read32le(&spu->regs.cap[i].dad,is);
+			read16le(&spu->regs.cap[i].len,is);
+			read8le(&spu->regs.cap[i].runtime.running,is);
+			read32le(&spu->regs.cap[i].runtime.curdad,is);
+			read32le(&spu->regs.cap[i].runtime.maxdad,is);
+			read_double_le(&spu->regs.cap[i].runtime.sampcnt,is);
+		}
+	}
+
+	if(version>=6)
+		for(int i=0;i<2;i++) spu->regs.cap[i].runtime.fifo.load(is);
+	else
+		for(int i=0;i<2;i++) spu->regs.cap[i].runtime.fifo.reset();
+
+	//older versions didnt store a mastervol; 
+	//we must reload this or else games will start silent
+	if(version<4)
+	{
+		spu->regs.mastervol = T1ReadByte(MMU.ARM7_REG, 0x500) & 0x7F;
+		spu->regs.masteren = BIT15(T1ReadWord(MMU.ARM7_REG, 0x500));
+	}
+
+
 	//copy the core spu (the more accurate) to the user spu
 	if(SPU_user) {
 		memcpy(SPU_user->channels,SPU_core->channels,sizeof(SPU_core->channels));
+		SPU_user->regs = SPU_core->regs;
 	}
 
 	return true;
