@@ -1,7 +1,6 @@
 /*  SPU.h
-
 	Copyright 2006 Theo Berkau
-    Copyright (C) 2006-2009 DeSmuME team
+    Copyright (C) 2006-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -37,9 +36,10 @@
 #define CHANSTAT_STOPPED          0
 #define CHANSTAT_PLAY             1
 
-static FORCEINLINE u32 sputrunc(float f) { return u32floor(f); }
-static FORCEINLINE u32 sputrunc(double d) { return u32floor(d); }
-static FORCEINLINE s32 spumuldiv7(s32 val, u8 multiplier) {
+//who made these static? theyre used in multiple places.
+FORCEINLINE u32 sputrunc(float f) { return u32floor(f); }
+FORCEINLINE u32 sputrunc(double d) { return u32floor(d); }
+FORCEINLINE s32 spumuldiv7(s32 val, u8 multiplier) {
 	assert(multiplier <= 127);
 	return (multiplier == 127) ? val : ((val * multiplier) >> 7);
 }
@@ -81,6 +81,7 @@ struct channel_struct
    u8 waveduty;
    u8 repeat;
    u8 format;
+   u8 keyon;
    u8 status;
    u32 addr;
    u16 timer;
@@ -102,7 +103,20 @@ struct channel_struct
    int loop_index;
    u16 x;
    s16 psgnoise_last;
-} ;
+};
+
+class SPUFifo
+{
+public:
+	SPUFifo();
+	void enqueue(s16 val);
+	s16 dequeue();
+	s16 buffer[16];
+	s32 head,tail,size;
+	void save(EMUFILE* fp);
+	bool load(EMUFILE* fp);
+	void reset();
+};
 
 class SPU_struct
 {
@@ -111,14 +125,69 @@ public:
    u32 bufpos;
    u32 buflength;
    s32 *sndbuf;
+   s32 lastdata; //the last sample that a channel generated
    s16 *outbuf;
    u32 bufsize;
    channel_struct channels[16];
 
+   //registers
+   struct REGS {
+	   REGS()
+			: mastervol(0)
+			, ctl_left(0)
+			, ctl_right(0)
+			, ctl_ch1bypass(0)
+			, ctl_ch3bypass(0)
+			, masteren(0)
+			, soundbias(0)
+	   {}
+
+	   u8 mastervol;
+	   u8 ctl_left, ctl_right;
+	   u8 ctl_ch1bypass, ctl_ch3bypass;
+	   u8 masteren;
+	   u16 soundbias;
+
+	   enum LeftOutputMode
+	   {
+		   LOM_LEFT_MIXER=0, LOM_CH1=1, LOM_CH3=2, LOM_CH1_PLUS_CH3=3
+	   };
+
+	   enum RightOutputMode
+	   {
+		   ROM_RIGHT_MIXER=0, ROM_CH1=1, ROM_CH3=2, ROM_CH1_PLUS_CH3=3
+	   };
+
+	   struct CAP {
+		   CAP()
+			   : add(0), source(0), oneshot(0), bits8(0), active(0), dad(0), len(0)
+		   {}
+		   u8 add, source, oneshot, bits8, active;
+		   u32 dad;
+		   u16 len;
+		   struct Runtime {
+			   Runtime()
+				   : running(0), curdad(0), maxdad(0)
+			   {}
+			   u8 running;
+			   u32 curdad;
+			   u32 maxdad;
+			   double sampcnt;
+			   SPUFifo fifo;
+		   } runtime;
+	   } cap[2];
+   } regs;
+
    void reset();
    ~SPU_struct();
+   void KeyOff(int channel);
    void KeyOn(int channel);
+   void KeyProbe(int channel);
+   void ProbeCapture(int which);
    void WriteByte(u32 addr, u8 val);
+   u8 ReadByte(u32 addr);
+   u16 ReadWord(u32 addr);
+   u32 ReadLong(u32 addr);
    void WriteWord(u32 addr, u16 val);
    void WriteLong(u32 addr, u32 val);
    
@@ -140,6 +209,9 @@ void SPU_KeyOn(int channel);
 void SPU_WriteByte(u32 addr, u8 val);
 void SPU_WriteWord(u32 addr, u16 val);
 void SPU_WriteLong(u32 addr, u32 val);
+u8 SPU_ReadByte(u32 addr);
+u16 SPU_ReadWord(u32 addr);
+u32 SPU_ReadLong(u32 addr);
 void SPU_Emulate_core(void);
 void SPU_Emulate_user(bool mix = true);
 
