@@ -111,14 +111,14 @@ armcpu_t NDS_ARM9;
 static void
 stall_cpu( void *instance) {
   armcpu_t *armcpu = (armcpu_t *)instance;
-
+  printf("UNSTALL\n");
   armcpu->stalled = 1;
 }
                       
 static void
 unstall_cpu( void *instance) {
   armcpu_t *armcpu = (armcpu_t *)instance;
-
+  printf("UNSTALL\n");
   armcpu->stalled = 0;
 }
 
@@ -200,15 +200,23 @@ int armcpu_new( armcpu_t *armcpu, u32 id)
 
 	*ctrl_iface_ret = &armcpu->ctrl_iface;
 
-	armcpu->stalled = 0;
 	armcpu->post_ex_fn = NULL;
 #endif
 
+	armcpu->stalled = 0;
+
 	armcpu_init(armcpu, 0);
 
 	return 0;
 } 
 
+//call this whenever CPSR is changed (other than CNVZQ or T flags); interrupts may need to be unleashed
+void armcpu_t::changeCPSR()
+{
+	//but all it does is give them a chance to unleash by forcing an immediate reschedule
+	NDS_Reschedule();
+}
+
 void armcpu_init(armcpu_t *armcpu, u32 adr)
 {
    u32 i;
@@ -259,7 +267,7 @@ void armcpu_init(armcpu_t *armcpu, u32 adr)
 
 u32 armcpu_switchMode(armcpu_t *armcpu, u8 mode)
 {
-        u32 oldmode = armcpu->CPSR.bits.mode;
+	u32 oldmode = armcpu->CPSR.bits.mode;
 	
 	switch(oldmode)
 	{
@@ -361,6 +369,7 @@ u32 armcpu_switchMode(armcpu_t *armcpu, u8 mode)
 	}
 	
 	armcpu->CPSR.bits.mode = mode & 0x1F;
+	armcpu->changeCPSR();
 	return oldmode;
 }
 
@@ -387,7 +396,7 @@ FORCEINLINE static u32 armcpu_prefetch()
 			armcpu->R[15] = armcpu->next_instruction + 4;
 		}
 #else
-		armcpu->instruction = MMU_read32_acl(PROCNUM, curInstruction&0xFFFFFFFC,CP15_ACCESS_EXECUTE);
+		armcpu->instruction = _MMU_read32<PROCNUM,MMU_AT_CODE>(curInstruction&0xFFFFFFFC);
 		armcpu->instruct_adr = curInstruction;
 		armcpu->next_instruction = curInstruction + 4;
 		armcpu->R[15] = curInstruction + 8;
@@ -409,7 +418,7 @@ FORCEINLINE static u32 armcpu_prefetch()
 		armcpu->R[15] = armcpu->next_instruction + 2;
 	}
 #else
-	armcpu->instruction = MMU_read16_acl(PROCNUM, curInstruction&0xFFFFFFFE,CP15_ACCESS_EXECUTE);
+	armcpu->instruction = _MMU_read16<PROCNUM, MMU_AT_CODE>(curInstruction&0xFFFFFFFE);
 	armcpu->instruct_adr = curInstruction;
 	armcpu->next_instruction = curInstruction + 2;
 	armcpu->R[15] = curInstruction + 4;
@@ -504,7 +513,6 @@ armcpu_flagIrq( armcpu_t *armcpu) {
   return TRUE;
 }
 
-
 template<int PROCNUM>
 u32 armcpu_exec()
 {
