@@ -19,21 +19,23 @@
 
 #import <Cocoa/Cocoa.h>
 
-//cocoa utils
-#define NSSTRc(x) ([[NSString alloc] initWithCString:(x) encoding:NSASCIIStringEncoding])
+#ifdef __cplusplus
+extern "C"
+{
+#endif
 
-void messageDialogBlank();
 void messageDialog(NSString *title, NSString *text);
 BOOL messageDialogYN(NSString *title, NSString *text);
 NSString* openDialog(NSArray *file_types);
 
 //
-#if __LP64__ || NS_BUILD_32_LIKE_64
-typedef long NSInteger;
-typedef unsigned long NSUInteger;
-#else
+#if !defined(__LP64__) && !defined(NS_BUILD_32_LIKE_64)
 typedef int NSInteger;
 typedef unsigned int NSUInteger;
 #endif
 
-typedef float CGFloat;
\ No newline at end of file
+typedef float CGFloat;
+
+#ifdef __cplusplus
+}
+#endif
\ No newline at end of file
