@@ -1,6 +1,6 @@
 /*  commandline.cpp
 
-    Copyright (C) 2009 DeSmuME team
+    Copyright (C) 2009-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -31,6 +31,7 @@
 #include "NDSSystem.h"
 
 int scanline_filter_a = 2, scanline_filter_b = 4;
+int _commandline_linux_nojoy = 0;
 
 CommandLine::CommandLine()
 : is_cflash_configured(false)
@@ -40,14 +41,22 @@ CommandLine::CommandLine()
 , _record_movie_file(0)
 , _cflash_image(0)
 , _cflash_path(0)
+, _gbaslot_rom(0)
 , _bios_arm9(NULL)
 , _bios_arm7(NULL)
 , _bios_swi(0)
+, _spu_advanced(0)
 , _num_cores(-1)
+, _rigorous_timing(0)
+, _advanced_timing(-1)
 {
 	load_slot = 0;
 	arm9_gdb_port = arm7_gdb_port = 0;
 	start_paused = FALSE;
+#ifndef _MSC_VER
+	disable_sound = 0;
+	disable_limiter = 0;
+#endif
 }
 
 CommandLine::~CommandLine()
@@ -69,18 +78,25 @@ void CommandLine::loadCommonOptions()
 		{ "start-paused", 0, 0, G_OPTION_ARG_NONE, &start_paused, "Indicates that emulation should start paused", "START_PAUSED"},
 		{ "cflash-image", 0, 0, G_OPTION_ARG_FILENAME, &_cflash_image, "Requests cflash in gbaslot with fat image at this path", "CFLASH_IMAGE"},
 		{ "cflash-path", 0, 0, G_OPTION_ARG_FILENAME, &_cflash_path, "Requests cflash in gbaslot with filesystem rooted at this path", "CFLASH_PATH"},
+		{ "gbaslot-rom", 0, 0, G_OPTION_ARG_FILENAME, &_gbaslot_rom, "Requests this GBA rom in gbaslot", "GBASLOT_ROM"},
 		{ "bios-arm9", 0, 0, G_OPTION_ARG_FILENAME, &_bios_arm9, "Uses the arm9 bios provided at the specified path", "BIOS_ARM9_PATH"},
 		{ "bios-arm7", 0, 0, G_OPTION_ARG_FILENAME, &_bios_arm7, "Uses the arm7 bios provided at the specified path", "BIOS_ARM7_PATH"},
 		{ "bios-swi", 0, 0, G_OPTION_ARG_INT, &_bios_swi, "Uses SWI from the provided bios files", "BIOS_SWI"},
-#ifdef _MSC_VER
+		{ "spu-advanced", 0, 0, G_OPTION_ARG_INT, &_spu_advanced, "Uses advanced SPU capture functions", "SPU_ADVANCED"},
 		{ "num-cores", 0, 0, G_OPTION_ARG_INT, &_num_cores, "Override numcores detection and use this many", "NUM_CORES"},
 		{ "scanline-filter-a", 0, 0, G_OPTION_ARG_INT, &scanline_filter_a, "Intensity of fadeout for scanlines filter (edge) (default 2)", "SCANLINE_FILTER_A"},
 		{ "scanline-filter-b", 0, 0, G_OPTION_ARG_INT, &scanline_filter_b, "Intensity of fadeout for scanlines filter (corner) (default 4)", "SCANLINE_FILTER_B"},
+		{ "rigorous-timing", 0, 0, G_OPTION_ARG_INT, &_rigorous_timing, "Use some rigorous timings instead of unrealistically generous (default 0)", "RIGOROUS_TIMING"},
+		{ "advanced-timing", 0, 0, G_OPTION_ARG_INT, &_advanced_timing, "Use advanced BUS-level timing (default 1)", "ADVANCED_TIMING"},
+#ifndef _MSC_VER
+		{ "disable-sound", 0, 0, G_OPTION_ARG_NONE, &disable_sound, "Disables the sound emulation", NULL},
+		{ "disable-limiter", 0, 0, G_OPTION_ARG_NONE, &disable_limiter, "Disables the 60fps limiter", NULL},
+		{ "nojoy", 0, 0, G_OPTION_ARG_INT, &_commandline_linux_nojoy, "Disables joystick support", "NOJOY"},
 #endif
-		#ifdef GDB_STUB
+#ifdef GDB_STUB
 		{ "arm9gdb", 0, 0, G_OPTION_ARG_INT, &arm9_gdb_port, "Enable the ARM9 GDB stub on the given port", "PORT_NUM"},
 		{ "arm7gdb", 0, 0, G_OPTION_ARG_INT, &arm7_gdb_port, "Enable the ARM7 GDB stub on the given port", "PORT_NUM"},
-		#endif
+#endif
 		{ NULL }
 	};
 
@@ -100,13 +116,17 @@ bool CommandLine::parse(int argc,char **argv)
 	if(_record_movie_file) record_movie_file = _record_movie_file;
 	if(_cflash_image) cflash_image = _cflash_image;
 	if(_cflash_path) cflash_path = _cflash_path;
+	if(_gbaslot_rom) gbaslot_rom = _gbaslot_rom;
 
 	if(_num_cores != -1) CommonSettings.num_cores = _num_cores;
+	if(_rigorous_timing) CommonSettings.rigorous_timing = true;
+	if(_advanced_timing != -1) CommonSettings.advanced_timing = _advanced_timing==1;
 
 	//TODO MAX PRIORITY! change ARM9BIOS etc to be a std::string
 	if(_bios_arm9) { CommonSettings.UseExtBIOS = true; strcpy(CommonSettings.ARM9BIOS,_bios_arm9); }
 	if(_bios_arm7) { CommonSettings.UseExtBIOS = true; strcpy(CommonSettings.ARM7BIOS,_bios_arm7); }
 	if(_bios_swi) CommonSettings.SWIFromBIOS = true;
+	if(_spu_advanced) CommonSettings.spu_advanced = true;
 
 	if (argc == 2)
 		nds_file = argv[1];
@@ -147,6 +167,10 @@ bool CommandLine::validate()
 		g_printerr("If either bios-swi is used, bios-arm9 and bios-arm7 must be specified.\n");
 	}
 
+	if((_cflash_image && _gbaslot_rom) || (_cflash_path && _gbaslot_rom)) {
+		g_printerr("Cannot specify both cflash and gbaslot rom (both occupy SLOT-2)\n");
+	}
+
 	return true;
 }
 
@@ -165,7 +189,7 @@ void CommandLine::process_movieCommands()
 	}
 	else if(record_movie_file != "")
 	{
-		FCEUI_SaveMovie(record_movie_file.c_str(), L"", 0, NULL);
+		FCEUI_SaveMovie(record_movie_file.c_str(), L"", 0, NULL, FCEUI_MovieGetRTCDefault());
 	}
 }
 
