@@ -1,6 +1,6 @@
 /*  commandline.h
 
-    Copyright (C) 2009 DeSmuME team
+    Copyright (C) 2009-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -28,6 +28,9 @@
 typedef struct _GOptionContext GOptionContext;
 typedef struct _GError GError;
 
+//hacky commandline options that i didnt want to route through commonoptions
+extern int _commandline_linux_nojoy;
+
 //this class will also eventually try to take over the responsibility of using the args that it handles
 //for example: preparing the emulator run by loading the rom, savestate, and/or movie in the correct pattern.
 //it should also populate CommonSettings with its initial values
@@ -44,6 +47,11 @@ public:
 	int start_paused;
 	std::string cflash_image;
 	std::string cflash_path;
+	std::string gbaslot_rom;
+#ifndef _MSC_VER
+	int disable_sound;
+	int disable_limiter;
+#endif
 
 	//load up the common commandline options
 	void loadCommonOptions();
@@ -73,9 +81,13 @@ private:
 	char* _record_movie_file;
 	char* _cflash_image;
 	char* _cflash_path;
+	char* _gbaslot_rom;
 	char* _bios_arm9, *_bios_arm7;
 	int _bios_swi;
+	int _spu_advanced;
 	int _num_cores;
+	int _rigorous_timing;
+	int _advanced_timing;
 };
 
 #endif
