@@ -25,7 +25,7 @@
 #include <string>
 #include "common.h"
 
-u8	logo_data[156] = {
+const u8 logo_data[156] = {
 	0x24,0xFF,0xAE,0x51,0x69,0x9A,0xA2,0x21,0x3D,0x84,0x82,0x0A,0x84,0xE4,0x09,0xAD,
 	0x11,0x24,0x8B,0x98,0xC0,0x81,0x7F,0x21,0xA3,0x52,0xBE,0x19,0x93,0x09,0xCE,0x20,
 	0x10,0x46,0x4A,0x4A,0xF8,0x27,0x31,0xEC,0x58,0xC7,0xE8,0x33,0x82,0xE3,0xCE,0xBF,
@@ -37,56 +37,6 @@ u8	logo_data[156] = {
 	0x78,0x00,0x90,0xCB,0x88,0x11,0x3A,0x94,0x65,0xC0,0x7C,0x63,0x87,0xF0,0x3C,0xAF,
 	0xD6,0x25,0xE4,0x8B,0x38,0x0A,0xAC,0x72,0x21,0xD4,0xF8,0x07};
 
-#ifdef WIN32
-char IniName[MAX_PATH];
-
-void GetINIPath()
-{   
-	char		vPath[MAX_PATH], *szPath;
-    /*if (*vPath)
-       szPath = vPath;
-    else
-    {*/
-       char *p;
-       ZeroMemory(vPath, sizeof(vPath));
-       GetModuleFileName(NULL, vPath, sizeof(vPath));
-       p = vPath + lstrlen(vPath);
-       while (p >= vPath && *p != '\\') p--;
-       if (++p >= vPath) *p = 0;
-       szPath = vPath;
-    //}
-	if (strlen(szPath) + strlen("\\desmume.ini") < MAX_PATH)
-	{
-		sprintf(IniName, "%s\\desmume.ini",szPath);
-	} else if (MAX_PATH> strlen(".\\desmume.ini")) {
-		sprintf(IniName, ".\\desmume.ini");
-	} else
-	{
-		memset(IniName,0,MAX_PATH) ;
-	}
-}
-
-void WritePrivateProfileBool(char* appname, char* keyname, bool val, char* file)
-{
-	char temp[256] = "";
-	sprintf(temp, "%d", val?1:0);
-	WritePrivateProfileString(appname, keyname, temp, file);
-}
-
-bool GetPrivateProfileBool(const char* appname, const char* keyname, bool defval, const char* filename)
-{
-	return GetPrivateProfileInt(appname,keyname,defval?1:0,filename) != 0;
-}
-
-void WritePrivateProfileInt(char* appname, char* keyname, int val, char* file)
-{
-	char temp[256] = "";
-	sprintf(temp, "%d", val);
-	WritePrivateProfileString(appname, keyname, temp, file);
-}
-
-#endif
-	
 u8 reverseBitsInByte(u8 x)
 {
 	u8 h = 0;
@@ -101,22 +51,31 @@ u8 reverseBitsInByte(u8 x)
 	return h;
 }
 
-void removeCR(char *buf)
+char *trim(char *s)
 {
-	int l=strlen(buf);
-	for (int i=0; i < l; i++)
-	{
-		if (buf[i]==0x0A) buf[i]=0;
-		if (buf[i]==0x0D) buf[i]=0;
-	}
+	char *ptr = NULL;
+	if (!s) return NULL;
+	if (!*s) return s;
+	
+	for (ptr = s + strlen(s) - 1; (ptr >= s) && isspace(*ptr); ptr--);
+	ptr[1] = '\0';
+	return s;
 }
 
-u32 strlen_ws(char *buf)		// length without last spaces
+char *removeSpecialChars(char *s)
 {
-	if (!strlen(buf)) return 0;
-	for (int i=strlen(buf); i>0; i--)
+	char	*buf = s;
+	if (!s) return NULL;
+	if (!*s) return s;
+
+	for (u32 i = 0; i < strlen(s); i++)
 	{
-		if (buf[i]!=32) return (i-1);		// space
+		if (isspace(s[i]) && (s[i] != 0x20))
+			*buf = 0x20;
+		else
+			*buf = s[i];
+		buf++;
 	}
-	return 0;
+	*buf = 0;
+	return s;
 }
