@@ -23,14 +23,14 @@
 #include "ctrlssdl.h"
 #include "saves.h"
 #include "SPU.h"
+#include "commandline.h"
+#include "NDSSystem.h"
 
 u16 keyboard_cfg[NB_KEYS];
 u16 joypad_cfg[NB_KEYS];
 u16 nbr_joy;
 mouse_status mouse;
 
-extern volatile BOOL execute;
-
 static SDL_Joystick **open_joysticks = NULL;
 
 /* Keypad key names */
@@ -90,6 +90,12 @@ BOOL init_joy( void) {
   int i;
   BOOL joy_init_good = TRUE;
 
+  //user asked for no joystick
+  if(_commandline_linux_nojoy) {
+	  printf("skipping joystick init\n");
+	  return TRUE;
+  }
+
   set_joy_keys(default_joypad_cfg);
 
   if(SDL_InitSubSystem(SDL_INIT_JOYSTICK) == -1)
@@ -190,8 +196,8 @@ static void clear_events( void)
   return;
 }
 
-/* Get and set a new joystick key */
-u16 get_set_joy_key(int index) {
+/* Get pressed joystick key */
+u16 get_joy_key(int index) {
   BOOL done = FALSE;
   SDL_Event event;
   u16 key = joypad_cfg[index];
@@ -214,11 +220,17 @@ u16 get_set_joy_key(int index) {
 
   if( SDL_JoystickEventState(SDL_QUERY) == SDL_ENABLE )
     SDL_JoystickEventState(SDL_IGNORE);
-  joypad_cfg[index] = key;
 
   return key;
 }
 
+/* Get and set a new joystick key */
+u16 get_set_joy_key(int index) {
+  joypad_cfg[index] = get_joy_key(index);
+
+  return joypad_cfg[index];
+}
+
 /* Reset corresponding key and its twin axis key */
 static u16 get_joy_axis_twin(u16 key)
 {
@@ -233,8 +245,8 @@ static u16 get_joy_axis_twin(u16 key)
     }
 }
 
-/* Get and set a new joystick axis */
-void get_set_joy_axis(int index, int index_o) {
+/* Get a new joystick axis */
+u16 get_joy_axis(int index, int index_o) {
   BOOL done = FALSE;
   SDL_Event event;
   u16 key = joypad_cfg[index];
@@ -261,6 +273,14 @@ void get_set_joy_axis(int index, int index_o) {
     }
   if( SDL_JoystickEventState(SDL_QUERY) == SDL_ENABLE )
     SDL_JoystickEventState(SDL_IGNORE);
+
+  return key;
+}
+
+/* Get and set a new joystick axis */
+void get_set_joy_axis(int index, int index_o) {
+  u16 key = get_joy_axis(index, index_o);
+
   /* Update configuration */
   joypad_cfg[index]   = key;
   joypad_cfg[index_o] = joypad_cfg[index];
@@ -292,17 +312,23 @@ void set_mouse_coord(signed long x,signed long y)
 /* Update NDS keypad */
 void update_keypad(u16 keys)
 {
-#ifdef WORDS_BIGENDIAN
-  MMU.ARM9_REG[0x130] = ~keys & 0xFF;
-  MMU.ARM9_REG[0x131] = (~keys >> 8) & 0x3;
-  MMU.ARM7_REG[0x130] = ~keys & 0xFF;
-  MMU.ARM7_REG[0x131] = (~keys >> 8) & 0x3;
-#else
-  ((u16 *)MMU.ARM9_REG)[0x130>>1] = ~keys & 0x3FF;
-  ((u16 *)MMU.ARM7_REG)[0x130>>1] = ~keys & 0x3FF;
-#endif
-  /* Update X and Y buttons */
-  MMU.ARM7_REG[0x136] = ( ~( keys >> 10) & 0x3 ) | (MMU.ARM7_REG[0x136] & ~0x3);
+	NDS_beginProcessingInput();
+	UserButtons& input = NDS_getProcessingUserInput().buttons;
+	input.G = (keys>>12)&1;
+	input.E = (keys>>8)&1;
+	input.W = (keys>>9)&1;
+	input.X = (keys>>10)&1;
+	input.Y = (keys>>11)&1;
+	input.A = (keys>>0)&1;
+	input.B = (keys>>1)&1;
+	input.S = (keys>>3)&1;
+	input.T = (keys>>2)&1;
+	input.U = (keys>>6)&1;
+	input.D = (keys>>7)&1;
+	input.L = (keys>>5)&1;
+	input.R = (keys>>4)&1;
+	input.F = 0;
+	NDS_endProcessingInput();
 }
 
 /* Retrieve current NDS keypad */
