@@ -82,7 +82,9 @@ BOOL init_joy( void);
 void uninit_joy( void);
 void set_joy_keys(const u16 joyCfg[]);
 void set_kb_keys(const u16 kbCfg[]);
+u16 get_joy_key(int index);
 u16 get_set_joy_key(int index);
+u16 get_joy_axis(int index, int index_opp);
 void get_set_joy_axis(int index, int index_opp);
 void update_keypad(u16 keys);
 u16 get_keypad( void);
