@@ -1,8 +1,6 @@
-/*	Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
-    Copyright (C) 2008-2009 DeSmuME team
+/*	gfx3d.cpp
+	Copyright (C) 2006 yopyop
+    Copyright (C) 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -27,6 +25,12 @@
 
 //#define FLUSHMODE_HACK
 
+//---------------
+//TODO TODO TODO TODO
+//make up mind once and for all whether fog, toon, etc. should reside in memory buffers (for easier handling in MMU)
+//if they do, then we need to copy them out in doFlush!!!
+//---------------
+
 #include <algorithm>
 #include <assert.h>
 #include <math.h>
@@ -44,8 +48,8 @@
 #include "NDSSystem.h"
 #include "readwrite.h"
 #include "FIFO.h"
+#include "movie.h" //only for currframecounter which really ought to be moved into the core emu....
 #include <queue>
-//#include "movie.h"
 
 /*
 thoughts on flush timing:
@@ -64,8 +68,6 @@ But since we're not sure how we'll eventually want this, I am leaving it sort of
 in this function: */
 static void gfx3d_doFlush();
 
-#define TESTS_ENABLED 1
-
 #define INVALID_COMMAND 0xFF
 #define UNDEFINED_COMMAND 0xCC
 static const u8 gfx3d_commandTypes[] = {
@@ -156,8 +158,6 @@ public:
 decode:
 			//printf("[%05d] gxf: decoding %08X\n",currFrameCounter,val);
 			
-			bool valid = true;
-			bool gimpyCommand = false;
 			const u8 commands[] = { val&0xFF, (val>>8)&0xFF, (val>>16)&0xFF, (val>>24)&0xFF };
 			const u8 commandTypes[] = { gfx3d_commandTypes[commands[0]], gfx3d_commandTypes[commands[1]],gfx3d_commandTypes[commands[2]], gfx3d_commandTypes[commands[3]] };
 
@@ -261,6 +261,7 @@ using std::max;
 using std::min;
 
 GFX3D gfx3d;
+Viewer3d_State viewer3d_state;
 static GFX3D_Clipper boxtestClipper;
 
 //tables that are provided to anyone
@@ -322,10 +323,10 @@ CACHE_ALIGN u8 gfx3d_convertedScreen[256*192*4];
 
 // Matrix stack handling
 CACHE_ALIGN MatrixStack	mtxStack[4] = {
-	MatrixStack(1), // Projection stack
-	MatrixStack(31), // Coordinate stack
-	MatrixStack(31), // Directional stack
-	MatrixStack(1), // Texture stack
+	MatrixStack(1, 0), // Projection stack
+	MatrixStack(31, 1), // Coordinate stack
+	MatrixStack(31, 2), // Directional stack
+	MatrixStack(1, 3), // Texture stack
 };
 
 int _hack_getMatrixStackLevel(int which) { return mtxStack[which].position; }
@@ -381,8 +382,7 @@ static u32 lightColor[4] = {0,0,0,0};
 static u32 lightDirection[4] = {0,0,0,0};
 //material state:
 static u16 dsDiffuse, dsAmbient, dsSpecular, dsEmission;
-// Shininess
-static float shininessTable[128] = {0};
+//used for indexing the shininess table during parameters to shininess command
 static int shininessInd = 0;
 
 
@@ -390,7 +390,6 @@ static int shininessInd = 0;
 //these dont need to go into the savestate. they can be regenerated from HW registers
 //from polygonattr:
 static unsigned int cullingMask=0;
-static u8 colorAlpha=0;
 static u32 envMode=0;
 static u32 lightMask=0;
 //other things:
@@ -502,8 +501,9 @@ void gfx3d_init()
 
 void gfx3d_reset()
 {
-	gfx3d = GFX3D();
-
+	//if this doesn't work on the xbox, we need to find out why.
+	reconstruct(&gfx3d);
+	
 	gxf_hardware.reset();
 
 	control = 0;
@@ -565,13 +565,15 @@ void gfx3d_reset()
 
 	memset(gfx3d_convertedScreen,0,sizeof(gfx3d_convertedScreen));
 
-	gfx3d.clearDepth = gfx3d_extendDepth_15_to_24(0x7FFF);
+	gfx3d.state.clearDepth = gfx3d_extendDepth_15_to_24(0x7FFF);
 	
 	clInd2 = 0;
 	isSwapBuffers = FALSE;
 
 	GFX_PIPEclear();
 	GFX_FIFOclear();
+
+	gfx3d.state.invalidateToon = true;
 }
 
 
@@ -790,19 +792,9 @@ static void gfx3d_glMatrixMode(u32 v)
 
 static void gfx3d_glPushMatrix()
 {
-	//u32 gxstat = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
 
-	if (mtxStack[mymode].position > mtxStack[mymode].size)
-	{
-		MMU_new.gxstat.se = 1;
-		//gxstat |= (1<<15);
-		return;
-	}
-
-	//gxstat &= 0xFFFF00FF;
-
 	MatrixStackPushMatrix(&mtxStack[mymode], mtxCurrent[mymode]);
 
 	GFX_DELAY(17);
@@ -810,36 +802,30 @@ static void gfx3d_glPushMatrix()
 	if(mymode==2)
 		MatrixStackPushMatrix (&mtxStack[1], mtxCurrent[1]);
 
-	//gxstat |= ((mtxStack[0].position << 13) | (mtxStack[1].position << 8));
+	MMU_new.gxstat.sb = 1; // set busy
 }
 
 static void gfx3d_glPopMatrix(s32 i)
 {
+	// The stack has only one level (at address 0) in projection mode, 
+	// in that mode, the parameter value is ignored, the offset is always +1 in that mode.
+	if (mode == 0) i = 1;
+
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
 
 	//6 bits, sign extended
 	//this was necessary to fix sims apartment pets
-	i = (i<<26)>>26;
-	
-	if (i > mtxStack[mymode].position)
-	{
-		//was commented out before zero sep modifications
-		//handled in matrix now
-		//MMU_new.gxstat.se = 1;
-	//	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
-	//	return;
-	}
-	
+	//i = (i<<26)>>26;
 
-	MatrixCopy(mtxCurrent[mymode], MatrixStackPopMatrix (&mtxStack[mymode], i));
+	MatrixStackPopMatrix((float*)mtxCurrent[mymode], &mtxStack[mymode], i);
 
 	GFX_DELAY(36);
 
 	if (mymode == 2)
-		MatrixCopy(mtxCurrent[1], MatrixStackPopMatrix (&mtxStack[1], i));
+		MatrixStackPopMatrix((float*)mtxCurrent[1], &mtxStack[1], i);
 
-	//gxstat |= ((mtxStack[0].position << 13) | (mtxStack[1].position << 8));
+	MMU_new.gxstat.sb = 1; // set busy
 }
 
 static void gfx3d_glStoreMatrix(u32 v)
@@ -899,7 +885,7 @@ static void gfx3d_glLoadIdentity()
 
 static BOOL gfx3d_glLoadMatrix4x4(s32 v)
 {
-	mtxCurrent[mode][ML4x4ind] = (float)v;
+	mtxCurrent[mode][ML4x4ind] = (float)((v<<4)>>4);
 
 	++ML4x4ind;
 	if(ML4x4ind<16) return FALSE;
@@ -918,7 +904,7 @@ static BOOL gfx3d_glLoadMatrix4x4(s32 v)
 
 static BOOL gfx3d_glLoadMatrix4x3(s32 v)
 {
-	mtxCurrent[mode][ML4x3ind] = (float)v;
+	mtxCurrent[mode][ML4x3ind] = (float)((v<<4)>>4);
 
 	ML4x3ind++;
 	if((ML4x3ind & 0x03) == 3) ML4x3ind++;
@@ -941,7 +927,7 @@ static BOOL gfx3d_glLoadMatrix4x3(s32 v)
 
 static BOOL gfx3d_glMultMatrix4x4(s32 v)
 {
-	mtxTemporal[MM4x4ind] = (float)v;
+	mtxTemporal[MM4x4ind] = (float)((v<<4)>>4);
 
 	MM4x4ind++;
 	if(MM4x4ind<16) return FALSE;
@@ -967,7 +953,7 @@ static BOOL gfx3d_glMultMatrix4x4(s32 v)
 
 static BOOL gfx3d_glMultMatrix4x3(s32 v)
 {
-	mtxTemporal[MM4x3ind] = (float)v;
+	mtxTemporal[MM4x3ind] = (float)((v<<4)>>4);
 
 	MM4x3ind++;
 	if((MM4x3ind & 0x03) == 3) MM4x3ind++;
@@ -999,7 +985,7 @@ static BOOL gfx3d_glMultMatrix4x3(s32 v)
 
 static BOOL gfx3d_glMultMatrix3x3(s32 v)
 {
-	mtxTemporal[MM3x3ind] = (float)v;
+	mtxTemporal[MM3x3ind] = (float)((v<<4)>>4);
 
 
 	MM3x3ind++;
@@ -1147,7 +1133,7 @@ static void gfx3d_glNormal(u32 v)
 			if(dsSpecular & 0x8000)
 			{
 				int shininessIndex = (int)(shininessLevel * 128);
-				if(shininessIndex >= (int)ARRAY_SIZE(shininessTable)) {
+				if(shininessIndex >= (int)ARRAY_SIZE(gfx3d.state.shininessTable)) {
 					//we can't print this right now, because when a game triggers this it triggers it _A_LOT_
 					//so wait until we have per-frame diagnostics.
 					//this was tested using Princess Debut (US) after proceeding through the intro and getting the tiara.
@@ -1159,7 +1145,7 @@ static void gfx3d_glNormal(u32 v)
 					//PROGINFO("ERROR: shininess table out of bounds.\n  maybe an emulator error; maybe a non-unit normal; setting to 0\n");
 					shininessIndex = 0;
 				}
-				shininessLevel = shininessTable[shininessIndex];
+				shininessLevel = gfx3d.state.shininessTable[shininessIndex];
 			}
 
 			for(c = 0; c < 3; c++)
@@ -1345,10 +1331,10 @@ static void gfx3d_glLightColor (u32 v)
 
 static BOOL gfx3d_glShininess (u32 val)
 {
-	shininessTable[shininessInd++] = ((val & 0xFF) / 256.0f);
-	shininessTable[shininessInd++] = (((val >> 8) & 0xFF) / 256.0f);
-	shininessTable[shininessInd++] = (((val >> 16) & 0xFF) / 256.0f);
-	shininessTable[shininessInd++] = (((val >> 24) & 0xFF) / 256.0f);
+	gfx3d.state.shininessTable[shininessInd++] = ((val & 0xFF) / 256.0f);
+	gfx3d.state.shininessTable[shininessInd++] = (((val >> 8) & 0xFF) / 256.0f);
+	gfx3d.state.shininessTable[shininessInd++] = (((val >> 16) & 0xFF) / 256.0f);
+	gfx3d.state.shininessTable[shininessInd++] = (((val >> 24) & 0xFF) / 256.0f);
 
 	if (shininessInd < 128) return FALSE;
 	shininessInd = 0;
@@ -1450,8 +1436,8 @@ static BOOL gfx3d_glBoxTest(u32 v)
 
 	//setup the clipper
 	GFX3D_Clipper::TClippedPoly tempClippedPoly;
-	boxtestClipper.clippedPolyCounter = 0;
 	boxtestClipper.clippedPolys = &tempClippedPoly;
+	boxtestClipper.reset();
 
 	////-----------------------------
 	////awesome hack:
@@ -1484,7 +1470,6 @@ static BOOL gfx3d_glBoxTest(u32 v)
 		//yuck.. cant use the sse2 accelerated ones because vert.coords is not cache aligned or something
 		//i dunno
 		
-		void _NOSSE_MatrixMultVec4x4 (const float *matrix, float *vecPtr);
 		_NOSSE_MatrixMultVec4x4(mtxCurrent[1],verts[i].coord);
 		_NOSSE_MatrixMultVec4x4(mtxCurrent[0],verts[i].coord);
 	}
@@ -1500,7 +1485,7 @@ static BOOL gfx3d_glBoxTest(u32 v)
 			&verts[poly->vertIndexes[3]]
 		};
 
-		boxtestClipper.clipPoly(poly,vertTable);
+		boxtestClipper.clipPoly<false>(poly,vertTable);
 		
 		//if any portion of this poly was retained, then the test passes.
 		if(boxtestClipper.clippedPolyCounter>0) {
@@ -1523,9 +1508,7 @@ static BOOL gfx3d_glPosTest(u32 v)
 	//this is apparently tested by transformers decepticons and ultimate spiderman
 
 	//printf("POSTEST\n");
-#ifdef TESTS_ENABLED
 	MMU_new.gxstat.tb = 1;
-#endif
 
 	PTcoords[PTind++] = float16table[v & 0xFFFF];
 	PTcoords[PTind++] = float16table[v >> 16];
@@ -1549,18 +1532,20 @@ static void gfx3d_glVecTest(u32 v)
 {
 	GFX_DELAY(5);
 
-	printf("VECTEST\n");
+	//this is tested by phoenix wright in its evidence inspector modelviewer
+	//i am not sure exactly what it is doing, maybe it is testing to ensure
+	//that the normal vector for the point of interest is camera-facing.
 
 	CACHE_ALIGN float normal[4] = { normalTable[v&1023],
 						normalTable[(v>>10)&1023],
 						normalTable[(v>>20)&1023],
-						1};
+						0};
 
 	MatrixMultVec4x4(mtxCurrent[2], normal);
 
-	s16 x = (s16)(normal[0]);
-	s16 y = (s16)(normal[1]);
-	s16 z = (s16)(normal[2]);
+	s16 x = (s16)(normal[0]*4096);
+	s16 y = (s16)(normal[1]*4096);
+	s16 z = (s16)(normal[2]*4096);
 
 	MMU_new.gxstat.tb = 0;		// clear busy
 	T1WriteWord(MMU.MMU_MEM[0][0x40], 0x630, x);
@@ -1580,25 +1565,20 @@ void VIEWPORT::decode(u32 v)
 	height = ((v>>24)+1)-((v>>8)&0xFF);
 }
 
-void gfx3d_glClearColor(u32 v)
-{
-	gfx3d.clearColor = v;
-}
-
 void gfx3d_glFogColor(u32 v)
 {
-	gfx3d.fogColor = v;
+	gfx3d.state.fogColor = v;
 }
 
 void gfx3d_glFogOffset (u32 v)
 {
-	gfx3d.fogOffset = (v&0x7fff);
+	gfx3d.state.fogOffset = (v&0x7fff);
 }
 
 void gfx3d_glClearDepth(u32 v)
 {
 	v &= 0x7FFF;
-	gfx3d.clearDepth = gfx3d_extendDepth_15_to_24(v);
+	gfx3d.state.clearDepth = gfx3d_extendDepth_15_to_24(v);
 }
 
 // Ignored for now
@@ -1620,14 +1600,16 @@ int gfx3d_GetNumVertex()
 
 void gfx3d_UpdateToonTable(u8 offset, u16 val)
 {
-	gfx3d.u16ToonTable[offset] =  val;
+	gfx3d.state.rgbToonTable[offset] = RGB15TO32(val, 255);
+	gfx3d.state.invalidateToon = true;
 }
 
 void gfx3d_UpdateToonTable(u8 offset, u32 val)
 {
 	//C.O.P. sets toon table via this method
-	gfx3d.u16ToonTable[offset] = val & 0xFFFF;
-	gfx3d.u16ToonTable[offset+1] = val >> 16;
+	gfx3d.state.rgbToonTable[offset] = RGB15TO32(val & 0xFFFF, 255);
+	gfx3d.state.rgbToonTable[offset+1] = RGB15TO32(val >> 16, 255);
+	gfx3d.state.invalidateToon = true;
 }
 
 s32 gfx3d_GetClipMatrix (unsigned int index)
@@ -1646,17 +1628,9 @@ s32 gfx3d_GetDirectionalMatrix (unsigned int index)
 	return (s32)(mtxCurrent[2][_index]*(1<<12));
 }
 
-void gfx3d_ClearStack()
-{
-	MatrixStackSetStackPosition(&mtxStack[0], -5);
-	//MatrixStackSetStackPosition(&mtxStack[1], -55);
-	//MatrixStackSetStackPosition(&mtxStack[2], -55); //?
-	MatrixStackSetStackPosition(&mtxStack[3], -5);
-}
-
 void gfx3d_glAlphaFunc(u32 v)
 {
-	gfx3d.alphaTestRef = v&31;
+	gfx3d.state.alphaTestRef = v&31;
 }
 
 unsigned int gfx3d_glGetPosRes(unsigned int index)
@@ -1664,20 +1638,137 @@ unsigned int gfx3d_glGetPosRes(unsigned int index)
 	return (unsigned int)(PTcoords[index] * 4096.0f);
 }
 
-unsigned short gfx3d_glGetVecRes(unsigned int index)
+void log3D(u8 cmd, u32 param)
 {
-	//INFO("NDS_glGetVecRes\n");
-	return 0;
+	INFO("3D command 0x%02X: ", cmd);
+	switch (cmd)
+		{
+			case 0x10:		// MTX_MODE - Set Matrix Mode (W)
+				printf("MTX_MODE(%08X)", param);
+			break;
+			case 0x11:		// MTX_PUSH - Push Current Matrix on Stack (W)
+				printf("MTX_PUSH()\t");
+			break;
+			case 0x12:		// MTX_POP - Pop Current Matrix from Stack (W)
+				printf("MTX_POP(%08X)", param);
+			break;
+			case 0x13:		// MTX_STORE - Store Current Matrix on Stack (W)
+				printf("MTX_STORE(%08X)", param);
+			break;
+			case 0x14:		// MTX_RESTORE - Restore Current Matrix from Stack (W)
+				printf("MTX_RESTORE(%08X)", param);
+			break;
+			case 0x15:		// MTX_IDENTITY - Load Unit Matrix to Current Matrix (W)
+				printf("MTX_IDENTIFY()\t");
+			break;
+			case 0x16:		// MTX_LOAD_4x4 - Load 4x4 Matrix to Current Matrix (W)
+				printf("MTX_LOAD_4x4(%08X)", param);
+			break;
+			case 0x17:		// MTX_LOAD_4x3 - Load 4x3 Matrix to Current Matrix (W)
+				printf("MTX_LOAD_4x3(%08X)", param);
+			break;
+			case 0x18:		// MTX_MULT_4x4 - Multiply Current Matrix by 4x4 Matrix (W)
+				printf("MTX_MULT_4x4(%08X)", param);
+			break;
+			case 0x19:		// MTX_MULT_4x3 - Multiply Current Matrix by 4x3 Matrix (W)
+				printf("MTX_MULT_4x3(%08X)", param);
+			break;
+			case 0x1A:		// MTX_MULT_3x3 - Multiply Current Matrix by 3x3 Matrix (W)
+				printf("MTX_MULT_3x3(%08X)", param);
+			break;
+			case 0x1B:		// MTX_SCALE - Multiply Current Matrix by Scale Matrix (W)
+				printf("MTX_SCALE(%08X)", param);
+			break;
+			case 0x1C:		// MTX_TRANS - Mult. Curr. Matrix by Translation Matrix (W)
+				printf("MTX_TRANS(%08X)", param);
+			break;
+			case 0x20:		// COLOR - Directly Set Vertex Color (W)
+				printf("COLOR(%08X)", param);
+			break;
+			case 0x21:		// NORMAL - Set Normal Vector (W)
+				printf("NORMAL(%08X)", param);
+			break;
+			case 0x22:		// TEXCOORD - Set Texture Coordinates (W)
+				printf("TEXCOORD(%08X)", param);
+			break;
+			case 0x23:		// VTX_16 - Set Vertex XYZ Coordinates (W)
+				printf("VTX_16(%08X)", param);
+			break;
+			case 0x24:		// VTX_10 - Set Vertex XYZ Coordinates (W)
+				printf("VTX_10(%08X)", param);
+			break;
+			case 0x25:		// VTX_XY - Set Vertex XY Coordinates (W)
+				printf("VTX_XY(%08X)", param);
+			break;
+			case 0x26:		// VTX_XZ - Set Vertex XZ Coordinates (W)
+				printf("VTX_XZ(%08X)", param);
+			break;
+			case 0x27:		// VTX_YZ - Set Vertex YZ Coordinates (W)
+				printf("VTX_YZ(%08X)", param);
+			break;
+			case 0x28:		// VTX_DIFF - Set Relative Vertex Coordinates (W)
+				printf("VTX_DIFF(%08X)", param);
+			break;
+			case 0x29:		// POLYGON_ATTR - Set Polygon Attributes (W)
+				printf("POLYGON_ATTR(%08X)", param);
+			break;
+			case 0x2A:		// TEXIMAGE_PARAM - Set Texture Parameters (W)
+				printf("TEXIMAGE_PARAM(%08X)", param);
+			break;
+			case 0x2B:		// PLTT_BASE - Set Texture Palette Base Address (W)
+				printf("PLTT_BASE(%08X)", param);
+			break;
+			case 0x30:		// DIF_AMB - MaterialColor0 - Diffuse/Ambient Reflect. (W)
+				printf("DIF_AMB(%08X)", param);
+			break;
+			case 0x31:		// SPE_EMI - MaterialColor1 - Specular Ref. & Emission (W)
+				printf("SPE_EMI(%08X)", param);
+			break;
+			case 0x32:		// LIGHT_VECTOR - Set Light's Directional Vector (W)
+				printf("LIGHT_VECTOR(%08X)", param);
+			break;
+			case 0x33:		// LIGHT_COLOR - Set Light Color (W)
+				printf("LIGHT_COLOR(%08X)", param);
+			break;
+			case 0x34:		// SHININESS - Specular Reflection Shininess Table (W)
+				printf("SHININESS(%08X)", param);
+			break;
+			case 0x40:		// BEGIN_VTXS - Start of Vertex List (W)
+				printf("BEGIN_VTXS(%08X)", param);
+			break;
+			case 0x41:		// END_VTXS - End of Vertex List (W)
+				printf("END_VTXS()\t");
+			break;
+			case 0x50:		// SWAP_BUFFERS - Swap Rendering Engine Buffer (W)
+				printf("SWAP_BUFFERS(%08X)", param);
+			break;
+			case 0x60:		// VIEWPORT - Set Viewport (W)
+				printf("VIEWPORT(%08X)", param);
+			break;
+			case 0x70:		// BOX_TEST - Test if Cuboid Sits inside View Volume (W)
+				printf("BOX_TEST(%08X)", param);
+			break;
+			case 0x71:		// POS_TEST - Set Position Coordinates for Test (W)
+				printf("POS_TEST(%08X)", param);
+			break;
+			case 0x72:		// VEC_TEST - Set Directional Vector for Test (W)
+				printf("VEC_TEST(%08X)", param);
+			break;
+			default:
+				INFO("!!! Unknown(%08X)", param);
+			break;
+		}
+		printf("\t\t(FIFO size %i)\n", gxFIFO.size);
 }
 
 //#define _3D_LOG_EXEC
 static void gfx3d_execute(u8 cmd, u32 param)
 {
-	//printf("*** gxFIFO: exec 0x%02X, size %03i\n", cmd, gxFIFO.size);
 #ifdef _3D_LOG_EXEC
-	u32 gxstat2 = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
-	INFO("*** gxFIFO: exec 0x%02X, tail %03i, gxstat 0x%08X (timer %i)\n", cmd, gxFIFO.tail, gxstat2, nds_timer);
+	log3D(cmd, param);
 #endif
+
+	MMU_new.gxstat.sb = 0;	// clear stack busy flag
 	switch (cmd)
 	{
 		case 0x10:		// MTX_MODE - Set Matrix Mode (W)
@@ -1859,8 +1950,7 @@ void gfx3d_execute3D()
 void gfx3d_glFlush(u32 v)
 {
 	//printf("-------------FLUSH------------- (vcount=%d\n",nds.VCount);
-	gfx3d.sortmode = BIT0(v);
-	gfx3d.wbuffer = BIT1(v);
+	gfx3d.state.pendingFlushCommand = v;
 #if 0
 	if (isSwapBuffers)
 	{
@@ -1888,8 +1978,8 @@ static bool gfx3d_ysort_compare(int num1, int num2)
 	//also the buttons in the knights in the nightmare frontend depend on this and the perspective division
 	if (poly1.maxy < poly2.maxy) return true;
 	if (poly1.maxy > poly2.maxy) return false;
-	if (poly1.miny > poly2.miny) return true;
-	if (poly1.miny < poly2.miny) return false;
+	if (poly1.miny < poly2.miny) return true;
+	if (poly1.miny > poly2.miny) return false;
 	//notably, the main shop interface in harvest moon will not have a correct RTN button
 	//i think this is due to a math error rounding its position to one pixel too high and it popping behind
 	//the bar that it sits on.
@@ -1911,17 +2001,23 @@ static void gfx3d_doFlush()
 	gfx3d.vertlist = vertlist;
 
 	//and also our current render state
-	if(BIT1(control)) gfx3d.shading = GFX3D::HIGHLIGHT;
-	else gfx3d.shading = GFX3D::TOON;
-	gfx3d.enableTexturing = BIT0(control);
-	gfx3d.enableAlphaTest = BIT2(control);
-	gfx3d.enableAlphaBlending = BIT3(control);
-	gfx3d.enableAntialiasing = BIT4(control);
-	gfx3d.enableEdgeMarking = BIT5(control);
-	gfx3d.enableFogAlphaOnly = BIT6(control);
-	gfx3d.enableFog = BIT7(control);
-	gfx3d.enableClearImage = BIT14(control);
-	gfx3d.fogShift = (control>>8)&0xF;
+	if(BIT1(control)) gfx3d.state.shading = GFX3D_State::HIGHLIGHT;
+	else gfx3d.state.shading = GFX3D_State::TOON;
+	gfx3d.state.enableTexturing = BIT0(control);
+	gfx3d.state.enableAlphaTest = BIT2(control);
+	gfx3d.state.enableAlphaBlending = BIT3(control);
+	gfx3d.state.enableAntialiasing = BIT4(control);
+	gfx3d.state.enableEdgeMarking = BIT5(control);
+	gfx3d.state.enableFogAlphaOnly = BIT6(control);
+	gfx3d.state.enableFog = BIT7(control);
+	gfx3d.state.enableClearImage = BIT14(control);
+	gfx3d.state.fogShift = (control>>8)&0xF;
+	gfx3d.state.sortmode = BIT0(gfx3d.state.activeFlushCommand);
+	gfx3d.state.wbuffer = BIT1(gfx3d.state.activeFlushCommand);
+
+	gfx3d.renderState = gfx3d.state;
+	
+	gfx3d.state.activeFlushCommand = gfx3d.state.pendingFlushCommand;
 
 	int polycount = polylist->count;
 
@@ -1935,17 +2031,18 @@ static void gfx3d_doFlush()
 		POLY &poly = polylist->list[i];
 		float verty = vertlist->list[poly.vertIndexes[0]].y;
 		float vertw = vertlist->list[poly.vertIndexes[0]].w;
-		verty = (verty+vertw)/(2*vertw);
+		verty = 1.0f-(verty+vertw)/(2*vertw);
 		poly.miny = poly.maxy = verty;
 
 		for(int j=1; j<poly.type; j++)
 		{
 			verty = vertlist->list[poly.vertIndexes[j]].y;
 			vertw = vertlist->list[poly.vertIndexes[j]].w;
-			verty = (verty+vertw)/(2*vertw);
+			verty = 1.0f-(verty+vertw)/(2*vertw);
 			poly.miny = min(poly.miny, verty);
 			poly.maxy = max(poly.maxy, verty);
 		}
+
 	}
 
 	//we need to sort the poly list with alpha polys last
@@ -1954,31 +2051,41 @@ static void gfx3d_doFlush()
 	for(int i=0;i<polycount;i++) {
 		POLY &poly = polylist->list[i];
 		if(!poly.isTranslucent())
-			gfx3d.indexlist[ctr++] = i;
+			gfx3d.indexlist.list[ctr++] = i;
 	}
 	int opaqueCount = ctr;
 	//then look for translucent polys
 	for(int i=0;i<polycount;i++) {
 		POLY &poly = polylist->list[i];
 		if(poly.isTranslucent())
-			gfx3d.indexlist[ctr++] = i;
+			gfx3d.indexlist.list[ctr++] = i;
 	}
 
 	//now we have to sort the opaque polys by y-value.
 	//(test case: harvest moon island of happiness character cretor UI)
 	//should this be done after clipping??
-	std::sort(gfx3d.indexlist, gfx3d.indexlist + opaqueCount, gfx3d_ysort_compare);
+	std::sort(gfx3d.indexlist.list, gfx3d.indexlist.list + opaqueCount, gfx3d_ysort_compare);
 	
-	if(!gfx3d.sortmode)
+	if(!gfx3d.state.sortmode)
 	{
 		//if we are autosorting translucent polys, we need to do this also
 		//TODO - this is unverified behavior. need a test case
-		std::sort(gfx3d.indexlist + opaqueCount, gfx3d.indexlist + polycount, gfx3d_ysort_compare);
+		std::sort(gfx3d.indexlist.list + opaqueCount, gfx3d.indexlist.list + polycount, gfx3d_ysort_compare);
 	}
 
 	//switch to the new lists
 	twiddleLists();
 
+	if(driver->view3d->IsRunning())
+	{
+		viewer3d_state.frameNumber = currFrameCounter;
+		viewer3d_state.state = gfx3d.state;
+		viewer3d_state.polylist = *gfx3d.polylist;
+		viewer3d_state.vertlist = *gfx3d.vertlist;
+		viewer3d_state.indexlist = gfx3d.indexlist;
+		driver->view3d->NewFrame();
+	}
+
 	drawPending = TRUE;
 }
 
@@ -2190,27 +2297,29 @@ SFORMAT SF_GFX3D[]={
 	{ "GMEM", 2, 1, &dsEmission},
 	{ "GFLP", 4, 1, &flushPending},
 	{ "GDRP", 4, 1, &drawPending},
-	{ "GSET", 4, 1, &gfx3d.enableTexturing},
-	{ "GSEA", 4, 1, &gfx3d.enableAlphaTest},
-	{ "GSEB", 4, 1, &gfx3d.enableAlphaBlending},
-	{ "GSEX", 4, 1, &gfx3d.enableAntialiasing},
-	{ "GSEE", 4, 1, &gfx3d.enableEdgeMarking},
-	{ "GSEC", 4, 1, &gfx3d.enableClearImage},
-	{ "GSEF", 4, 1, &gfx3d.enableFog},
-	{ "GSEO", 4, 1, &gfx3d.enableFogAlphaOnly},
-	{ "GFSH", 4, 1, &gfx3d.fogShift},
-	{ "GSSH", 4, 1, &gfx3d.shading},
-	{ "GSWB", 4, 1, &gfx3d.wbuffer},
-	{ "GSSM", 4, 1, &gfx3d.sortmode},
-	{ "GSAR", 1, 1, &gfx3d.alphaTestRef},
+	{ "GSET", 4, 1, &gfx3d.state.enableTexturing},
+	{ "GSEA", 4, 1, &gfx3d.state.enableAlphaTest},
+	{ "GSEB", 4, 1, &gfx3d.state.enableAlphaBlending},
+	{ "GSEX", 4, 1, &gfx3d.state.enableAntialiasing},
+	{ "GSEE", 4, 1, &gfx3d.state.enableEdgeMarking},
+	{ "GSEC", 4, 1, &gfx3d.state.enableClearImage},
+	{ "GSEF", 4, 1, &gfx3d.state.enableFog},
+	{ "GSEO", 4, 1, &gfx3d.state.enableFogAlphaOnly},
+	{ "GFSH", 4, 1, &gfx3d.state.fogShift},
+	{ "GSSH", 4, 1, &gfx3d.state.shading},
+	{ "GSWB", 4, 1, &gfx3d.state.wbuffer},
+	{ "GSSM", 4, 1, &gfx3d.state.sortmode},
+	{ "GSAR", 1, 1, &gfx3d.state.alphaTestRef},
 	{ "GSVP", 4, 1, &viewport},
-	{ "GSCC", 4, 1, &gfx3d.clearColor},
-	{ "GSCD", 4, 1, &gfx3d.clearDepth},
-	{ "GSFC", 4, 4, &gfx3d.fogColor},
-	{ "GSFO", 4, 1, &gfx3d.fogOffset},
-	{ "GST2", 2, 32, gfx3d.u16ToonTable},
-	{ "GSST", 4, 128, shininessTable},
+	{ "GSCC", 4, 1, &gfx3d.state.clearColor},
+	{ "GSCD", 4, 1, &gfx3d.state.clearDepth},
+	{ "GSFC", 4, 4, &gfx3d.state.fogColor},
+	{ "GSFO", 4, 1, &gfx3d.state.fogOffset},
+	{ "GST3", 4, 32, gfx3d.state.rgbToonTable},
+	{ "GSST", 4, 128, &gfx3d.state.shininessTable[0]},
 	{ "GSSI", 4, 1, &shininessInd},
+	{ "GSAF", 4, 1, &gfx3d.state.activeFlushCommand},
+	{ "GSPF", 4, 1, &gfx3d.state.pendingFlushCommand},
 	//------------------------
 	{ "GTST", 4, 1, &triStripToggle},
 	{ "GTVC", 4, 1, &tempVertInfo.count},
@@ -2326,7 +2435,7 @@ static T interpolate(const float ratio, const T& x0, const T& x1) {
 
 //http://www.cs.berkeley.edu/~ug/slide/pipeline/assignments/as6/discussion.shtml
 #ifdef OPTIMIZED_CLIPPING_METHOD
-template<int coord, int which> static FORCEINLINE VERT clipPoint(VERT* inside, VERT* outside)
+template<int coord, int which> static FORCEINLINE VERT clipPoint(bool hirez, VERT* inside, VERT* outside)
 #else
 static FORCEINLINE VERT clipPoint(VERT* inside, VERT* outside, int coord, int which)
 #endif
@@ -2353,7 +2462,8 @@ static FORCEINLINE VERT clipPoint(VERT* inside, VERT* outside, int coord, int wh
 	INTERP(coord[0]); INTERP(coord[1]); INTERP(coord[2]); INTERP(coord[3]);
 	INTERP(texcoord[0]); INTERP(texcoord[1]);
 
-	if(CommonSettings.GFX3D_HighResolutionInterpolateColor)
+	//if(CommonSettings.GFX3D_HighResolutionInterpolateColor)
+	if(hirez)
 	{
 		INTERP(fcolor[0]); INTERP(fcolor[1]); INTERP(fcolor[2]);
 	}
@@ -2392,20 +2502,20 @@ public:
 		m_next.init(verts);
 	}
 
-	void clipVert(VERT* vert)
+	void clipVert(bool hirez, VERT* vert)
 	{
 		if(m_prevVert)
-			this->clipSegmentVsPlane(m_prevVert, vert);
+			this->clipSegmentVsPlane(hirez, m_prevVert, vert);
 		else
 			m_firstVert = vert;
 		m_prevVert = vert;
 	}
 
 	// closes the loop and returns the number of clipped output verts
-	int finish()
+	int finish(bool hirez)
 	{
-		this->clipVert(m_firstVert);
-		return m_next.finish();
+		this->clipVert(hirez, m_firstVert);
+		return m_next.finish(hirez);
 	}
 
 private:
@@ -2414,7 +2524,7 @@ private:
 	VERT* m_firstVert;
 	Next& m_next;
 
-	FORCEINLINE void clipSegmentVsPlane(VERT* vert0, VERT* vert1)
+	FORCEINLINE void clipSegmentVsPlane(bool hirez, VERT* vert0, VERT* vert1)
 	{
 		float* vert0coord = vert0->coord;
 		float* vert1coord = vert1->coord;
@@ -2443,7 +2553,7 @@ private:
 		if(!out0 && !out1) 
 		{
 			CLIPLOG(" both inside\n");
-			m_next.clipVert(vert1);
+			m_next.clipVert(hirez,vert1);
 		}
 
 		//exiting volume: insert the clipped point
@@ -2451,17 +2561,17 @@ private:
 		{
 			CLIPLOG(" exiting\n");
 			assert((u32)numScratchClipVerts < MAX_SCRATCH_CLIP_VERTS);
-			scratchClipVerts[numScratchClipVerts] = clipPoint<coord, which>(vert0,vert1);
-			m_next.clipVert(&scratchClipVerts[numScratchClipVerts++]);
+			scratchClipVerts[numScratchClipVerts] = clipPoint<coord, which>(hirez,vert0,vert1);
+			m_next.clipVert(hirez,&scratchClipVerts[numScratchClipVerts++]);
 		}
 
 		//entering volume: insert clipped point and the next (interior) point
 		if(out0 && !out1) {
 			CLIPLOG(" entering\n");
 			assert((u32)numScratchClipVerts < MAX_SCRATCH_CLIP_VERTS);
-			scratchClipVerts[numScratchClipVerts] = clipPoint<coord, which>(vert1,vert0);
-			m_next.clipVert(&scratchClipVerts[numScratchClipVerts++]);
-			m_next.clipVert(vert1);
+			scratchClipVerts[numScratchClipVerts] = clipPoint<coord, which>(hirez,vert1,vert0);
+			m_next.clipVert(hirez,&scratchClipVerts[numScratchClipVerts++]);
+			m_next.clipVert(hirez,vert1);
 		}
 	}
 };
@@ -2474,13 +2584,13 @@ public:
 		m_nextDestVert = verts;
 		m_numVerts = 0;
 	}
-	void clipVert(VERT* vert)
+	void clipVert(bool hirez, VERT* vert)
 	{
 		assert((u32)m_numVerts < MAX_CLIPPED_VERTS);
 		*m_nextDestVert++ = *vert;
 		m_numVerts++;
 	}
-	int finish()
+	int finish(bool hirez)
 	{
 		return m_numVerts;
 	}
@@ -2499,7 +2609,7 @@ typedef ClipperPlane<1,-1,Stage4> Stage3;        static Stage3 clipper3 (clipper
 typedef ClipperPlane<0, 1,Stage3> Stage2;        static Stage2 clipper2 (clipper3); // right plane
 typedef ClipperPlane<0,-1,Stage2> Stage1;        static Stage1 clipper  (clipper2); // left plane
 
-void GFX3D_Clipper::clipPoly(POLY* poly, VERT** verts)
+template<bool hirez> void GFX3D_Clipper::clipPoly(POLY* poly, VERT** verts)
 {
 	CLIPLOG("==Begin poly==\n");
 
@@ -2508,8 +2618,8 @@ void GFX3D_Clipper::clipPoly(POLY* poly, VERT** verts)
 
 	clipper.init(clippedPolys[clippedPolyCounter].clipVerts);
 	for(int i=0;i<type;i++)
-		clipper.clipVert(verts[i]);
-	int outType = clipper.finish();
+		clipper.clipVert(hirez,verts[i]);
+	int outType = clipper.finish(hirez);
 
 	assert((u32)outType < MAX_CLIPPED_VERTS);
 	if(outType < 3)
@@ -2524,6 +2634,9 @@ void GFX3D_Clipper::clipPoly(POLY* poly, VERT** verts)
 		clippedPolyCounter++;
 	}
 }
+//these templates needed to be instantiated manually
+template void GFX3D_Clipper::clipPoly<true>(POLY* poly, VERT** verts);
+template void GFX3D_Clipper::clipPoly<false>(POLY* poly, VERT** verts);
 
 void GFX3D_Clipper::clipSegmentVsPlane(VERT** verts, const int coord, int which)
 {
