@@ -1,8 +1,6 @@
-/*	Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com 
-
-	Copyright (C) 2008-2009 DeSmuME team
+/*	gfx3d.h
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -251,6 +249,10 @@ struct VERTLIST {
 	int count;
 };
 
+struct INDEXLIST {
+	int list[POLYLIST_SIZE];
+};
+
 
 struct VIEWPORT {
 	int x, y, width, height;
@@ -274,12 +276,13 @@ public:
 	};
 
 	//the entry point for poly clipping
-	void clipPoly(POLY* poly, VERT** verts);
+	template<bool hirez> void clipPoly(POLY* poly, VERT** verts);
 
 	//the output of clipping operations goes into here.
 	//be sure you init it before clipping!
 	TClippedPoly *clippedPolys;
 	int clippedPolyCounter;
+	void reset() { clippedPolyCounter=0; }
 
 private:
 	TClippedPoly tempClippedPoly;
@@ -289,9 +292,9 @@ private:
 };
 
 //used to communicate state to the renderer
-struct GFX3D
+struct GFX3D_State
 {
-	GFX3D()
+	GFX3D_State()
 		: enableTexturing(true)
 		, enableAlphaTest(true)
 		, enableAlphaBlending(true)
@@ -300,37 +303,35 @@ struct GFX3D
 		, enableClearImage(false)
 		, enableFog(false)
 		, enableFogAlphaOnly(false)
-		, fogShift(0)
 		, shading(TOON)
-		, polylist(0)
-		, vertlist(0)
 		, alphaTestRef(0)
+		, activeFlushCommand(0)
+		, pendingFlushCommand(0)
 		, clearDepth(1)
 		, clearColor(0)
 		, fogColor(0)
 		, fogOffset(0)
-		, frameCtr(0)
-		, frameCtrRaw(0)
+		, fogShift(0)
+		, invalidateToon(true)
 	{
-		for(int i=0;i<ARRAY_SIZE(u16ToonTable);i++)
-			u16ToonTable[i] = 0;
+		for(u32 i=0;i<ARRAY_SIZE(shininessTable);i++)
+			shininessTable[i] = 0;
+
+		for(u32 i=0;i<ARRAY_SIZE(rgbToonTable);i++)
+			rgbToonTable[i] = 0;
 	}
+
 	BOOL enableTexturing, enableAlphaTest, enableAlphaBlending, 
 		enableAntialiasing, enableEdgeMarking, enableClearImage, enableFog, enableFogAlphaOnly;
 
-	u32 fogShift;
-
 	static const u32 TOON = 0;
 	static const u32 HIGHLIGHT = 1;
 	u32 shading;
 
-	POLYLIST* polylist;
-	VERTLIST* vertlist;
-	int indexlist[POLYLIST_SIZE];
-
 	BOOL wbuffer, sortmode;
-
 	u8 alphaTestRef;
+	u32 activeFlushCommand;
+	u32 pendingFlushCommand;
 
 	u32 clearDepth;
 	u32 clearColor;
@@ -341,14 +342,48 @@ struct GFX3D
 	};
 	#include "PACKED_END.h"
 	u32 fogOffset;
+	u32 fogShift;
+
+	bool invalidateToon;
+	u32 rgbToonTable[32];
+	float shininessTable[128];
+};
+
+struct Viewer3d_State
+{
+	int frameNumber;
+	GFX3D_State state;
+	VERTLIST vertlist;
+	POLYLIST polylist;
+	INDEXLIST indexlist;
+};
+
+extern Viewer3d_State viewer3d_state;
+
+struct GFX3D
+{
+	GFX3D()
+		: polylist(0)
+		, vertlist(0)
+		, frameCtr(0)
+		, frameCtrRaw(0) {
+	}
+
+	//currently set values
+	GFX3D_State state;
+
+	//values used for the currently-rendered frame (committed with each flush)
+	GFX3D_State renderState;
+
+	POLYLIST* polylist;
+	VERTLIST* vertlist;
+	INDEXLIST indexlist;
 
 	//ticks every time flush() is called
 	int frameCtr;
 
 	//you can use this to track how many real frames passed, for comparing to frameCtr;
 	int frameCtrRaw;
-
-	u16 u16ToonTable[32];
 };
 extern GFX3D gfx3d;
 
@@ -376,7 +411,6 @@ int _hack_getMatrixStackLevel(int);
 void gfx3d_glFlush(u32 v);
 // end GE commands
 
-void gfx3d_glClearColor(u32 v);
 void gfx3d_glFogColor(u32 v);
 void gfx3d_glFogOffset (u32 v);
 void gfx3d_glClearDepth(u32 v);
