@@ -478,10 +478,16 @@ void  on_menu_palview_activate      (GtkMenuItem *menuitem, gpointer user_data)
 	GtkWidget * dlg = glade_xml_get_widget(xml_tools, "wtools_3_PalView");
 	gtk_widget_show(dlg);
 }
+#ifdef GTKGLEXT_AVAILABLE
 void  on_menu_tileview_activate     (GtkMenuItem *menuitem, gpointer user_data) {
 	GtkWidget * dlg = glade_xml_get_widget(xml_tools, "wtools_4_TileView");
 	gtk_widget_show(dlg);
 }
+#else
+void  on_menu_tileview_activate     (GtkMenuItem *menuitem, gpointer user_data) {
+	g_printerr("You need gtkglext for the tile viewer\n");
+}
+#endif
 void  on_menu_wtoolsXX_activate     (GtkMenuItem *menuitem, gpointer user_data) {
 	GtkWidget * w = (GtkWidget *) user_data;
 	gtk_widget_show(w);
