@@ -31,6 +31,7 @@
 #include <SDL.h>
 
 #include "types.h"
+#include "firmware.h"
 #include "armcpu.h"
 #include "NDSSystem.h"
 #include "sndsdl.h"
@@ -98,6 +99,7 @@ enum {
 
 gboolean EmuLoop(gpointer data);
 
+static void DoQuit();
 static void RecordMovieDialog();
 static void PlayMovieDialog();
 static void StopMovie();
@@ -109,6 +111,7 @@ void Pause();
 static void Printscreen();
 static void Reset();
 static void Edit_Controls();
+static void Edit_Joystick_Controls();
 static void MenuSave(GtkMenuItem *item, gpointer slot);
 static void MenuLoad(GtkMenuItem *item, gpointer slot);
 static void About();
@@ -181,16 +184,11 @@ static const char *ui_description =
 "      <menuitem action='micnoise'/>"
 #endif
 "      <menu action='FrameskipMenu'>"
+"        <menuitem action='frameskipA'/>"
 "        <menuitem action='frameskip0'/>"
 "        <menuitem action='frameskip1'/>"
 "        <menuitem action='frameskip2'/>"
 "        <menuitem action='frameskip3'/>"
-"        <menuitem action='frameskip4'/>"
-"        <menuitem action='frameskip5'/>"
-"        <menuitem action='frameskip6'/>"
-"        <menuitem action='frameskip7'/>"
-"        <menuitem action='frameskip8'/>"
-"        <menuitem action='frameskip9'/>"
 "      </menu>"
 "      <menu action='LayersMenu'>"
 "        <menuitem action='layermainbg0'/>"
@@ -231,6 +229,7 @@ static const char *ui_description =
 "      </menu>"
 "      <menuitem action='gap'/>"
 "      <menuitem action='editctrls'/>"
+"      <menuitem action='editjoyctrls'/>"
 "      <menu action='ViewMenu'>"
 "        <menuitem action='view_menu'/>"
 "        <menuitem action='view_toolbar'/>"
@@ -269,7 +268,7 @@ static const GtkActionEntry action_entries[] = {
       { "loadfirmware","gtk-open",        "Load Firm_ware file", "<Ctrl>l",  NULL, SelectFirmwareFile },
 #endif
       { "printscreen","gtk-media-record", "Take a _screenshot",    "<Ctrl>s",  NULL,   Printscreen },
-      { "quit",       "gtk-quit",         "_Quit",         "<Ctrl>q",  NULL,   gtk_main_quit },
+      { "quit",       "gtk-quit",         "_Quit",         "<Ctrl>q",  NULL,   DoQuit },
 
     { "EmulationMenu", NULL, "_Emulation" },
       { "run",        "gtk-media-play",   "_Run",          "<Ctrl>r",  NULL,   Launch },
@@ -283,7 +282,8 @@ static const GtkActionEntry action_entries[] = {
 
     { "ConfigMenu", NULL, "_Config" },
       { "ConfigSaveMenu", NULL, "_Saves" },
-      { "editctrls",  NULL,               "_Edit controls",NULL,       NULL,   Edit_Controls },
+      { "editctrls",  NULL,        "_Edit controls",NULL,    NULL,   Edit_Controls },
+      { "editjoyctrls",  NULL,     "Edit _Joystick controls",NULL,       NULL,   Edit_Joystick_Controls },
       { "RotationMenu", NULL, "_Rotation" },
         { "rotate_0",   "gtk-orientation-portrait",          "_0",  NULL, NULL, G_CALLBACK(SetRotation) },
         { "rotate_90",  "gtk-orientation-landscape",         "_90", NULL, NULL, G_CALLBACK(SetRotation) },
@@ -314,17 +314,20 @@ static const GtkRadioActionEntry interpolation_entries[] = {
     { "interp_bilinear", NULL, "_Bilinear", NULL, NULL, 1},
 };
 
+enum frameskip_enum {
+    FRAMESKIP_0 = 0,
+    FRAMESKIP_1 = 1,
+    FRAMESKIP_2 = 2,
+    FRAMESKIP_3 = 3,
+    FRAMESKIP_AUTO
+};
+
 static const GtkRadioActionEntry frameskip_entries[] = {
-    { "frameskip0", NULL, "_0", NULL, NULL, 0},
-    { "frameskip1", NULL, "_1", NULL, NULL, 1},
-    { "frameskip2", NULL, "_2", NULL, NULL, 2},
-    { "frameskip3", NULL, "_3", NULL, NULL, 3},
-    { "frameskip4", NULL, "_4", NULL, NULL, 4},
-    { "frameskip5", NULL, "_5", NULL, NULL, 5},
-    { "frameskip6", NULL, "_6", NULL, NULL, 6},
-    { "frameskip7", NULL, "_7", NULL, NULL, 7},
-    { "frameskip8", NULL, "_8", NULL, NULL, 8},
-    { "frameskip9", NULL, "_9", NULL, NULL, 9},
+    { "frameskipA", NULL, "_Auto", NULL, NULL, FRAMESKIP_AUTO},
+    { "frameskip0", NULL, "_0", NULL, NULL, FRAMESKIP_0},
+    { "frameskip1", NULL, "_1", NULL, NULL, FRAMESKIP_1},
+    { "frameskip2", NULL, "_2", NULL, NULL, FRAMESKIP_2},
+    { "frameskip3", NULL, "_3", NULL, NULL, FRAMESKIP_3},
 };
 
 static const GtkRadioActionEntry savet_entries[] = {
@@ -352,11 +355,29 @@ GPU3DInterface *core3DList[] = {
 #endif
 };
 
+static const u16 gtk_kb_cfg[NB_KEYS] = {
+    GDK_x,         // A
+    GDK_z,         // B
+    GDK_Shift_R,   // select
+    GDK_Return,    // start
+    GDK_Right,     // Right
+    GDK_Left,      // Left
+    GDK_Up,        // Up
+    GDK_Down,      // Down       
+    GDK_w,         // R
+    GDK_q,         // L
+    GDK_s,         // X
+    GDK_a,         // Y
+    GDK_p,         // DEBUG
+    GDK_o          // BOOST
+};
+
 GKeyFile *keyfile;
 
 struct modify_key_ctx {
     gint mk_key_chosen;
     GtkWidget *label;
+    u8 key_id;
 };
 
 static u16 Cur_Keypad = 0;
@@ -366,17 +387,11 @@ u16 Keypad_Temp[NB_KEYS];
 class configured_features : public CommandLine
 {
 public:
-  int disable_sound;
   int engine_3d;
-  int disable_limiter;
   int savetype;
 
-  int arm9_gdb_port;
-  int arm7_gdb_port;
-
   int firmware_language;
 
-  const char *cflash_disk_image_file;
 #ifdef HAVE_TIMEOUT
   int timeout;
 #endif
@@ -385,19 +400,10 @@ public:
 static void
 init_configured_features( struct configured_features *config)
 {
-  config->arm9_gdb_port = 0;
-  config->arm7_gdb_port = 0;
-
-  config->disable_sound = 0;
-
   config->engine_3d = 1;
 
-  config->disable_limiter = 0;
-
   config->savetype = 0;
 
-  config->cflash_disk_image_file = NULL;
-
 #ifdef HAVE_TIMEOUT
   config->timeout = 0;
 #endif
@@ -411,8 +417,6 @@ fill_configured_features( struct configured_features *config,
                           int argc, char ** argv)
 {
   GOptionEntry options[] = {
-    { "disable-sound", 0, 0, G_OPTION_ARG_NONE, &config->disable_sound, "Disables the sound emulation", NULL},
-    { "disable-limiter", 0, 0, G_OPTION_ARG_NONE, &config->disable_limiter, "Disables the 60fps limiter", NULL},
     { "3d-engine", 0, 0, G_OPTION_ARG_INT, &config->engine_3d, "Select 3d rendering engine. Available engines:\n"
         "\t\t\t\t  0 = 3d disabled\n"
         "\t\t\t\t  1 = internal rasterizer (default)\n"
@@ -444,24 +448,23 @@ fill_configured_features( struct configured_features *config,
   };
 
   config->loadCommonOptions();
-	g_option_context_add_main_entries (config->ctx, options, "options");
-	g_option_context_add_group (config->ctx, gtk_get_option_group (TRUE));
-	config->parse(argc,argv);
+  g_option_context_add_main_entries (config->ctx, options, "options");
+  g_option_context_add_group (config->ctx, gtk_get_option_group (TRUE));
+  config->parse(argc,argv);
 
-	if(!config->validate())
-		goto error;
+  if(!config->validate())
+    goto error;
 
-	if (config->savetype < 0 || config->savetype > 6) {
-		g_printerr("Accepted savetypes are from 0 to 6.\n");
-		return false;
-	}
+  if (config->savetype < 0 || config->savetype > 6) {
+    g_printerr("Accepted savetypes are from 0 to 6.\n");
+    return false;
+  }
 
   if (config->firmware_language < -1 || config->firmware_language > 5) {
     g_printerr("Firmware language must be set to a value from 0 to 5.\n");
     goto error;
   }
 
-
   if (config->engine_3d != 0 && config->engine_3d != 1
 #if defined(HAVE_LIBOSMESA)
            && config->engine_3d != 2
@@ -492,7 +495,7 @@ fill_configured_features( struct configured_features *config,
 error:
   config->errorHelp(argv[0]);
 
-    return 0;
+  return 0;
 }
 
 
@@ -521,6 +524,7 @@ joinThread_gdb( void *thread_handle) {
 /************************ GTK *******************************/
 
 uint Frameskip = 0;
+bool autoframeskip = false;
 GdkInterpType Interpolation = GDK_INTERP_BILINEAR;
 
 static GtkWidget *pWindow;
@@ -730,7 +734,7 @@ static void RecordMovieDialog()
     switch(gtk_dialog_run(GTK_DIALOG(pFileSelection))) {
     case GTK_RESPONSE_OK:
         sPath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pFileSelection));
-        FCEUI_SaveMovie(sPath,L"",0,NULL);
+		FCEUI_SaveMovie(sPath,L"",0,"", FCEUI_MovieGetRTCDefault());
         g_free(sPath);
         break;
     default:
@@ -933,8 +937,6 @@ static void OpenNdsDialog()
             gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "run"), TRUE);
         }
 
-        //Launch(NULL, pWindow);
-
         g_free(sPath);
         break;
     default:
@@ -1387,6 +1389,122 @@ static void Edit_Controls()
     case GTK_RESPONSE_OK:
         memcpy(&keyboard_cfg, &Keypad_Temp, sizeof(keyboard_cfg));
         desmume_config_update_keys(keyfile);
+        break;
+    case GTK_RESPONSE_CANCEL:
+    case GTK_RESPONSE_NONE:
+        break;
+    }
+    gtk_widget_destroy(ecDialog);
+
+}
+
+static void AcceptNewJoyKey(GtkWidget *w, GdkEventFocus *e, struct modify_key_ctx *ctx)
+{
+    gchar *YouPressed;
+
+    switch (ctx->key_id) {
+    case KEY_RIGHT:
+    case KEY_LEFT:
+        ctx->mk_key_chosen = get_joy_axis(KEY_LEFT, KEY_RIGHT);
+        break;
+    case KEY_UP:
+    case KEY_DOWN:
+        ctx->mk_key_chosen = get_joy_axis(KEY_UP, KEY_DOWN);
+        break;
+    default:
+        ctx->mk_key_chosen = get_joy_key(ctx->key_id);
+        break;
+    }
+
+    YouPressed = g_strdup_printf("You pressed : %d\nClick OK to keep this key.", ctx->mk_key_chosen);
+    gtk_label_set(GTK_LABEL(ctx->label), YouPressed);
+    g_free(YouPressed);
+}
+
+static void Modify_JoyKey(GtkWidget* widget, gpointer data)
+{
+    struct modify_key_ctx ctx;
+    GtkWidget *mkDialog;
+    gchar *Key_Label;
+    gchar *Title;
+    gint Key;
+
+    Key = GPOINTER_TO_INT(data);
+    /* Joypad keys start at 1 */
+    ctx.key_id = Key+1;
+    ctx.mk_key_chosen = 0;
+    Title = g_strdup_printf("Press \"%s\" key ...\n", key_names[Key]);
+    mkDialog = gtk_dialog_new_with_buttons(Title,
+        GTK_WINDOW(pWindow),
+        GTK_DIALOG_MODAL,
+        GTK_STOCK_OK,GTK_RESPONSE_OK,
+        GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
+        NULL);
+
+    ctx.label = gtk_label_new(Title);
+    g_free(Title);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(mkDialog)->vbox), ctx.label, TRUE, FALSE, 0);
+    gtk_widget_show_all(GTK_DIALOG(mkDialog)->vbox);
+
+    g_signal_connect(G_OBJECT(mkDialog), "focus_in_event", G_CALLBACK(AcceptNewJoyKey), &ctx);
+   
+    switch(gtk_dialog_run(GTK_DIALOG(mkDialog))) {
+    case GTK_RESPONSE_OK:
+        switch (ctx.key_id) {
+        case KEY_RIGHT:
+        case KEY_LEFT:
+          Keypad_Temp[KEY_RIGHT-1] = Keypad_Temp[KEY_LEFT-1] = ctx.mk_key_chosen;
+        case KEY_UP:
+        case KEY_DOWN:
+          Keypad_Temp[KEY_UP-1] = Keypad_Temp[KEY_DOWN-1] = ctx.mk_key_chosen;
+        default:
+          Keypad_Temp[Key] = ctx.mk_key_chosen;
+        }
+        Key_Label = g_strdup_printf("%s (%d)", key_names[Key], Keypad_Temp[Key]);
+        gtk_button_set_label(GTK_BUTTON(widget), Key_Label);
+        g_free(Key_Label);
+        break;
+    case GTK_RESPONSE_CANCEL:
+    case GTK_RESPONSE_NONE:
+        ctx.mk_key_chosen = 0;
+        break;
+    }
+
+    gtk_widget_destroy(mkDialog);
+
+}
+
+static void Edit_Joystick_Controls()
+{
+    GtkWidget *ecDialog;
+    GtkWidget *ecKey;
+    gchar *Key_Label;
+    int i;
+
+    memcpy(&Keypad_Temp, &joypad_cfg, sizeof(joypad_cfg));
+
+    ecDialog = gtk_dialog_new_with_buttons("Edit controls",
+                        GTK_WINDOW(pWindow),
+                        GTK_DIALOG_MODAL,
+                        GTK_STOCK_OK,GTK_RESPONSE_OK,
+                        GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
+                        NULL);
+
+    for(i = 0; i < NB_KEYS; i++) {
+        Key_Label = g_strdup_printf("%s (%d)", key_names[i], Keypad_Temp[i]);
+        ecKey = gtk_button_new_with_label(Key_Label);
+        g_free(Key_Label);
+        g_signal_connect(G_OBJECT(ecKey), "clicked", G_CALLBACK(Modify_JoyKey), GINT_TO_POINTER(i));
+        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ecDialog)->vbox), ecKey,TRUE, FALSE, 0);
+    }
+
+    gtk_widget_show_all(GTK_DIALOG(ecDialog)->vbox);
+
+    switch (gtk_dialog_run(GTK_DIALOG(ecDialog))) {
+    case GTK_RESPONSE_OK:
+        memcpy(&joypad_cfg, &Keypad_Temp, sizeof(keyboard_cfg));
+        desmume_config_update_joykeys(keyfile);
+        break;
     case GTK_RESPONSE_CANCEL:
     case GTK_RESPONSE_NONE:
         break;
@@ -1442,11 +1560,11 @@ static void Printscreen()
     u8 *rgb;
     static int seq = 0;
 
-    rgb = (u8 *) malloc(SCREENS_PIXEL_SIZE*3);
+    rgb = (u8 *) malloc(SCREENS_PIXEL_SIZE*SCREEN_BYTES_PER_PIXEL);
     if (!rgb)
         return;
 
-    gpu_screen_to_rgb(rgb, SCREENS_PIXEL_SIZE);
+    gpu_screen_to_rgb(rgb, SCREENS_PIXEL_SIZE*SCREEN_BYTES_PER_PIXEL);
     screenshot = gdk_pixbuf_new_from_data(rgb,
                           GDK_COLORSPACE_RGB,
                           FALSE,
@@ -1533,6 +1651,7 @@ static void Modify_Interpolation(GtkAction *action, GtkRadioAction *current)
 static void Modify_Frameskip(GtkAction *action, GtkRadioAction *current)
 {
     Frameskip = gtk_radio_action_get_current_value(current) ;
+    autoframeskip = Frameskip == FRAMESKIP_AUTO;
 }
 
 /////////////////////////////// TOOLS MANAGEMENT ///////////////////////////////
@@ -1574,9 +1693,28 @@ static inline void _updateDTools()
 
 /////////////////////////////// MAIN EMULATOR LOOP ///////////////////////////////
 
+class GtkDriver : public BaseDriver
+{
+public:
+    virtual void EMU_DebugIdleUpdate()
+    {
+        usleep(1000);
+        _updateDTools();
+        while (gtk_events_pending())
+            gtk_main_iteration();
+    }
+};
+
+static void DoQuit()
+{
+    emu_halt();
+    gtk_main_quit();
+}
+
+
 gboolean EmuLoop(gpointer data)
 {
-    static Uint32 fps_SecStart, fps_FrameCount;
+    static Uint32 fps_SecStart, next_fps_SecStart, fps_FrameCount, skipped_frames; 
     static int limiter_frame_counter;
     unsigned int i;
     gchar Title[20];
@@ -1587,22 +1725,38 @@ gboolean EmuLoop(gpointer data)
     }
 
     /* If desmume is currently running */
-    fps_FrameCount += Frameskip + 1;
     if (!fps_SecStart)
       fps_SecStart = SDL_GetTicks();
 
-    if (SDL_GetTicks() - fps_SecStart >= 1000) {
-        fps_SecStart = SDL_GetTicks();
+    /* don't count autoframeskip as real frameskip  */
+    if (Frameskip == FRAMESKIP_AUTO)
+      Frameskip = 0;
+
+    fps_FrameCount += Frameskip + 1;
+    next_fps_SecStart = SDL_GetTicks();
+    if ((next_fps_SecStart - fps_SecStart) >= 1000) {
+        fps_SecStart = next_fps_SecStart;
 
         snprintf(Title, sizeof(Title), "Desmume - %dfps", fps_FrameCount);
         gtk_window_set_title(GTK_WINDOW(pWindow), Title);
+
+        float emu_ratio = fps_FrameCount / 60.0;
+        if (autoframeskip) {
+            if (emu_ratio < 1 && (fps_FrameCount - skipped_frames) > skipped_frames)
+                Frameskip = MIN(Frameskip+1, FRAMESKIP_AUTO-1);
+            if (emu_ratio > 1 || skipped_frames >= fps_FrameCount-Frameskip)
+                Frameskip = Frameskip ? Frameskip-1 : 0;
+        }
+        LOG("auto: %d fps: %u skipped: %u emu_ratio: %f Frameskip: %u\n", autoframeskip, fps_FrameCount, skipped_frames, emu_ratio, Frameskip);
         fps_FrameCount = 0;
+        skipped_frames = 0;
     }
 
     desmume_cycle();    /* Emule ! */
     for (i = 0; i < Frameskip; i++) {
         NDS_SkipNextFrame();
         desmume_cycle();
+        skipped_frames++;
     }
 
     _updateDTools();
@@ -1746,7 +1900,7 @@ static void desmume_gtk_menu_tools (GtkActionGroup *ag)
 #ifdef HAVE_TIMEOUT
 static gboolean timeout_exit_cb(gpointer data)
 {
-    gtk_main_quit();
+    DoQuit();
     INFO("Quit after %d seconds timeout\n", GPOINTER_TO_INT(data));
 
     return FALSE;
@@ -1757,6 +1911,8 @@ static gboolean timeout_exit_cb(gpointer data)
 static int
 common_gtk_main( struct configured_features *my_config)
 {
+    driver = new GtkDriver();
+
     SDL_TimerID limiter_timer = NULL;
 
     GtkAccelGroup * accel_group;
@@ -1791,6 +1947,11 @@ common_gtk_main( struct configured_features *my_config)
     if (my_config->is_cflash_configured)
         addon_type = NDS_ADDON_CFLASH;
 
+    if(my_config->gbaslot_rom != "") {
+        addon_type = NDS_ADDON_GBAGAME;
+        strncpy(GBAgameName, my_config->gbaslot_rom.c_str(), MAX_PATH);
+    }
+
     switch (addon_type) {
     case NDS_ADDON_CFLASH:
     case NDS_ADDON_RUMBLEPAK:
@@ -1805,7 +1966,7 @@ common_gtk_main( struct configured_features *my_config)
 
 #ifdef GDB_STUB
     if ( my_config->arm9_gdb_port != 0) {
-         arm9_gdb_stub = createStub_gdb( my_config->arm9_gdb_port,
+		arm9_gdb_stub = createStub_gdb( my_config->arm9_gdb_port,
                                           &arm9_memio,
                                           &arm9_base_memory_iface);
 
@@ -1828,8 +1989,8 @@ common_gtk_main( struct configured_features *my_config)
     }
 #endif
 
-        /* FIXME: SDL_INIT_VIDEO is needed for joystick support to work!?
-           Perhaps it needs a "window" to catch events...? */
+    /* FIXME: SDL_INIT_VIDEO is needed for joystick support to work!?
+     * Perhaps it needs a "window" to catch events...? */
     if(SDL_Init(SDL_INIT_TIMER|SDL_INIT_VIDEO) == -1) {
         g_printerr("Error trying to initialize SDL: %s\n",
                     SDL_GetError());
@@ -1840,9 +2001,11 @@ common_gtk_main( struct configured_features *my_config)
                       my_config->disable_sound);
 
     /* Init the hud / osd stuff */
+#ifdef HAVE_LIBAGG
     Desmume_InitOnce();
     Hud.reset();
     aggDraw.hud->attach(GPU_screen, 256, 384, 512);
+#endif
 
     /*
      * Activate the GDB stubs
@@ -1868,7 +2031,7 @@ common_gtk_main( struct configured_features *my_config)
     if (dTools_running != NULL) 
       memset(dTools_running, FALSE, sizeof(BOOL) * dTools_list_size); 
 
-    keyfile = desmume_config_read_file();
+    keyfile = desmume_config_read_file(gtk_kb_cfg);
 
     /* Create the window */
     pWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -1876,7 +2039,7 @@ common_gtk_main( struct configured_features *my_config)
     gtk_window_set_resizable(GTK_WINDOW (pWindow), TRUE);
     gtk_window_set_icon(GTK_WINDOW (pWindow), gdk_pixbuf_new_from_xpm_data(DeSmuME_xpm));
 
-    g_signal_connect(G_OBJECT(pWindow), "destroy", G_CALLBACK(gtk_main_quit), NULL);
+    g_signal_connect(G_OBJECT(pWindow), "destroy", G_CALLBACK(DoQuit), NULL);
     g_signal_connect(G_OBJECT(pWindow), "key_press_event", G_CALLBACK(Key_Press), NULL);
     g_signal_connect(G_OBJECT(pWindow), "key_release_event", G_CALLBACK(Key_Release), NULL);
 
@@ -2023,10 +2186,6 @@ common_gtk_main( struct configured_features *my_config)
 #endif
 
     /* Main loop */
-
-//  gtk_idle_add(&EmuLoop, pWindow);
-//  g_idle_add(&EmuLoop, pWindow);
-
     gtk_main();
 
     desmume_free();
