@@ -37,10 +37,11 @@
 
 struct MatrixStack
 {
-	MatrixStack(int size);
+	MatrixStack(int size, int type);
 	float	*matrix;
 	s32		position;
 	s32		size;
+	u8		type;
 };
 
 void	MatrixInit				(float *matrix);
@@ -58,9 +59,8 @@ void	MatrixIdentity			(float *matrix);
 void	MatrixTranspose				(float *matrix);
 void	MatrixStackInit				(MatrixStack *stack);
 void	MatrixStackSetMaxSize		(MatrixStack *stack, int size);
-void	MatrixStackSetStackPosition (MatrixStack *stack, int pos);
 void	MatrixStackPushMatrix		(MatrixStack *stack, const float *ptr);
-float*	MatrixStackPopMatrix		(MatrixStack *stack, int size);
+void	MatrixStackPopMatrix		(float *mtxCurr, MatrixStack *stack, int size);
 float*	MatrixStackGetPos			(MatrixStack *stack, int pos);
 float*	MatrixStackGet				(MatrixStack *stack);
 void	MatrixStackLoadMatrix		(MatrixStack *stack, int pos, const float *ptr);
@@ -115,7 +115,7 @@ FORCEINLINE s32 s32floor(float f)
 
 //switched SSE2 functions
 //-------------
-#ifdef ENABLE_SSE
+#ifdef ENABLE_SSE2
 
 template<int NUM>
 FORCEINLINE void memset_u16_le(void* dst, u16 val)
@@ -139,7 +139,11 @@ FORCEINLINE void memset_u16_le(void* dst, u16 val)
 #else //no sse2
 
 template<int NUM>
+#ifdef _XBOX
+FORCEINLINE void memset_u16_le(void* dst, u16 val)
+#else
 static FORCEINLINE void memset_u16_le(void* dst, u16 val)
+#endif
 {
 	for(int i=0;i<NUM;i++)
 		T1WriteWord((u8*)dst,i<<1,val);
@@ -147,6 +151,9 @@ static FORCEINLINE void memset_u16_le(void* dst, u16 val)
 
 #endif
 
+// NOSSE version always used in gfx3d.cpp
+void _NOSSE_MatrixMultVec4x4 (const float *matrix, float *vecPtr);
+
 //---------------------------
 //switched SSE functions
 #ifdef ENABLE_SSE
@@ -312,7 +319,11 @@ FORCEINLINE void vector_fix2float(float* matrix, const float divisor)
 }
 
 template<int NUM>
+#ifdef _XBOX
+FORCEINLINE void memset_u8(void* dst, u8 val)
+#else
 static FORCEINLINE void memset_u8(void* dst, u8 val)
+#endif
 {
 	memset(dst,val,NUM);
 }
