@@ -29,8 +29,10 @@
 #include "NDSSystem.h"
 
 //temporary hack until we have better error reporting facilities
-#ifdef _MSC_VER
+#ifdef _WINDOWS
 #include <windows.h>
+#elif defined(_XBOX)
+#include <xtl.h>	// it`s really need?
 #endif
 
 #define FW_CMD_READ             0x3
@@ -58,26 +60,66 @@
 #define CARDFLASH_DEEP_POWDOWN		0xB9    /* Not used*/
 #define CARDFLASH_WAKEUP			0xAB    /* Not used*/
 
-//this should probably be 0xFF but we're using 0x00 until we find out otherwise
-//(no$ appears definitely to initialized to 0xFF)
-static const u8 kUninitializedSaveDataValue = 0x00; 
+//since r2203 this was 0x00.
+//but baby pals proves finally that it should be 0xFF:
+//the game reads its initial sound volumes from uninitialized data, and if it is 0, the game will be silent
+//if it is 0xFF then the game starts with its sound and music at max, as presumably it is supposed to.
+//so in r3303 I finally changed it (no$ appears definitely to initialized to 0xFF)
+static const u8 kUninitializedSaveDataValue = 0xFF; 
 
 static const char* kDesmumeSaveCookie = "|-DESMUME SAVE-|";
 
-static const u32 saveSizes[] = {512,8*1024,32*1024,64*1024,256*1024,512*1024,0xFFFFFFFF};
+static const u32 saveSizes[] = {512,			// 4k
+								8*1024,			// 64k
+								32*1024,		// 512k
+								64*1024,		// 1Mbit
+								256*1024,		// 2Mbit
+								512*1024,		// 4Mbit
+								1024*1024,		// 8Mbit
+								2048*1024,		// 16Mbit
+								4096*1024,		// 32Mbit
+								8192*1024,		// 64Mbit
+								16384*1024,		// 128Mbit
+								32768*1024,		// 256Mbit
+								65536*1024,		// 512Mbit
+								0xFFFFFFFF};
 static const u32 saveSizes_count = ARRAY_SIZE(saveSizes);
 
 //the lookup table from user save types to save parameters
-static const int save_types[7][2] = {
+static const int save_types[][2] = {
         {MC_TYPE_AUTODETECT,1},
         {MC_TYPE_EEPROM1,MC_SIZE_4KBITS},
         {MC_TYPE_EEPROM2,MC_SIZE_64KBITS},
         {MC_TYPE_EEPROM2,MC_SIZE_512KBITS},
         {MC_TYPE_FRAM,MC_SIZE_256KBITS},
         {MC_TYPE_FLASH,MC_SIZE_2MBITS},
-		{MC_TYPE_FLASH,MC_SIZE_4MBITS}
+		{MC_TYPE_FLASH,MC_SIZE_4MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_8MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_16MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_32MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_64MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_128MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_256MBITS},
+		{MC_TYPE_FLASH,MC_SIZE_512MBITS}
 };
 
+const char *save_names[] = {
+		"EEPROM 4kbit",
+		"EEPROM 64kbit",
+		"EEPROM 512kbit",
+		"FRAM 256kbit",
+		"FLASH 2Mbit",
+		"FLASH 4Mbit",
+		"FLASH 8Mbit",
+		"FLASH 16Mbit",
+		"FLASH 32Mbit",
+		"FLASH 64Mbit",
+		"FLASH 128Mbit",
+		"FLASH 256Mbit",
+		"FLASH 512Mbit"
+};
+
+
 void backup_setManualBackupType(int type)
 {
 	CommonSettings.manualBackupType = type;
@@ -117,6 +159,7 @@ u8 *mc_alloc(memory_chip_t *mc, u32 size)
 	buffer = new u8[size];
 	memset(buffer,0,size);
 
+	if (mc->data) delete [] mc->data;
 	mc->data = buffer;
 	if(!buffer) { return NULL; }
 	mc->size = size;
@@ -345,7 +388,7 @@ void BackupDevice::reset_command()
 			case 0:
 			case 1:
 				printf("Catastrophic error while autodetecting save type.\nIt will need to be specified manually\n");
-				#ifdef _MSC_VER
+				#ifdef _WINDOWS
 				MessageBox(0,"Catastrophic Error Code: Camel;\nyour save type has not been autodetected correctly;\nplease report to developers",0,0);
 				#endif
 				addr_size = 1; //choose 1 just to keep the busted savefile from growing too big
@@ -358,6 +401,7 @@ void BackupDevice::reset_command()
 				//another modern typical case..
 				//but unfortunately we select this case for spider-man 3, when what it meant to do was
 				//present the archaic 1+2 case
+				//it seems that over the hedge does this also.
 				addr_size = 2;
 				break;
 			case 4:
@@ -692,13 +736,9 @@ bool BackupDevice::load_no_gba(const char *fname)
 		if (fread(in_buf, 1, fsize, fsrc) == fsize)
 		{
 			out_buf = new u8 [8 * 1024 * 1024 / 8];
-			for (int jj = 0; jj < 8 * 1024 * 1024 / 8; jj++)
-			{
-				out_buf[jj] = 0xFF;
-			}
-
 			u32 size = 0;
-	
+
+			memset(out_buf, 0xFF, 8 * 1024 * 1024 / 8);
 			if (no_gba_unpackSAV(in_buf, fsize, out_buf, size) == 0)
 			{
 				//printf("New size %i byte(s)\n", size);
@@ -717,11 +757,13 @@ bool BackupDevice::load_no_gba(const char *fname)
 
 				if (in_buf) delete [] in_buf;
 				if (out_buf) delete [] out_buf;
+				fclose(fsrc);
 				return true;
 			}
 			if (out_buf) delete [] out_buf;
 		}
 		if (in_buf) delete [] in_buf;
+		fclose(fsrc);
 	}
 
 	return false;
@@ -915,13 +957,17 @@ void BackupDevice::raw_applyUserSettings(u32& size)
 {
 	//respect the user's choice of backup memory type
 	if(CommonSettings.manualBackupType == MC_TYPE_AUTODETECT)
+	{
 		addr_size = addr_size_for_old_save_size(size);
+		data.resize(size);
+	}
 	else
 	{
 		int savetype = save_types[CommonSettings.manualBackupType][0];
 		int savesize = save_types[CommonSettings.manualBackupType][1];
 		addr_size = addr_size_for_old_save_type(savetype);
 		if((u32)savesize<size) size = savesize;
+		data.resize(savesize);
 	}
 
 	state = RUNNING;
@@ -934,10 +980,9 @@ bool BackupDevice::load_raw(const char* filename)
 	fseek(inf, 0, SEEK_END);
 	u32 size = (u32)ftell(inf);
 	fseek(inf, 0, SEEK_SET);
-	
+
 	raw_applyUserSettings(size);
 
-	data.resize(size);
 	fread(&data[0],1,size,inf);
 	fclose(inf);
 
