@@ -1,28 +1,30 @@
-/* movie.cpp
- *
- * Copyright (C) 2006-2009 DeSmuME team
- *
- * This file is part of DeSmuME
- *
- * DeSmuME is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * DeSmuME is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with DeSmuME; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+/*  Copyright 2008-2010 DeSmuME team
+
+    This file is part of DeSmuME
+
+    DeSmuME is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    DeSmuME is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with DeSmuME; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#define WIN32_LEAN_AND_MEAN
 #include <assert.h>
 #include <limits.h>
+#include <ctype.h>
+#include <time.h>
 #include "utils/guid.h"
 #include "utils/xstring.h"
+#include "utils/mkgmtime.h"
 #include "movie.h"
 #include "NDSSystem.h"
 #include "readwrite.h"
@@ -165,12 +167,36 @@ void MovieRecord::dump(MovieData* md, EMUFILE* fp, int index)
 	fp->fputc('\n');
 }
 
+time_t FCEUI_MovieGetRTCDefault()
+{
+	time_t timer;
+
+	// compatible with old desmume
+	struct tm t;
+	t.tm_year = 109; // 2009
+	t.tm_mon  = 0;   // 1 (Jan)
+	t.tm_mday = 1;
+	t.tm_wday = 4;
+	t.tm_hour = 0; //12 AM
+	t.tm_min  = 0;
+	t.tm_sec  = 0;
+	timer = mkgmtime(&t);
+
+	// current time
+	//timer = time(NULL);
+	//struct tm *tp = localtime(&timer);
+	//timer = mkgmtime(tp);
+
+	return timer;
+}
+
 MovieData::MovieData()
 	: version(MOVIE_VERSION)
-	, emuVersion(DESMUME_VERSION_NUMERIC)
+	, emuVersion(EMU_DESMUME_VERSION_NUMERIC())
 	, romChecksum(0)
 	, rerecordCount(0)
 	, binaryFlag(false)
+	, rtcStart(FCEUI_MovieGetRTCDefault())
 {
 }
 
@@ -198,6 +224,29 @@ void MovieData::installValue(std::string& key, std::string& val)
 		romSerial = val;
 	else if(key == "guid")
 		guid = Desmume_Guid::fromString(val);
+	else if(key == "rtcStart") {
+		// sloppy format check and parse
+		const char *validFormatStr = "####-##-##T##:##:##Z";
+		bool validFormat = true;
+		for (int i = 0; validFormatStr[i] != '\0'; i++) {
+			if (validFormatStr[i] != val[i] && 
+					!(validFormatStr[i] == '#' && isdigit(val[i]))) {
+				validFormat = false;
+				break;
+			}
+		}
+		if (validFormat) {
+			struct tm t;
+			const char *s = val.data();
+			t.tm_year = atoi(&s[0]) - 1900;
+			t.tm_mon  = atoi(&s[5]) - 1;
+			t.tm_mday = atoi(&s[8]);
+			t.tm_hour = atoi(&s[11]);
+			t.tm_min  = atoi(&s[14]);
+			t.tm_sec  = atoi(&s[17]);
+			rtcStart = mkgmtime(&t);
+		}
+	}
 	else if(key == "comment")
 		comments.push_back(mbstowcs(val));
 	else if(key == "binary")
@@ -236,17 +285,52 @@ int MovieData::dump(EMUFILE* fp, bool binary)
 	fp->fprintf("romChecksum %s\n", u32ToHexString(gameInfo.crc).c_str());
 	fp->fprintf("romSerial %s\n", romSerial.c_str());
 	fp->fprintf("guid %s\n", guid.toString().c_str());
-	fp->fprintf("useExtBios %d\n", CommonSettings.UseExtBIOS);
+	fp->fprintf("useExtBios %d\n", CommonSettings.UseExtBIOS?1:0);
+	fp->fprintf("advancedTiming %d\n", CommonSettings.advanced_timing?1:0);
 
 	if(CommonSettings.UseExtBIOS)
-		fp->fprintf("swiFromBios %d\n", CommonSettings.SWIFromBIOS);
+		fp->fprintf("swiFromBios %d\n", CommonSettings.SWIFromBIOS?1:0);
+
+	fp->fprintf("useExtFirmware %d\n", CommonSettings.UseExtFirmware?1:0);
+
+	if(CommonSettings.UseExtFirmware) {
+		fp->fprintf("bootFromFirmware %d\n", CommonSettings.BootFromFirmware?1:0);
+	}
+	else {
+		char temp_str[27];
+		int i;
+
+		/* FIXME: harshly only use the lower byte of the UTF-16 character.
+		 * This would cause strange behaviour if the user could set UTF-16 but
+		 * they cannot yet.
+		 */
+		for (i = 0; i < CommonSettings.InternalFirmConf.nickname_len; i++) {
+			temp_str[i] = CommonSettings.InternalFirmConf.nickname[i];
+		}
+		temp_str[i] = '\0';
+		fp->fprintf("firmNickname %s\n", temp_str);
+		for (i = 0; i < CommonSettings.InternalFirmConf.message_len; i++) {
+			temp_str[i] = CommonSettings.InternalFirmConf.message[i];
+		}
+		temp_str[i] = '\0';
+		fp->fprintf("firmMessage %s\n", temp_str);
+		fp->fprintf("firmFavColour %d\n", CommonSettings.InternalFirmConf.fav_colour);
+		fp->fprintf("firmBirthMonth %d\n", CommonSettings.InternalFirmConf.birth_month);
+		fp->fprintf("firmBirthDay %d\n", CommonSettings.InternalFirmConf.birth_day);
+		fp->fprintf("firmLanguage %d\n", CommonSettings.InternalFirmConf.language);
+	}
+
+	char timestr[32];
+	struct tm *tm = gmtime(&rtcStart);
+	strftime(timestr, 32, "%Y-%m-%dT%H:%M:%SZ", tm);
+	fp->fprintf("rtcStart %s\n", timestr);
 
 	for(uint32 i=0;i<comments.size();i++)
 		fp->fprintf("comment %s\n", wcstombs(comments[i]).c_str());
 	
 	if(binary)
 		fp->fprintf("binary 1\n");
-		
+
 	if(savestate.size() != 0)
 		fp->fprintf("savestate %s\n", BytesToString(&savestate[0],savestate.size()).c_str());
 	if(sram.size() != 0)
@@ -458,6 +542,7 @@ const char* _CDECL_ FCEUI_LoadMovie(const char *fname, bool _read_only, bool tas
 			opened = true;
 //		}
 //		fs.close();
+		delete fp;
 //	}
 	if(!opened)
 	{
@@ -465,6 +550,7 @@ const char* _CDECL_ FCEUI_LoadMovie(const char *fname, bool _read_only, bool tas
 //		fstream fs (fname, std::ios_base::in);
 		loadedfm2 = LoadFM2(currMovieData, fp, INT_MAX, false);
 //		fs.close();
+		delete fp;
 	}
 
 	if(!loadedfm2)
@@ -492,7 +578,6 @@ const char* _CDECL_ FCEUI_LoadMovie(const char *fname, bool _read_only, bool tas
 	movie_readonly = _read_only;
 	movieMode = MOVIEMODE_PLAY;
 	currRerecordCount = currMovieData.rerecordCount;
-	InitMovieTime();
 	MMU_new.backupDevice.movie_mode();
 	if(currMovieData.sram.size() != 0)
 	{
@@ -561,7 +646,7 @@ bool MovieData::loadSramFrom(std::vector<u8>* buf)
 
 //begin recording a new movie
 //TODO - BUG - the record-from-another-savestate doesnt work.
-void _CDECL_ FCEUI_SaveMovie(const char *fname, std::wstring author, int flag, std::string sramfname)
+void _CDECL_ FCEUI_SaveMovie(const char *fname, std::wstring author, int flag, std::string sramfname, time_t rtcstart)
 {
 	//if(!FCEU_IsValidUI(FCEUI_RECORDMOVIE))
 	//	return;
@@ -582,7 +667,8 @@ void _CDECL_ FCEUI_SaveMovie(const char *fname, std::wstring author, int flag, s
 	currMovieData.romChecksum = gameInfo.crc;
 	currMovieData.romSerial = gameInfo.ROMserial;
 	currMovieData.romFilename = path.GetRomName();
-	
+	currMovieData.rtcStart = rtcstart;
+
 	NDS_Reset();
 
 	//todo ?
@@ -605,7 +691,6 @@ void _CDECL_ FCEUI_SaveMovie(const char *fname, std::wstring author, int flag, s
 	movieMode = MOVIEMODE_RECORD;
 	movie_readonly = false;
 	currRerecordCount = 0;
-	InitMovieTime();
 	MMU_new.backupDevice.movie_mode();
 
 	if(currMovieData.sram.size() != 0)
@@ -1093,7 +1178,7 @@ void FCEUI_MakeBackupMovie(bool dispMessage)
 	MovieData md = currMovieData;								//Get current movie data
 	EMUFILE* outf = new EMUFILE_FILE(backupFn.c_str(),"wb"); //FCEUD_UTF8_fstream(backupFn, "wb");	//open/create file
 	md.dump(outf,false);										//dump movie data
-//	delete outf;												//clean up, delete file object
+	delete outf;												//clean up, delete file object
 	
 	//TODO, decide if fstream successfully opened the file and print error message if it doesn't
 
