@@ -23,89 +23,65 @@
 #include "types.h"
 
 //well. just for the sake of consistency
-int write8le(u8 b, FILE *fp)
+int write8le(u8 b, EMUFILE*os)
 {
-	return((fwrite(&b,1,1,fp)<1)?0:1);
-}
-
-//well. just for the sake of consistency
-int write8le(u8 b, std::ostream *os)
-{
-	os->write((char*)&b,1);
+	os->fwrite((char*)&b,1);
 	return 1;
 }
 
 //well. just for the sake of consistency
-int read8le(u8 *Bufo, std::istream *is)
+int read8le(u8 *Bufo, EMUFILE*is)
 {
-	if(is->read((char*)Bufo,1).gcount() != 1)
+	if(is->_fread((char*)Bufo,1) != 1)
 		return 0;
 	return 1;
 }
 
 ///writes a little endian 16bit value to the specified file
-int write16le(u16 b, FILE *fp)
+int write16le(u16 b, EMUFILE *fp)
 {
 	u8 s[2];
-	s[0]=b;
-	s[1]=b>>8;
-	return((fwrite(s,1,2,fp)<2)?0:2);
-}
-
-
-///writes a little endian 16bit value to the specified file
-int write16le(u16 b, std::ostream *os)
-{
-	u8 s[2];
-	s[0]=b;
-	s[1]=b>>8;
-	os->write((char*)&s,2);
+	s[0]=(u8)b;
+	s[1]=(u8)(b>>8);
+	fp->fwrite(s,2);
 	return 2;
 }
 
-///writes a little endian 32bit value to the specified file
-int write32le(u32 b, FILE *fp)
-{
-	u8 s[4];
-	s[0]=b;
-	s[1]=b>>8;
-	s[2]=b>>16;
-	s[3]=b>>24;
-	return((fwrite(s,1,4,fp)<4)?0:4);
-}
 
-int write32le(u32 b, std::ostream* os)
+///writes a little endian 32bit value to the specified file
+int write32le(u32 b, EMUFILE *fp)
 {
 	u8 s[4];
-	s[0]=b;
-	s[1]=b>>8;
-	s[2]=b>>16;
-	s[3]=b>>24;
-	os->write((char*)&s,4);
+	s[0]=(u8)b;
+	s[1]=(u8)(b>>8);
+	s[2]=(u8)(b>>16);
+	s[3]=(u8)(b>>24);
+	fp->fwrite(s,4);
 	return 4;
 }
 
-int write64le(u64 b, std::ostream* os)
+void writebool(bool b, EMUFILE* os) { write32le(b?1:0,os); }
+
+int write64le(u64 b, EMUFILE* os)
 {
 	u8 s[8];
-	s[0]=b;
-	s[1]=b>>8;
-	s[2]=b>>16;
-	s[3]=b>>24;
-	s[4]=b>>32;
-	s[5]=b>>40;
-	s[6]=b>>48;
-	s[7]=b>>56;
-	os->write((char*)&s,8);
+	s[0]=(u8)b;
+	s[1]=(u8)(b>>8);
+	s[2]=(u8)(b>>16);
+	s[3]=(u8)(b>>24);
+	s[4]=(u8)(b>>32);
+	s[5]=(u8)(b>>40);
+	s[6]=(u8)(b>>48);
+	s[7]=(u8)(b>>56);
+	os->fwrite((char*)&s,8);
 	return 8;
 }
 
 
-///reads a little endian 32bit value from the specified file
-int read32le(u32 *Bufo, FILE *fp)
+int read32le(u32 *Bufo, EMUFILE *fp)
 {
 	u32 buf;
-	if(fread(&buf,1,4,fp)<4)
+	if(fp->_fread(&buf,4)<4)
 		return 0;
 #ifdef LOCAL_LE
 	*(u32*)Bufo=buf;
@@ -115,10 +91,10 @@ int read32le(u32 *Bufo, FILE *fp)
 	return 1;
 }
 
-int read16le(u16 *Bufo, std::istream *is)
+int read16le(u16 *Bufo, EMUFILE *is)
 {
 	u16 buf;
-	if(is->read((char*)&buf,2).gcount() != 2)
+	if(is->_fread((char*)&buf,2) != 2)
 		return 0;
 #ifdef LOCAL_LE
 	*Bufo=buf;
@@ -128,11 +104,10 @@ int read16le(u16 *Bufo, std::istream *is)
 	return 1;
 }
 
-///reads a little endian 64bit value from the specified file
-int read64le(u64 *Bufo, std::istream *is)
+int read64le(u64 *Bufo, EMUFILE *is)
 {
 	u64 buf;
-	if(is->read((char*)&buf,8).gcount() != 8)
+	if(is->_fread((char*)&buf,8) != 8)
 		return 0;
 #ifdef LOCAL_LE
 	*Bufo=buf;
@@ -142,7 +117,7 @@ int read64le(u64 *Bufo, std::istream *is)
 	return 1;
 }
 
-int read32le(u32 *Bufo, std::istream *is)
+static int read32le(u32 *Bufo, std::istream *is)
 {
 	u32 buf;
 	if(is->read((char*)&buf,4).gcount() != 4)
@@ -155,19 +130,27 @@ int read32le(u32 *Bufo, std::istream *is)
 	return 1;
 }
 
-int readbuffer(std::vector<u8> &vec, std::istream* is)
+int readbool(bool *b, EMUFILE* is)
+{
+	u32 temp;
+	int ret = read32le(&temp,is);
+	*b = temp!=0;
+	return ret;
+}
+
+int readbuffer(std::vector<u8> &vec, EMUFILE* is)
 {
 	u32 size;
 	if(read32le(&size,is) != 1) return 0;
 	vec.resize(size);
-	if(size>0) is->read((char*)&vec[0],size);
+	if(size>0) is->fread((char*)&vec[0],size);
 	return 1;
 }
 
-int writebuffer(std::vector<u8>& vec, std::ostream* os)
+int writebuffer(std::vector<u8>& vec, EMUFILE* os)
 {
 	u32 size = vec.size();
 	write32le(size,os);
-	if(size>0) os->write((char*)&vec[0],size);
+	if(size>0) os->fwrite((char*)&vec[0],size);
 	return 1;
 }
