@@ -1,9 +1,6 @@
 /*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
     Copyright 2008 CrazyMax
-	Copyright 2008-2009 DeSmuME team
+	Copyright 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -28,7 +25,6 @@
 #include "common.h"
 #include "debug.h"
 #include "armcpu.h"
-#include <time.h>
 #include <string.h>
 #include "saves.h"
 #ifdef WIN32
@@ -36,6 +32,9 @@
 #endif
 #include "movie.h"
 
+#include "utils/mkgmtime.h"
+
+
 typedef struct
 {
 	// RTC registers
@@ -92,58 +91,53 @@ SFORMAT SF_RTC[]={
 
 static const u8 kDefaultCmdBitsSize[8] = {8, 8, 56, 24, 0, 24, 8, 8};
 
-#define toBCD(x) ((x / 10) << 4) | (x % 10);
-
-struct movietime {
-
-	int sec;
-	int minute;
-	int hour;
-	int monthday;
-	int month;
-	int year;
-	int weekday;
-};
-
-struct movietime movie;
+static inline u8 toBCD(u8 x)
+{
+	return ((x / 10) << 4) | (x % 10);
+}
 
 bool moviemode=false;
 
-void InitMovieTime(void)
+struct tm rtcGetTime(void)
 {
-	movie.year=9;
-	movie.month=1;
-	movie.monthday=1;
-	movie.weekday=4;
-}	
+	struct tm *tm = NULL;
+	if(movieMode == MOVIEMODE_INACTIVE) {
+		time_t	timer;
+		time(&timer);
+		tm = localtime(&timer);
+	}
+	else {
+		//now, you might think it is silly to go through all these conniptions
+		//when we could just assume that there are 60fps and base the seconds on frameCounter/60
+		//but, we were imagining that one day we might need more precision
 
-static void MovieTime(void) {
+		const u32 arm9rate_unitsperframe = 560190<<1;
+		const u32 arm9rate_unitspersecond = (u32)(arm9rate_unitsperframe * 59.8261);
 
-	//now, you might think it is silly to go through all these conniptions
-	//when we could just assume that there are 60fps and base the seconds on frameCounter/60
-	//but, we were imagining that one day we might need more precision
+		u64 totalcycles = (u64)arm9rate_unitsperframe * currFrameCounter;
+		u64 totalseconds=totalcycles/arm9rate_unitspersecond;
 
-	const u32 arm9rate_unitsperframe = 560190<<1;
-	const u32 arm9rate_unitspersecond = (u32)(arm9rate_unitsperframe * 59.8261);
-	const u64 noon = (u64)arm9rate_unitspersecond * 60 * 60 * 12;
-	           
-	u64 frameCycles = (u64)arm9rate_unitsperframe * currFrameCounter;
-	u64 totalcycles = frameCycles + noon;
-	u64 totalseconds=totalcycles/arm9rate_unitspersecond;
+		time_t timer;
+		struct tm t;
 
-	movie.sec=(int)(totalseconds % 60);
-	movie.minute=(int)(totalseconds/60);
-	movie.hour=movie.minute/60;
-
-	//convert to sane numbers
-	movie.minute=movie.minute % 60;
-	movie.hour=movie.hour % 24;
+		// store start time into structure tm
+		timer = (time_t) currMovieData.rtcStart;
+		memcpy(&t, gmtime(&timer), sizeof(struct tm));
+		// advance it according to the frame counter
+		t.tm_sec += totalseconds;
+		// then, normalize it
+		timer = mkgmtime(&t);
+		tm = gmtime(&timer);
+	}
+	tm->tm_year = 100 + (tm->tm_year % 100); // 20XX
+	return *tm;
 }
 
 static void rtcRecv()
 {
 	//INFO("RTC Read command 0x%02X\n", (rtc.cmd >> 1));
-	memset(rtc.data, 0, sizeof(rtc.data));
+
+	memset(&rtc.data[0], 0, sizeof(rtc.data));
 	switch (rtc.cmd >> 1)
 	{
 		case 0:				// status register 1
@@ -159,63 +153,27 @@ static void rtcRecv()
 		case 2:				// date & time
 			{
 				//INFO("RTC: read date & time\n");
-				time_t	tm;
-				time(&tm);
-				struct tm *tm_local= localtime(&tm);
-				tm_local->tm_year %= 100;
-				tm_local->tm_mon++;
-
-				if(movieMode != MOVIEMODE_INACTIVE) {
-
-					MovieTime();
-					
-					rtc.data[0]=toBCD(movie.year);
-					rtc.data[1]=toBCD(movie.month);
-					rtc.data[2]=toBCD(movie.monthday);
-					rtc.data[3]=(movie.weekday + 6) & 7;
-					if (!(rtc.regStatus1 & 0x02)) movie.hour %= 12;
-					rtc.data[4] = ((movie.hour < 12) ? 0x00 : 0x40) | toBCD(movie.hour);		
-					rtc.data[5]=toBCD(movie.minute);
-					rtc.data[6]=toBCD(movie.sec);
-					break;
-				}
-				else {
-
-					rtc.data[0] = toBCD(tm_local->tm_year);
-					rtc.data[1] = toBCD(tm_local->tm_mon);
-					rtc.data[2] = toBCD(tm_local->tm_mday);
-					rtc.data[3] =  (tm_local->tm_wday + 6) & 7;
-					if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
-					rtc.data[4] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
-					rtc.data[5] =  toBCD(tm_local->tm_min);
-					rtc.data[6] =  toBCD(tm_local->tm_sec);
-					break;
-				}
+				struct tm tm = rtcGetTime();
+				rtc.data[0] = toBCD(tm.tm_year % 100);
+				rtc.data[1] = toBCD(tm.tm_mon + 1);
+				rtc.data[2] = toBCD(tm.tm_mday);
+				rtc.data[3] = (tm.tm_wday + 7) & 7;
+				if (rtc.data[3] == 7) rtc.data[3] = 6;
+				if (!(rtc.regStatus1 & 0x02)) tm.tm_hour %= 12;
+				rtc.data[4] = ((tm.tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm.tm_hour);
+				rtc.data[5] =  toBCD(tm.tm_min);
+				rtc.data[6] =  toBCD(tm.tm_sec);
+				break;
 			}
 		case 3:				// time
 			{
 				//INFO("RTC: read time\n");
-				time_t	tm;
-				time(&tm);
-				struct tm *tm_local= localtime(&tm);
-
-				if(movieMode != MOVIEMODE_INACTIVE) {
-
-					MovieTime();
-
-					if (!(rtc.regStatus1 & 0x02)) movie.hour %= 12;
-					rtc.data[0] = ((movie.hour < 12) ? 0x00 : 0x40) | toBCD(movie.hour);
-					rtc.data[1] =  toBCD(movie.minute);
-					rtc.data[2] =  toBCD(movie.sec);
-				}
-				else {
-
-					if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
-					rtc.data[0] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
-					rtc.data[1] =  toBCD(tm_local->tm_min);
-					rtc.data[2] =  toBCD(tm_local->tm_sec);
-					break;
-				}
+				struct tm tm = rtcGetTime();
+				if (!(rtc.regStatus1 & 0x02)) tm.tm_hour %= 12;
+				rtc.data[0] = ((tm.tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm.tm_hour);
+				rtc.data[1] =  toBCD(tm.tm_min);
+				rtc.data[2] =  toBCD(tm.tm_sec);
+				break;
 			}
 		case 4:				// freq/alarm 1
 			/*if (cmdBitsSize[0x04] == 8)
@@ -240,7 +198,7 @@ static void rtcRecv()
 
 static void rtcSend()
 {
-	//INFO("RTC write 0x%02X\n", (rtc.cmd >> 1));
+	//INFO("RTC write command 0x%02X\n", (rtc.cmd >> 1));
 	switch (rtc.cmd >> 1)
 	{
 		case 0:				// status register 1
@@ -281,8 +239,9 @@ static void rtcSend()
 
 void rtcInit() 
 {
-	memset(&rtc, 0, sizeof(_RTC));
-	memcpy(rtc.cmdBitsSize,kDefaultCmdBitsSize,8);
+	memset(&rtc, 0, sizeof(rtc));
+	memcpy(&rtc.cmdBitsSize[0],kDefaultCmdBitsSize,8);
+
 	rtc.regStatus1 |= 0x02;
 }
 
@@ -380,13 +339,13 @@ void rtcWrite(u16 val)
 			if( (rtc._prevSCK) && (!rtc._SCK) )
 			{
 				rtc._REG = val;
-				if(rtc.data[rtc.bitsCount >> 3] >> (rtc.bitsCount & 0x07) & 0x01) 
+				if((rtc.data[(rtc.bitsCount >> 3)] >> (rtc.bitsCount & 0x07)) & 0x01) 
 					rtc._REG |= 0x01;
 				else
 					rtc._REG &= ~0x01;
 
 				rtc.bitsCount++;
-				if (rtc.bitsCount == rtc.cmdBitsSize[rtc.cmd >> 1])
+				if (rtc.bitsCount == rtc.cmdBitsSize[rtc.cmd >> 1] || (!(val & 0x04)))
 					rtc.cmdStat = 0;
 			}
 		break;
