@@ -1,9 +1,6 @@
 /*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
     Copyright 2008 CrazyMax
-	Copyright 2008-2009 DeSmuME team
+	Copyright 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -25,11 +22,12 @@
 #ifndef _RTC_H_
 #define _RTC_H_
 #include <stdlib.h>
+#include <time.h>
 #include "types.h"
 
+struct tm rtcGetTime(void);
+
 extern	void rtcInit();
 extern	u16 rtcRead();
 extern	void rtcWrite(u16 val);
-
-void InitMovieTime(void);
 #endif
