@@ -1,7 +1,7 @@
 /*  Copyright (C) 2006 Normmatt
     Copyright (C) 2006 Theo Berkau
     Copyright (C) 2007 Pascal Giard
-	Copyright (C) 2008-2009 DeSmuME team
+	Copyright (C) 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -46,7 +46,7 @@
 
 #include "path.h"
 
-#ifdef _MSC_VER
+#ifdef _WINDOWS
 #include "windows/main.h"
 #endif
 
@@ -64,11 +64,6 @@ static const char* magic = "DeSmuME SState\0";
 //a savestate chunk loader can set this if it wants to permit a silent failure (for compatibility)
 static bool SAV_silent_fail_flag;
 
-#ifndef MAX_PATH
-#define MAX_PATH 256
-#endif
-
-
 SFORMAT SF_ARM7[]={
 	{ "7INS", 4, 1, &NDS_ARM7.instruction },
 	{ "7INA", 4, 1, &NDS_ARM7.instruct_adr },
@@ -173,6 +168,19 @@ SFORMAT SF_NDS[]={
 	{ "_TPY", 2, 1, &nds.touchY},
 	{ "_TPB", 4, 1, &nds.isTouch},
 	{ "_DBG", 4, 1, &nds.debugConsole},
+	{ "_ENS", 4, 1, &nds.ensataEmulation},
+	{ "_ENH", 4, 1, &nds.ensataHandshake},
+	{ "_ENI", 4, 1, &nds.ensataIpcSyncCounter},
+	{ "_SLP", 4, 1, &nds.sleeping},
+	{ "_CEJ", 4, 1, &nds.cardEjected},
+	{ "_P00", 1, 1, &nds.power1.lcd},
+	{ "_P01", 1, 1, &nds.power1.gpuMain},
+	{ "_P02", 1, 1, &nds.power1.gfx3d_render},
+	{ "_P03", 1, 1, &nds.power1.gfx3d_geometry},
+	{ "_P04", 1, 1, &nds.power1.gpuSub},
+	{ "_P05", 1, 1, &nds.power1.dispswap},
+	{ "_P06", 1, 1, &nds.power2.speakers},
+	{ "_P07", 1, 1, &nds.power2.wifi},
 	{ 0 }
 };
 
@@ -205,12 +213,10 @@ SFORMAT SF_MMU[]={
 	{ "MDV1", 4, 1,       &MMU.divRunning},
 	{ "MDV2", 8, 1,       &MMU.divResult},
 	{ "MDV3", 8, 1,       &MMU.divMod},
-	{ "MDV4", 4, 1,       &MMU.divCnt},
 	{ "MDV5", 8, 1,       &MMU.divCycles},
 
 	{ "MSQ1", 4, 1,       &MMU.sqrtRunning},
 	{ "MSQ2", 4, 1,       &MMU.sqrtResult},
-	{ "MSQ3", 4, 1,       &MMU.sqrtCnt},
 	{ "MSQ4", 8, 1,       &MMU.sqrtCycles},
 	
 	//begin memory chips
@@ -242,6 +248,10 @@ SFORMAT SF_MMU[]={
 	{ "FDHD", 4, 1,       &disp_fifo.head},
 	{ "FDTL", 4, 1,       &disp_fifo.tail},
 	{ "FDBF", 4, 0x6000,  disp_fifo.buf},
+
+	{ "PMCN", 1, 1,			&MMU.powerMan_CntReg},
+	{ "PMCW", 4, 1,			&MMU.powerMan_CntRegWritten},
+	{ "PMCR", 1, 5,			&MMU.powerMan_Reg},
 	
 	{ 0 }
 };
@@ -254,7 +264,7 @@ SFORMAT SF_MOVIE[]={
 
 static void mmu_savestate(EMUFILE* os)
 {
-	u32 version = 3;
+	u32 version = 4;
 	write32le(version,os);
 	
 	//version 2:
@@ -272,6 +282,10 @@ static void mmu_savestate(EMUFILE* os)
 	MMU_timing.arm7dataFetch.savestate(os, version);
 	MMU_timing.arm9codeCache.savestate(os, version);
 	MMU_timing.arm9dataCache.savestate(os, version);
+
+	//version 4:
+	MMU_new.sqrt.savestate(os);
+	MMU_new.div.savestate(os);
 }
 
 SFORMAT SF_WIFI[]={
@@ -281,8 +295,8 @@ SFORMAT SF_WIFI[]={
 	{ "W020", 2, 1, &wifiMac.rfStatus},
 	{ "W030", 2, 1, &wifiMac.rfPins},
 
-	{ "W040", 2, 1, &wifiMac.IE.val},
-	{ "W050", 2, 1, &wifiMac.IF.val},
+	{ "W040", 2, 1, &wifiMac.IE},
+	{ "W050", 2, 1, &wifiMac.IF},
 
 	{ "W060", 2, 1, &wifiMac.macMode},
 	{ "W070", 2, 1, &wifiMac.wepMode},
@@ -339,14 +353,10 @@ SFORMAT SF_WIFI[]={
 	{ "W370", 4, 1, &wifiMac.rfIOData.val},
 	{ "W380", 2, 1, &wifiMac.bbIOCnt.val},
 
-	{ "W390", 1, 1, &wifiMac.bbDataToWrite},
-
-	{ "W400", 2, 0x1000, &wifiMac.circularBuffer[0]},
+	{ "W400", 2, 0x1000, &wifiMac.RAM[0]},
 	{ "W410", 2, 1, &wifiMac.RXRangeBegin},
 	{ "W420", 2, 1, &wifiMac.RXRangeEnd},
-	{ "W430", 2, 1, &wifiMac.RXHWWriteCursor},
-	{ "W440", 2, 1, &wifiMac.RXHWWriteCursorReg},
-	{ "W450", 2, 1, &wifiMac.RXHWWriteCursorLatched},
+	{ "W430", 2, 1, &wifiMac.RXWriteCursor},
 	{ "W460", 2, 1, &wifiMac.RXReadCursor},
 	{ "W470", 2, 1, &wifiMac.RXUnits},
 	{ "W480", 2, 1, &wifiMac.RXBufCount},
@@ -361,14 +371,14 @@ SFORMAT SF_WIFI[]={
 	{ "W560", 4, 1, &wifiMac.curPacketPos[0]},
 	{ "W570", 4, 1, &wifiMac.curPacketSending[0]},
 
-	{ "W580", 2, 0x800, &wifiMac.ioMem[0]},
+	{ "W580", 2, 0x800, &wifiMac.IOPorts[0]},
 	{ "W590", 2, 1, &wifiMac.randomSeed},
 
-	{ "WX00", 8, 1, &wifiMac.SoftAP.usecCounter},
-	{ "WX10", 1, 4096, &wifiMac.SoftAP.curPacket[0]},
-	{ "WX20", 4, 1, &wifiMac.SoftAP.curPacketSize},
-	{ "WX30", 4, 1, &wifiMac.SoftAP.curPacketPos},
-	{ "WX40", 4, 1, &wifiMac.SoftAP.curPacketSending},
+	{ "WX00", 8, 1, &SoftAP.usecCounter},
+	{ "WX10", 1, 4096, &SoftAP.curPacket[0]},
+	{ "WX20", 4, 1, &SoftAP.curPacketSize},
+	{ "WX30", 4, 1, &SoftAP.curPacketPos},
+	{ "WX40", 4, 1, &SoftAP.curPacketSending},
 
 	{ 0 }
 };
@@ -414,13 +424,11 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 		if(is->fail()) return false;
 	}
 
-	if(version < 2)
-		return true;
+	if(version < 2) return true;
 
 	bool ok = MMU_new.backupDevice.load_state(is);
 
-	if(version < 3)
-		return true;
+	if(version < 3) return true;
 
 	ok &= MMU_new.gxstat.loadstate(is);
 	
@@ -435,6 +443,11 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 	ok &= MMU_timing.arm9codeCache.loadstate(is, version);
 	ok &= MMU_timing.arm9dataCache.loadstate(is, version);
 
+	if(version < 4) return true;
+
+	ok &= MMU_new.sqrt.loadstate(is,version);
+	ok &= MMU_new.div.loadstate(is,version);
+
 	return ok;
 }
 
@@ -569,25 +582,24 @@ void clear_savestates()
     savestates[i].exists = FALSE;
 }
 
-/* Scan for existing savestates and update struct */
+// Scan for existing savestates and update struct
 void scan_savestates()
 {
   struct stat sbuf;
   char filename[MAX_PATH+1];
-  u8 i;
 
   clear_savestates();
 
-  for( i = 1; i <= NB_STATES; i++ )
+  for(int i = 0; i < NB_STATES; i++ )
     {
-    path.getpathnoext(path.STATES, filename);
+     path.getpathnoext(path.STATES, filename);
 	  
 	  if (strlen(filename) + strlen(".dst") + strlen("-2147483648") /* = biggest string for i */ >MAX_PATH) return ;
       sprintf(filename+strlen(filename), ".ds%d", i);
       if( stat(filename,&sbuf) == -1 ) continue;
-      savestates[i-1].exists = TRUE;
-      strncpy(savestates[i-1].date, format_time(sbuf.st_mtime),40);
-	  savestates[i-1].date[40-1] = '\0';
+      savestates[i].exists = TRUE;
+      strncpy(savestates[i].date, format_time(sbuf.st_mtime),40);
+	  savestates[i].date[40-1] = '\0';
     }
 
   return ;
@@ -852,13 +864,14 @@ static int SubWrite(EMUFILE* os, const SFORMAT *sf)
 			keyset.insert(sf->desc);
 			#endif
 
+
 		#ifdef LOCAL_LE
 			// no need to ever loop one at a time if not flipping byte order
 			os->fwrite((char *)sf->v,size*count);
 		#else
-			if(sz == 1) {
+			if(size == 1) {
 				//special case: write a huge byte array
-				os->fwrite((char *)sf->v,1,count);
+				os->fwrite((char *)sf->v,count);
 			} else {
 				for(int i=0;i<count;i++) {
 					FlipByteOrder((u8*)sf->v + i*size, size);
@@ -978,7 +991,7 @@ bool savestate_save(EMUFILE* outstream, int compressionLevel)
 	outstream->fseek(0,SEEK_SET);
 	outstream->fwrite(magic,16);
 	write32le(SAVESTATE_VERSION,outstream);
-	write32le(DESMUME_VERSION_NUMERIC,outstream); //desmume version
+	write32le(EMU_DESMUME_VERSION_NUMERIC(),outstream); //desmume version
 	write32le(len,outstream); //uncompressed length
 	write32le(comprlen,outstream); //compressed length (-1 if it is not compressed)
 
@@ -1168,7 +1181,7 @@ bool savestate_load(EMUFILE* is)
 	if(!x && !SAV_silent_fail_flag)
 	{
 		printf("Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked");
-#ifdef _MSC_VER
+#ifdef _WINDOWS
 		//HACK! we really need a better way to handle this kind of feedback
 		MessageBox(0,"Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked",0,0);
 #endif
