@@ -26,7 +26,7 @@ public:
 	static const int MAXSIZE = 8;
 
 	MemSpan() 
-		: numItems(0)
+		: numItems(0), size(0)
 	{}
 
 	int numItems;
@@ -179,10 +179,6 @@ static void DebugDumpTexture(TexCacheItem* item)
 }
 #endif
 
-//notes on the cache:
-//I am really unhappy with the ref counting. this needs to be automatic.
-//We could do something better than a linear search through cache items, but it may not be worth it.
-//Also we may need to rescan more often (every time a sample loops)
 class TexCache
 {
 public:
@@ -193,7 +189,11 @@ public:
 	TTexCacheItemMultimap index;
 
 	//this ought to be enough for anyone
-	static const u32 kMaxCacheSize = 64*1024*1024; 
+	//static const u32 kMaxCacheSize = 64*1024*1024; 
+	//changed by zeromus on 15-dec. I couldnt find any games that were getting anywhere NEAR 64
+	static const u32 kMaxCacheSize = 16*1024*1024; 
+	//metal slug burns through sprites so fast, it can test it pretty quickly though
+
 	//this is not really precise, it is off by a constant factor
 	u32 cache_size;
 
@@ -307,7 +307,7 @@ public:
 			if(mspal.size != 0 && memcmp(curr->dump.palette,pal,mspal.size)) goto REJECT;
 
 			//when the texture data doesn't match
-			if(ms.memcmp(curr->dump.texture,sizeof(curr->dump.texture))) goto REJECT;
+			if(ms.memcmp(&curr->dump.texture[0],curr->dump.textureSize)) goto REJECT;
 
 			//if the texture is 4x4 then the index data must match
 			if(textureMode == TEXMODE_4X4)
@@ -342,7 +342,6 @@ public:
 		newitem->sizeY=sizeY;
 		newitem->invSizeX=1.0f/((float)(sizeX));
 		newitem->invSizeY=1.0f/((float)(sizeY));
-		newitem->dump.textureSize = ms.dump(newitem->dump.texture,sizeof(newitem->dump.texture));
 		newitem->decode_len = sizeX*sizeY*4;
 		newitem->mode = textureMode;
 		newitem->decoded = new u8[newitem->decode_len];
@@ -356,13 +355,15 @@ public:
 		{
 			memcpy(newitem->dump.palette, pal, palSize*2);
 		}
-		//dump 4x4 index data for cache keying
-		newitem->dump.indexSize = 0;
+
+		//dump texture and 4x4 index data for cache keying
+		const int texsize = newitem->dump.textureSize = ms.size;
+		const int indexsize = newitem->dump.indexSize = msIndex.size;
+		newitem->dump.texture = new u8[texsize+indexsize];
+		ms.dump(&newitem->dump.texture[0],newitem->dump.maxTextureSize); //dump texture
 		if(textureMode == TEXMODE_4X4)
-		{
-			newitem->dump.indexSize = min(msIndex.size,(int)sizeof(newitem->dump.texture) - newitem->dump.textureSize);
-			msIndex.dump(newitem->dump.texture+newitem->dump.textureSize,newitem->dump.indexSize);
-		}
+			msIndex.dump(newitem->dump.texture+newitem->dump.textureSize,newitem->dump.indexSize); //dump 4x4
+
 
 		//============================================================================ 
 		//Texture conversion
@@ -465,7 +466,7 @@ public:
 				//this check isnt necessary since the addressing is tied to the texture data which will also run out:
 				//if(msIndex.numItems != 1) PROGINFO("Your 4x4 texture index has overrun its slot.\n");
 
-	#define PAL4X4(offset) ( *(u16*)( MMU.texInfo.texPalSlot[((paletteAddress + (offset)*2)>>14)] + ((paletteAddress + (offset)*2)&0x3FFF) ) )
+	#define PAL4X4(offset) ( *(u16*)( MMU.texInfo.texPalSlot[((paletteAddress + (offset)*2)>>14)&0x7] + ((paletteAddress + (offset)*2)&0x3FFF) ) )
 
 				u16* slot1;
 				u32* map = (u32*)ms.items[0].ptr;
@@ -637,6 +638,9 @@ public:
 
 	void evict(u32 target = kMaxCacheSize)
 	{
+		//debug print
+		//printf("%d %d/%d\n",index.size(),cache_size/1024,target/1024);
+
 		//dont do anything unless we're over the target
 		if(cache_size<target) return;
 
